namespace fa {
enum fa_common_icons : uint16_t {
	fa_0 = 0x30,
	fa_1 = 0x31,
	fa_2 = 0x32,
	fa_3 = 0x33,
	fa_4 = 0x34,
	fa_5 = 0x35,
	fa_6 = 0x36,
	fa_7 = 0x37,
	fa_8 = 0x38,
	fa_9 = 0x39,
	fa_a = 0x41,
	fa_address_book = 0xf2b9,
	fa_contact_book = 0xf2b9,
	fa_address_card = 0xf2bb,
	fa_contact_card = 0xf2bb,
	fa_vcard = 0xf2bb,
	fa_align_center = 0xf037,
	fa_align_justify = 0xf039,
	fa_align_left = 0xf036,
	fa_align_right = 0xf038,
	fa_anchor = 0xf13d,
	fa_anchor_circle_check = 0xe4aa,
	fa_anchor_circle_exclamation = 0xe4ab,
	fa_anchor_circle_xmark = 0xe4ac,
	fa_anchor_lock = 0xe4ad,
	fa_angle_down = 0xf107,
	fa_angle_left = 0xf104,
	fa_angle_right = 0xf105,
	fa_angle_up = 0xf106,
	fa_angles_down = 0xf103,
	fa_angle_double_down = 0xf103,
	fa_angles_left = 0xf100,
	fa_angle_double_left = 0xf100,
	fa_angles_right = 0xf101,
	fa_angle_double_right = 0xf101,
	fa_angles_up = 0xf102,
	fa_angle_double_up = 0xf102,
	fa_ankh = 0xf644,
	fa_apple_whole = 0xf5d1,
	fa_apple_alt = 0xf5d1,
	fa_archway = 0xf557,
	fa_arrow_down = 0xf063,
	fa_arrow_down_1_9 = 0xf162,
	fa_sort_numeric_asc = 0xf162,
	fa_sort_numeric_down = 0xf162,
	fa_arrow_down_9_1 = 0xf886,
	fa_sort_numeric_desc = 0xf886,
	fa_sort_numeric_down_alt = 0xf886,
	fa_arrow_down_a_z = 0xf15d,
	fa_sort_alpha_asc = 0xf15d,
	fa_sort_alpha_down = 0xf15d,
	fa_arrow_down_long = 0xf175,
	fa_long_arrow_down = 0xf175,
	fa_arrow_down_short_wide = 0xf884,
	fa_sort_amount_desc = 0xf884,
	fa_sort_amount_down_alt = 0xf884,
	fa_arrow_down_up_across_line = 0xe4af,
	fa_arrow_down_up_lock = 0xe4b0,
	fa_arrow_down_wide_short = 0xf160,
	fa_sort_amount_asc = 0xf160,
	fa_sort_amount_down = 0xf160,
	fa_arrow_down_z_a = 0xf881,
	fa_sort_alpha_desc = 0xf881,
	fa_sort_alpha_down_alt = 0xf881,
	fa_arrow_left = 0xf060,
	fa_arrow_left_long = 0xf177,
	fa_long_arrow_left = 0xf177,
	fa_arrow_pointer = 0xf245,
	fa_mouse_pointer = 0xf245,
	fa_arrow_right = 0xf061,
	fa_arrow_right_arrow_left = 0xf0ec,
	fa_exchange = 0xf0ec,
	fa_arrow_right_from_bracket = 0xf08b,
	fa_sign_out = 0xf08b,
	fa_arrow_right_long = 0xf178,
	fa_long_arrow_right = 0xf178,
	fa_arrow_right_to_bracket = 0xf090,
	fa_sign_in = 0xf090,
	fa_arrow_right_to_city = 0xe4b3,
	fa_arrow_rotate_left = 0xf0e2,
	fa_arrow_left_rotate = 0xf0e2,
	fa_arrow_rotate_back = 0xf0e2,
	fa_arrow_rotate_backward = 0xf0e2,
	fa_undo = 0xf0e2,
	fa_arrow_rotate_right = 0xf01e,
	fa_arrow_right_rotate = 0xf01e,
	fa_arrow_rotate_forward = 0xf01e,
	fa_redo = 0xf01e,
	fa_arrow_trend_down = 0xe097,
	fa_arrow_trend_up = 0xe098,
	fa_arrow_turn_down = 0xf149,
	fa_level_down = 0xf149,
	fa_arrow_turn_up = 0xf148,
	fa_level_up = 0xf148,
	fa_arrow_up = 0xf062,
	fa_arrow_up_1_9 = 0xf163,
	fa_sort_numeric_up = 0xf163,
	fa_arrow_up_9_1 = 0xf887,
	fa_sort_numeric_up_alt = 0xf887,
	fa_arrow_up_a_z = 0xf15e,
	fa_sort_alpha_up = 0xf15e,
	fa_arrow_up_from_bracket = 0xe09a,
	fa_arrow_up_from_ground_water = 0xe4b5,
	fa_arrow_up_from_water_pump = 0xe4b6,
	fa_arrow_up_long = 0xf176,
	fa_long_arrow_up = 0xf176,
	fa_arrow_up_right_dots = 0xe4b7,
	fa_arrow_up_right_from_square = 0xf08e,
	fa_external_link = 0xf08e,
	fa_arrow_up_short_wide = 0xf885,
	fa_sort_amount_up_alt = 0xf885,
	fa_arrow_up_wide_short = 0xf161,
	fa_sort_amount_up = 0xf161,
	fa_arrow_up_z_a = 0xf882,
	fa_sort_alpha_up_alt = 0xf882,
	fa_arrows_down_to_line = 0xe4b8,
	fa_arrows_down_to_people = 0xe4b9,
	fa_arrows_left_right = 0xf07e,
	fa_arrows_h = 0xf07e,
	fa_arrows_left_right_to_line = 0xe4ba,
	fa_arrows_rotate = 0xf021,
	fa_refresh = 0xf021,
	fa_sync = 0xf021,
	fa_arrows_spin = 0xe4bb,
	fa_arrows_split_up_and_left = 0xe4bc,
	fa_arrows_to_circle = 0xe4bd,
	fa_arrows_to_dot = 0xe4be,
	fa_arrows_to_eye = 0xe4bf,
	fa_arrows_turn_right = 0xe4c0,
	fa_arrows_turn_to_dots = 0xe4c1,
	fa_arrows_up_down = 0xf07d,
	fa_arrows_v = 0xf07d,
	fa_arrows_up_down_left_right = 0xf047,
	fa_arrows = 0xf047,
	fa_arrows_up_to_line = 0xe4c2,
	fa_asterisk = 0x2a,
	fa_at = 0x40,
	fa_atom = 0xf5d2,
	fa_audio_description = 0xf29e,
	fa_austral_sign = 0xe0a9,
	fa_award = 0xf559,
	fa_b = 0x42,
	fa_baby = 0xf77c,
	fa_baby_carriage = 0xf77d,
	fa_carriage_baby = 0xf77d,
	fa_backward = 0xf04a,
	fa_backward_fast = 0xf049,
	fa_fast_backward = 0xf049,
	fa_backward_step = 0xf048,
	fa_step_backward = 0xf048,
	fa_bacon = 0xf7e5,
	fa_bacteria = 0xe059,
	fa_bacterium = 0xe05a,
	fa_bag_shopping = 0xf290,
	fa_shopping_bag = 0xf290,
	fa_bahai = 0xf666,
	fa_haykal = 0xf666,
	fa_baht_sign = 0xe0ac,
	fa_ban = 0xf05e,
	fa_cancel = 0xf05e,
	fa_ban_smoking = 0xf54d,
	fa_smoking_ban = 0xf54d,
	fa_bandage = 0xf462,
	fa_band_aid = 0xf462,
	fa_bangladeshi_taka_sign = 0xe2e6,
	fa_barcode = 0xf02a,
	fa_bars = 0xf0c9,
	fa_navicon = 0xf0c9,
	fa_bars_progress = 0xf828,
	fa_tasks_alt = 0xf828,
	fa_bars_staggered = 0xf550,
	fa_reorder = 0xf550,
	fa_stream = 0xf550,
	fa_baseball = 0xf433,
	fa_baseball_ball = 0xf433,
	fa_baseball_bat_ball = 0xf432,
	fa_basket_shopping = 0xf291,
	fa_shopping_basket = 0xf291,
	fa_basketball = 0xf434,
	fa_basketball_ball = 0xf434,
	fa_bath = 0xf2cd,
	fa_bathtub = 0xf2cd,
	fa_battery_empty = 0xf244,
	fa_battery_0 = 0xf244,
	fa_battery_full = 0xf240,
	fa_battery = 0xf240,
	fa_battery_5 = 0xf240,
	fa_battery_half = 0xf242,
	fa_battery_3 = 0xf242,
	fa_battery_quarter = 0xf243,
	fa_battery_2 = 0xf243,
	fa_battery_three_quarters = 0xf241,
	fa_battery_4 = 0xf241,
	fa_bed = 0xf236,
	fa_bed_pulse = 0xf487,
	fa_procedures = 0xf487,
	fa_beer_mug_empty = 0xf0fc,
	fa_beer = 0xf0fc,
	fa_bell = 0xf0f3,
	fa_bell_concierge = 0xf562,
	fa_concierge_bell = 0xf562,
	fa_bell_slash = 0xf1f6,
	fa_bezier_curve = 0xf55b,
	fa_bicycle = 0xf206,
	fa_binoculars = 0xf1e5,
	fa_biohazard = 0xf780,
	fa_bitcoin_sign = 0xe0b4,
	fa_blender = 0xf517,
	fa_blender_phone = 0xf6b6,
	fa_blog = 0xf781,
	fa_bold = 0xf032,
	fa_bolt = 0xf0e7,
	fa_zap = 0xf0e7,
	fa_bolt_lightning = 0xe0b7,
	fa_bomb = 0xf1e2,
	fa_bone = 0xf5d7,
	fa_bong = 0xf55c,
	fa_book = 0xf02d,
	fa_book_atlas = 0xf558,
	fa_atlas = 0xf558,
	fa_book_bible = 0xf647,
	fa_bible = 0xf647,
	fa_book_bookmark = 0xe0bb,
	fa_book_journal_whills = 0xf66a,
	fa_journal_whills = 0xf66a,
	fa_book_medical = 0xf7e6,
	fa_book_open = 0xf518,
	fa_book_open_reader = 0xf5da,
	fa_book_reader = 0xf5da,
	fa_book_quran = 0xf687,
	fa_quran = 0xf687,
	fa_book_skull = 0xf6b7,
	fa_book_dead = 0xf6b7,
	fa_book_tanakh = 0xf827,
	fa_tanakh = 0xf827,
	fa_bookmark = 0xf02e,
	fa_border_all = 0xf84c,
	fa_border_none = 0xf850,
	fa_border_top_left = 0xf853,
	fa_border_style = 0xf853,
	fa_bore_hole = 0xe4c3,
	fa_bottle_droplet = 0xe4c4,
	fa_bottle_water = 0xe4c5,
	fa_bowl_food = 0xe4c6,
	fa_bowl_rice = 0xe2eb,
	fa_bowling_ball = 0xf436,
	fa_box = 0xf466,
	fa_box_archive = 0xf187,
	fa_archive = 0xf187,
	fa_box_open = 0xf49e,
	fa_box_tissue = 0xe05b,
	fa_boxes_packing = 0xe4c7,
	fa_boxes_stacked = 0xf468,
	fa_boxes = 0xf468,
	fa_boxes_alt = 0xf468,
	fa_braille = 0xf2a1,
	fa_brain = 0xf5dc,
	fa_brazilian_real_sign = 0xe46c,
	fa_bread_slice = 0xf7ec,
	fa_bridge = 0xe4c8,
	fa_bridge_circle_check = 0xe4c9,
	fa_bridge_circle_exclamation = 0xe4ca,
	fa_bridge_circle_xmark = 0xe4cb,
	fa_bridge_lock = 0xe4cc,
	fa_bridge_water = 0xe4ce,
	fa_briefcase = 0xf0b1,
	fa_briefcase_medical = 0xf469,
	fa_broom = 0xf51a,
	fa_broom_ball = 0xf458,
	fa_quidditch = 0xf458,
	fa_quidditch_broom_ball = 0xf458,
	fa_brush = 0xf55d,
	fa_bucket = 0xe4cf,
	fa_bug = 0xf188,
	fa_bug_slash = 0xe490,
	fa_bugs = 0xe4d0,
	fa_building = 0xf1ad,
	fa_building_circle_arrow_right = 0xe4d1,
	fa_building_circle_check = 0xe4d2,
	fa_building_circle_exclamation = 0xe4d3,
	fa_building_circle_xmark = 0xe4d4,
	fa_building_columns = 0xf19c,
	fa_bank = 0xf19c,
	fa_institution = 0xf19c,
	fa_museum = 0xf19c,
	fa_university = 0xf19c,
	fa_building_flag = 0xe4d5,
	fa_building_lock = 0xe4d6,
	fa_building_ngo = 0xe4d7,
	fa_building_shield = 0xe4d8,
	fa_building_un = 0xe4d9,
	fa_building_user = 0xe4da,
	fa_building_wheat = 0xe4db,
	fa_bullhorn = 0xf0a1,
	fa_bullseye = 0xf140,
	fa_burger = 0xf805,
	fa_hamburger = 0xf805,
	fa_burst = 0xe4dc,
	fa_bus = 0xf207,
	fa_bus_simple = 0xf55e,
	fa_bus_alt = 0xf55e,
	fa_business_time = 0xf64a,
	fa_briefcase_clock = 0xf64a,
	fa_c = 0x43,
	fa_cable_car = 0xf7da,
	fa_tram = 0xf7da,
	fa_cake_candles = 0xf1fd,
	fa_birthday_cake = 0xf1fd,
	fa_cake = 0xf1fd,
	fa_calculator = 0xf1ec,
	fa_calendar = 0xf133,
	fa_calendar_check = 0xf274,
	fa_calendar_day = 0xf783,
	fa_calendar_days = 0xf073,
	fa_calendar_alt = 0xf073,
	fa_calendar_minus = 0xf272,
	fa_calendar_plus = 0xf271,
	fa_calendar_week = 0xf784,
	fa_calendar_xmark = 0xf273,
	fa_calendar_times = 0xf273,
	fa_camera = 0xf030,
	fa_camera_alt = 0xf030,
	fa_camera_retro = 0xf083,
	fa_camera_rotate = 0xe0d8,
	fa_campground = 0xf6bb,
	fa_candy_cane = 0xf786,
	fa_cannabis = 0xf55f,
	fa_capsules = 0xf46b,
	fa_car = 0xf1b9,
	fa_automobile = 0xf1b9,
	fa_car_battery = 0xf5df,
	fa_battery_car = 0xf5df,
	fa_car_burst = 0xf5e1,
	fa_car_crash = 0xf5e1,
	fa_car_on = 0xe4dd,
	fa_car_rear = 0xf5de,
	fa_car_alt = 0xf5de,
	fa_car_side = 0xf5e4,
	fa_car_tunnel = 0xe4de,
	fa_caravan = 0xf8ff,
	fa_caret_down = 0xf0d7,
	fa_caret_left = 0xf0d9,
	fa_caret_right = 0xf0da,
	fa_caret_up = 0xf0d8,
	fa_carrot = 0xf787,
	fa_cart_arrow_down = 0xf218,
	fa_cart_flatbed = 0xf474,
	fa_dolly_flatbed = 0xf474,
	fa_cart_flatbed_suitcase = 0xf59d,
	fa_luggage_cart = 0xf59d,
	fa_cart_plus = 0xf217,
	fa_cart_shopping = 0xf07a,
	fa_shopping_cart = 0xf07a,
	fa_cash_register = 0xf788,
	fa_cat = 0xf6be,
	fa_cedi_sign = 0xe0df,
	fa_cent_sign = 0xe3f5,
	fa_certificate = 0xf0a3,
	fa_chair = 0xf6c0,
	fa_chalkboard = 0xf51b,
	fa_blackboard = 0xf51b,
	fa_chalkboard_user = 0xf51c,
	fa_chalkboard_teacher = 0xf51c,
	fa_champagne_glasses = 0xf79f,
	fa_glass_cheers = 0xf79f,
	fa_charging_station = 0xf5e7,
	fa_chart_area = 0xf1fe,
	fa_area_chart = 0xf1fe,
	fa_chart_bar = 0xf080,
	fa_bar_chart = 0xf080,
	fa_chart_column = 0xe0e3,
	fa_chart_gantt = 0xe0e4,
	fa_chart_line = 0xf201,
	fa_line_chart = 0xf201,
	fa_chart_pie = 0xf200,
	fa_pie_chart = 0xf200,
	fa_chart_simple = 0xe473,
	fa_check = 0xf00c,
	fa_check_double = 0xf560,
	fa_check_to_slot = 0xf772,
	fa_vote_yea = 0xf772,
	fa_cheese = 0xf7ef,
	fa_chess = 0xf439,
	fa_chess_bishop = 0xf43a,
	fa_chess_board = 0xf43c,
	fa_chess_king = 0xf43f,
	fa_chess_knight = 0xf441,
	fa_chess_pawn = 0xf443,
	fa_chess_queen = 0xf445,
	fa_chess_rook = 0xf447,
	fa_chevron_down = 0xf078,
	fa_chevron_left = 0xf053,
	fa_chevron_right = 0xf054,
	fa_chevron_up = 0xf077,
	fa_child = 0xf1ae,
	fa_child_combatant = 0xe4e0,
	fa_child_rifle = 0xe4e0,
	fa_child_dress = 0xe59c,
	fa_child_reaching = 0xe59d,
	fa_children = 0xe4e1,
	fa_church = 0xf51d,
	fa_circle = 0xf111,
	fa_circle_arrow_down = 0xf0ab,
	fa_arrow_circle_down = 0xf0ab,
	fa_circle_arrow_left = 0xf0a8,
	fa_arrow_circle_left = 0xf0a8,
	fa_circle_arrow_right = 0xf0a9,
	fa_arrow_circle_right = 0xf0a9,
	fa_circle_arrow_up = 0xf0aa,
	fa_arrow_circle_up = 0xf0aa,
	fa_circle_check = 0xf058,
	fa_check_circle = 0xf058,
	fa_circle_chevron_down = 0xf13a,
	fa_chevron_circle_down = 0xf13a,
	fa_circle_chevron_left = 0xf137,
	fa_chevron_circle_left = 0xf137,
	fa_circle_chevron_right = 0xf138,
	fa_chevron_circle_right = 0xf138,
	fa_circle_chevron_up = 0xf139,
	fa_chevron_circle_up = 0xf139,
	fa_circle_dollar_to_slot = 0xf4b9,
	fa_donate = 0xf4b9,
	fa_circle_dot = 0xf192,
	fa_dot_circle = 0xf192,
	fa_circle_down = 0xf358,
	fa_arrow_alt_circle_down = 0xf358,
	fa_circle_exclamation = 0xf06a,
	fa_exclamation_circle = 0xf06a,
	fa_circle_h = 0xf47e,
	fa_hospital_symbol = 0xf47e,
	fa_circle_half_stroke = 0xf042,
	fa_adjust = 0xf042,
	fa_circle_info = 0xf05a,
	fa_info_circle = 0xf05a,
	fa_circle_left = 0xf359,
	fa_arrow_alt_circle_left = 0xf359,
	fa_circle_minus = 0xf056,
	fa_minus_circle = 0xf056,
	fa_circle_nodes = 0xe4e2,
	fa_circle_notch = 0xf1ce,
	fa_circle_pause = 0xf28b,
	fa_pause_circle = 0xf28b,
	fa_circle_play = 0xf144,
	fa_play_circle = 0xf144,
	fa_circle_plus = 0xf055,
	fa_plus_circle = 0xf055,
	fa_circle_question = 0xf059,
	fa_question_circle = 0xf059,
	fa_circle_radiation = 0xf7ba,
	fa_radiation_alt = 0xf7ba,
	fa_circle_right = 0xf35a,
	fa_arrow_alt_circle_right = 0xf35a,
	fa_circle_stop = 0xf28d,
	fa_stop_circle = 0xf28d,
	fa_circle_up = 0xf35b,
	fa_arrow_alt_circle_up = 0xf35b,
	fa_circle_user = 0xf2bd,
	fa_user_circle = 0xf2bd,
	fa_circle_xmark = 0xf057,
	fa_times_circle = 0xf057,
	fa_xmark_circle = 0xf057,
	fa_city = 0xf64f,
	fa_clapperboard = 0xe131,
	fa_clipboard = 0xf328,
	fa_clipboard_check = 0xf46c,
	fa_clipboard_list = 0xf46d,
	fa_clipboard_question = 0xe4e3,
	fa_clipboard_user = 0xf7f3,
	fa_clock = 0xf017,
	fa_clock_four = 0xf017,
	fa_clock_rotate_left = 0xf1da,
	fa_history = 0xf1da,
	fa_clone = 0xf24d,
	fa_closed_captioning = 0xf20a,
	fa_cloud = 0xf0c2,
	fa_cloud_arrow_down = 0xf0ed,
	fa_cloud_download = 0xf0ed,
	fa_cloud_download_alt = 0xf0ed,
	fa_cloud_arrow_up = 0xf0ee,
	fa_cloud_upload = 0xf0ee,
	fa_cloud_upload_alt = 0xf0ee,
	fa_cloud_bolt = 0xf76c,
	fa_thunderstorm = 0xf76c,
	fa_cloud_meatball = 0xf73b,
	fa_cloud_moon = 0xf6c3,
	fa_cloud_moon_rain = 0xf73c,
	fa_cloud_rain = 0xf73d,
	fa_cloud_showers_heavy = 0xf740,
	fa_cloud_showers_water = 0xe4e4,
	fa_cloud_sun = 0xf6c4,
	fa_cloud_sun_rain = 0xf743,
	fa_clover = 0xe139,
	fa_code = 0xf121,
	fa_code_branch = 0xf126,
	fa_code_commit = 0xf386,
	fa_code_compare = 0xe13a,
	fa_code_fork = 0xe13b,
	fa_code_merge = 0xf387,
	fa_code_pull_request = 0xe13c,
	fa_coins = 0xf51e,
	fa_colon_sign = 0xe140,
	fa_comment = 0xf075,
	fa_comment_dollar = 0xf651,
	fa_comment_dots = 0xf4ad,
	fa_commenting = 0xf4ad,
	fa_comment_medical = 0xf7f5,
	fa_comment_slash = 0xf4b3,
	fa_comment_sms = 0xf7cd,
	fa_sms = 0xf7cd,
	fa_comments = 0xf086,
	fa_comments_dollar = 0xf653,
	fa_compact_disc = 0xf51f,
	fa_compass = 0xf14e,
	fa_compass_drafting = 0xf568,
	fa_drafting_compass = 0xf568,
	fa_compress = 0xf066,
	fa_computer = 0xe4e5,
	fa_computer_mouse = 0xf8cc,
	fa_mouse = 0xf8cc,
	fa_cookie = 0xf563,
	fa_cookie_bite = 0xf564,
	fa_copy = 0xf0c5,
	fa_copyright = 0xf1f9,
	fa_couch = 0xf4b8,
	fa_cow = 0xf6c8,
	fa_credit_card = 0xf09d,
	fa_credit_card_alt = 0xf09d,
	fa_crop = 0xf125,
	fa_crop_simple = 0xf565,
	fa_crop_alt = 0xf565,
	fa_cross = 0xf654,
	fa_crosshairs = 0xf05b,
	fa_crow = 0xf520,
	fa_crown = 0xf521,
	fa_crutch = 0xf7f7,
	fa_cruzeiro_sign = 0xe152,
	fa_cube = 0xf1b2,
	fa_cubes = 0xf1b3,
	fa_cubes_stacked = 0xe4e6,
	fa_d = 0x44,
	fa_database = 0xf1c0,
	fa_delete_left = 0xf55a,
	fa_backspace = 0xf55a,
	fa_democrat = 0xf747,
	fa_desktop = 0xf390,
	fa_desktop_alt = 0xf390,
	fa_dharmachakra = 0xf655,
	fa_diagram_next = 0xe476,
	fa_diagram_predecessor = 0xe477,
	fa_diagram_project = 0xf542,
	fa_project_diagram = 0xf542,
	fa_diagram_successor = 0xe47a,
	fa_diamond = 0xf219,
	fa_diamond_turn_right = 0xf5eb,
	fa_directions = 0xf5eb,
	fa_dice = 0xf522,
	fa_dice_d20 = 0xf6cf,
	fa_dice_d6 = 0xf6d1,
	fa_dice_five = 0xf523,
	fa_dice_four = 0xf524,
	fa_dice_one = 0xf525,
	fa_dice_six = 0xf526,
	fa_dice_three = 0xf527,
	fa_dice_two = 0xf528,
	fa_disease = 0xf7fa,
	fa_display = 0xe163,
	fa_divide = 0xf529,
	fa_dna = 0xf471,
	fa_dog = 0xf6d3,
	fa_dollar_sign = 0x24,
	fa_dollar = 0x24,
	fa_usd = 0x24,
	fa_dolly = 0xf472,
	fa_dolly_box = 0xf472,
	fa_dong_sign = 0xe169,
	fa_door_closed = 0xf52a,
	fa_door_open = 0xf52b,
	fa_dove = 0xf4ba,
	fa_down_left_and_up_right_to_center = 0xf422,
	fa_compress_alt = 0xf422,
	fa_down_long = 0xf309,
	fa_long_arrow_alt_down = 0xf309,
	fa_download = 0xf019,
	fa_dragon = 0xf6d5,
	fa_draw_polygon = 0xf5ee,
	fa_droplet = 0xf043,
	fa_tint = 0xf043,
	fa_droplet_slash = 0xf5c7,
	fa_tint_slash = 0xf5c7,
	fa_drum = 0xf569,
	fa_drum_steelpan = 0xf56a,
	fa_drumstick_bite = 0xf6d7,
	fa_dumbbell = 0xf44b,
	fa_dumpster = 0xf793,
	fa_dumpster_fire = 0xf794,
	fa_dungeon = 0xf6d9,
	fa_e = 0x45,
	fa_ear_deaf = 0xf2a4,
	fa_deaf = 0xf2a4,
	fa_deafness = 0xf2a4,
	fa_hard_of_hearing = 0xf2a4,
	fa_ear_listen = 0xf2a2,
	fa_assistive_listening_systems = 0xf2a2,
	fa_earth_africa = 0xf57c,
	fa_globe_africa = 0xf57c,
	fa_earth_americas = 0xf57d,
	fa_earth = 0xf57d,
	fa_earth_america = 0xf57d,
	fa_globe_americas = 0xf57d,
	fa_earth_asia = 0xf57e,
	fa_globe_asia = 0xf57e,
	fa_earth_europe = 0xf7a2,
	fa_globe_europe = 0xf7a2,
	fa_earth_oceania = 0xe47b,
	fa_globe_oceania = 0xe47b,
	fa_egg = 0xf7fb,
	fa_eject = 0xf052,
	fa_elevator = 0xe16d,
	fa_ellipsis = 0xf141,
	fa_ellipsis_h = 0xf141,
	fa_ellipsis_vertical = 0xf142,
	fa_ellipsis_v = 0xf142,
	fa_envelope = 0xf0e0,
	fa_envelope_circle_check = 0xe4e8,
	fa_envelope_open = 0xf2b6,
	fa_envelope_open_text = 0xf658,
	fa_envelopes_bulk = 0xf674,
	fa_mail_bulk = 0xf674,
	fa_equals = 0x3d,
	fa_eraser = 0xf12d,
	fa_ethernet = 0xf796,
	fa_euro_sign = 0xf153,
	fa_eur = 0xf153,
	fa_euro = 0xf153,
	fa_exclamation = 0x21,
	fa_expand = 0xf065,
	fa_explosion = 0xe4e9,
	fa_eye = 0xf06e,
	fa_eye_dropper = 0xf1fb,
	fa_eye_dropper_empty = 0xf1fb,
	fa_eyedropper = 0xf1fb,
	fa_eye_low_vision = 0xf2a8,
	fa_low_vision = 0xf2a8,
	fa_eye_slash = 0xf070,
	fa_f = 0x46,
	fa_face_angry = 0xf556,
	fa_angry = 0xf556,
	fa_face_dizzy = 0xf567,
	fa_dizzy = 0xf567,
	fa_face_flushed = 0xf579,
	fa_flushed = 0xf579,
	fa_face_frown = 0xf119,
	fa_frown = 0xf119,
	fa_face_frown_open = 0xf57a,
	fa_frown_open = 0xf57a,
	fa_face_grimace = 0xf57f,
	fa_grimace = 0xf57f,
	fa_face_grin = 0xf580,
	fa_grin = 0xf580,
	fa_face_grin_beam = 0xf582,
	fa_grin_beam = 0xf582,
	fa_face_grin_beam_sweat = 0xf583,
	fa_grin_beam_sweat = 0xf583,
	fa_face_grin_hearts = 0xf584,
	fa_grin_hearts = 0xf584,
	fa_face_grin_squint = 0xf585,
	fa_grin_squint = 0xf585,
	fa_face_grin_squint_tears = 0xf586,
	fa_grin_squint_tears = 0xf586,
	fa_face_grin_stars = 0xf587,
	fa_grin_stars = 0xf587,
	fa_face_grin_tears = 0xf588,
	fa_grin_tears = 0xf588,
	fa_face_grin_tongue = 0xf589,
	fa_grin_tongue = 0xf589,
	fa_face_grin_tongue_squint = 0xf58a,
	fa_grin_tongue_squint = 0xf58a,
	fa_face_grin_tongue_wink = 0xf58b,
	fa_grin_tongue_wink = 0xf58b,
	fa_face_grin_wide = 0xf581,
	fa_grin_alt = 0xf581,
	fa_face_grin_wink = 0xf58c,
	fa_grin_wink = 0xf58c,
	fa_face_kiss = 0xf596,
	fa_kiss = 0xf596,
	fa_face_kiss_beam = 0xf597,
	fa_kiss_beam = 0xf597,
	fa_face_kiss_wink_heart = 0xf598,
	fa_kiss_wink_heart = 0xf598,
	fa_face_laugh = 0xf599,
	fa_laugh = 0xf599,
	fa_face_laugh_beam = 0xf59a,
	fa_laugh_beam = 0xf59a,
	fa_face_laugh_squint = 0xf59b,
	fa_laugh_squint = 0xf59b,
	fa_face_laugh_wink = 0xf59c,
	fa_laugh_wink = 0xf59c,
	fa_face_meh = 0xf11a,
	fa_meh = 0xf11a,
	fa_face_meh_blank = 0xf5a4,
	fa_meh_blank = 0xf5a4,
	fa_face_rolling_eyes = 0xf5a5,
	fa_meh_rolling_eyes = 0xf5a5,
	fa_face_sad_cry = 0xf5b3,
	fa_sad_cry = 0xf5b3,
	fa_face_sad_tear = 0xf5b4,
	fa_sad_tear = 0xf5b4,
	fa_face_smile = 0xf118,
	fa_smile = 0xf118,
	fa_face_smile_beam = 0xf5b8,
	fa_smile_beam = 0xf5b8,
	fa_face_smile_wink = 0xf4da,
	fa_smile_wink = 0xf4da,
	fa_face_surprise = 0xf5c2,
	fa_surprise = 0xf5c2,
	fa_face_tired = 0xf5c8,
	fa_tired = 0xf5c8,
	fa_fan = 0xf863,
	fa_faucet = 0xe005,
	fa_faucet_drip = 0xe006,
	fa_fax = 0xf1ac,
	fa_feather = 0xf52d,
	fa_feather_pointed = 0xf56b,
	fa_feather_alt = 0xf56b,
	fa_ferry = 0xe4ea,
	fa_file = 0xf15b,
	fa_file_arrow_down = 0xf56d,
	fa_file_download = 0xf56d,
	fa_file_arrow_up = 0xf574,
	fa_file_upload = 0xf574,
	fa_file_audio = 0xf1c7,
	fa_file_circle_check = 0xe5a0,
	fa_file_circle_exclamation = 0xe4eb,
	fa_file_circle_minus = 0xe4ed,
	fa_file_circle_plus = 0xe494,
	fa_file_circle_question = 0xe4ef,
	fa_file_circle_xmark = 0xe5a1,
	fa_file_code = 0xf1c9,
	fa_file_contract = 0xf56c,
	fa_file_csv = 0xf6dd,
	fa_file_excel = 0xf1c3,
	fa_file_export = 0xf56e,
	fa_arrow_right_from_file = 0xf56e,
	fa_file_image = 0xf1c5,
	fa_file_import = 0xf56f,
	fa_arrow_right_to_file = 0xf56f,
	fa_file_invoice = 0xf570,
	fa_file_invoice_dollar = 0xf571,
	fa_file_lines = 0xf15c,
	fa_file_alt = 0xf15c,
	fa_file_text = 0xf15c,
	fa_file_medical = 0xf477,
	fa_file_pdf = 0xf1c1,
	fa_file_pen = 0xf31c,
	fa_file_edit = 0xf31c,
	fa_file_powerpoint = 0xf1c4,
	fa_file_prescription = 0xf572,
	fa_file_shield = 0xe4f0,
	fa_file_signature = 0xf573,
	fa_file_video = 0xf1c8,
	fa_file_waveform = 0xf478,
	fa_file_medical_alt = 0xf478,
	fa_file_word = 0xf1c2,
	fa_file_zipper = 0xf1c6,
	fa_file_archive = 0xf1c6,
	fa_fill = 0xf575,
	fa_fill_drip = 0xf576,
	fa_film = 0xf008,
	fa_filter = 0xf0b0,
	fa_filter_circle_dollar = 0xf662,
	fa_funnel_dollar = 0xf662,
	fa_filter_circle_xmark = 0xe17b,
	fa_fingerprint = 0xf577,
	fa_fire = 0xf06d,
	fa_fire_burner = 0xe4f1,
	fa_fire_extinguisher = 0xf134,
	fa_fire_flame_curved = 0xf7e4,
	fa_fire_alt = 0xf7e4,
	fa_fire_flame_simple = 0xf46a,
	fa_burn = 0xf46a,
	fa_fish = 0xf578,
	fa_fish_fins = 0xe4f2,
	fa_flag = 0xf024,
	fa_flag_checkered = 0xf11e,
	fa_flag_usa = 0xf74d,
	fa_flask = 0xf0c3,
	fa_flask_vial = 0xe4f3,
	fa_floppy_disk = 0xf0c7,
	fa_save = 0xf0c7,
	fa_florin_sign = 0xe184,
	fa_folder = 0xf07b,
	fa_folder_blank = 0xf07b,
	fa_folder_closed = 0xe185,
	fa_folder_minus = 0xf65d,
	fa_folder_open = 0xf07c,
	fa_folder_plus = 0xf65e,
	fa_folder_tree = 0xf802,
	fa_font = 0xf031,
	fa_football = 0xf44e,
	fa_football_ball = 0xf44e,
	fa_forward = 0xf04e,
	fa_forward_fast = 0xf050,
	fa_fast_forward = 0xf050,
	fa_forward_step = 0xf051,
	fa_step_forward = 0xf051,
	fa_franc_sign = 0xe18f,
	fa_frog = 0xf52e,
	fa_futbol = 0xf1e3,
	fa_futbol_ball = 0xf1e3,
	fa_soccer_ball = 0xf1e3,
	fa_g = 0x47,
	fa_gamepad = 0xf11b,
	fa_gas_pump = 0xf52f,
	fa_gauge = 0xf624,
	fa_dashboard = 0xf624,
	fa_gauge_med = 0xf624,
	fa_tachometer_alt_average = 0xf624,
	fa_gauge_high = 0xf625,
	fa_tachometer_alt = 0xf625,
	fa_tachometer_alt_fast = 0xf625,
	fa_gauge_simple = 0xf629,
	fa_gauge_simple_med = 0xf629,
	fa_tachometer_average = 0xf629,
	fa_gauge_simple_high = 0xf62a,
	fa_tachometer = 0xf62a,
	fa_tachometer_fast = 0xf62a,
	fa_gavel = 0xf0e3,
	fa_legal = 0xf0e3,
	fa_gear = 0xf013,
	fa_cog = 0xf013,
	fa_gears = 0xf085,
	fa_cogs = 0xf085,
	fa_gem = 0xf3a5,
	fa_genderless = 0xf22d,
	fa_ghost = 0xf6e2,
	fa_gift = 0xf06b,
	fa_gifts = 0xf79c,
	fa_glass_water = 0xe4f4,
	fa_glass_water_droplet = 0xe4f5,
	fa_glasses = 0xf530,
	fa_globe = 0xf0ac,
	fa_golf_ball_tee = 0xf450,
	fa_golf_ball = 0xf450,
	fa_gopuram = 0xf664,
	fa_graduation_cap = 0xf19d,
	fa_mortar_board = 0xf19d,
	fa_greater_than = 0x3e,
	fa_greater_than_equal = 0xf532,
	fa_grip = 0xf58d,
	fa_grip_horizontal = 0xf58d,
	fa_grip_lines = 0xf7a4,
	fa_grip_lines_vertical = 0xf7a5,
	fa_grip_vertical = 0xf58e,
	fa_group_arrows_rotate = 0xe4f6,
	fa_guarani_sign = 0xe19a,
	fa_guitar = 0xf7a6,
	fa_gun = 0xe19b,
	fa_h = 0x48,
	fa_hammer = 0xf6e3,
	fa_hamsa = 0xf665,
	fa_hand = 0xf256,
	fa_hand_paper = 0xf256,
	fa_hand_back_fist = 0xf255,
	fa_hand_rock = 0xf255,
	fa_hand_dots = 0xf461,
	fa_allergies = 0xf461,
	fa_hand_fist = 0xf6de,
	fa_fist_raised = 0xf6de,
	fa_hand_holding = 0xf4bd,
	fa_hand_holding_dollar = 0xf4c0,
	fa_hand_holding_usd = 0xf4c0,
	fa_hand_holding_droplet = 0xf4c1,
	fa_hand_holding_water = 0xf4c1,
	fa_hand_holding_hand = 0xe4f7,
	fa_hand_holding_heart = 0xf4be,
	fa_hand_holding_medical = 0xe05c,
	fa_hand_lizard = 0xf258,
	fa_hand_middle_finger = 0xf806,
	fa_hand_peace = 0xf25b,
	fa_hand_point_down = 0xf0a7,
	fa_hand_point_left = 0xf0a5,
	fa_hand_point_right = 0xf0a4,
	fa_hand_point_up = 0xf0a6,
	fa_hand_pointer = 0xf25a,
	fa_hand_scissors = 0xf257,
	fa_hand_sparkles = 0xe05d,
	fa_hand_spock = 0xf259,
	fa_handcuffs = 0xe4f8,
	fa_hands = 0xf2a7,
	fa_sign_language = 0xf2a7,
	fa_signing = 0xf2a7,
	fa_hands_asl_interpreting = 0xf2a3,
	fa_american_sign_language_interpreting = 0xf2a3,
	fa_asl_interpreting = 0xf2a3,
	fa_hands_american_sign_language_interpreting = 0xf2a3,
	fa_hands_bound = 0xe4f9,
	fa_hands_bubbles = 0xe05e,
	fa_hands_wash = 0xe05e,
	fa_hands_clapping = 0xe1a8,
	fa_hands_holding = 0xf4c2,
	fa_hands_holding_child = 0xe4fa,
	fa_hands_holding_circle = 0xe4fb,
	fa_hands_praying = 0xf684,
	fa_praying_hands = 0xf684,
	fa_handshake = 0xf2b5,
	fa_handshake_angle = 0xf4c4,
	fa_hands_helping = 0xf4c4,
	fa_handshake_simple = 0xf4c6,
	fa_handshake_alt = 0xf4c6,
	fa_handshake_simple_slash = 0xe05f,
	fa_handshake_alt_slash = 0xe05f,
	fa_handshake_slash = 0xe060,
	fa_hanukiah = 0xf6e6,
	fa_hard_drive = 0xf0a0,
	fa_hdd = 0xf0a0,
	fa_hashtag = 0x23,
	fa_hat_cowboy = 0xf8c0,
	fa_hat_cowboy_side = 0xf8c1,
	fa_hat_wizard = 0xf6e8,
	fa_head_side_cough = 0xe061,
	fa_head_side_cough_slash = 0xe062,
	fa_head_side_mask = 0xe063,
	fa_head_side_virus = 0xe064,
	fa_heading = 0xf1dc,
	fa_header = 0xf1dc,
	fa_headphones = 0xf025,
	fa_headphones_simple = 0xf58f,
	fa_headphones_alt = 0xf58f,
	fa_headset = 0xf590,
	fa_heart = 0xf004,
	fa_heart_circle_bolt = 0xe4fc,
	fa_heart_circle_check = 0xe4fd,
	fa_heart_circle_exclamation = 0xe4fe,
	fa_heart_circle_minus = 0xe4ff,
	fa_heart_circle_plus = 0xe500,
	fa_heart_circle_xmark = 0xe501,
	fa_heart_crack = 0xf7a9,
	fa_heart_broken = 0xf7a9,
	fa_heart_pulse = 0xf21e,
	fa_heartbeat = 0xf21e,
	fa_helicopter = 0xf533,
	fa_helicopter_symbol = 0xe502,
	fa_helmet_safety = 0xf807,
	fa_hard_hat = 0xf807,
	fa_hat_hard = 0xf807,
	fa_helmet_un = 0xe503,
	fa_highlighter = 0xf591,
	fa_hill_avalanche = 0xe507,
	fa_hill_rockslide = 0xe508,
	fa_hippo = 0xf6ed,
	fa_hockey_puck = 0xf453,
	fa_holly_berry = 0xf7aa,
	fa_horse = 0xf6f0,
	fa_horse_head = 0xf7ab,
	fa_hospital = 0xf0f8,
	fa_hospital_alt = 0xf0f8,
	fa_hospital_wide = 0xf0f8,
	fa_hospital_user = 0xf80d,
	fa_hot_tub_person = 0xf593,
	fa_hot_tub = 0xf593,
	fa_hotdog = 0xf80f,
	fa_hotel = 0xf594,
	fa_hourglass = 0xf254,
	fa_hourglass_empty = 0xf254,
	fa_hourglass_end = 0xf253,
	fa_hourglass_3 = 0xf253,
	fa_hourglass_half = 0xf252,
	fa_hourglass_2 = 0xf252,
	fa_hourglass_start = 0xf251,
	fa_hourglass_1 = 0xf251,
	fa_house = 0xf015,
	fa_home = 0xf015,
	fa_home_alt = 0xf015,
	fa_home_lg_alt = 0xf015,
	fa_house_chimney = 0xe3af,
	fa_home_lg = 0xe3af,
	fa_house_chimney_crack = 0xf6f1,
	fa_house_damage = 0xf6f1,
	fa_house_chimney_medical = 0xf7f2,
	fa_clinic_medical = 0xf7f2,
	fa_house_chimney_user = 0xe065,
	fa_house_chimney_window = 0xe00d,
	fa_house_circle_check = 0xe509,
	fa_house_circle_exclamation = 0xe50a,
	fa_house_circle_xmark = 0xe50b,
	fa_house_crack = 0xe3b1,
	fa_house_fire = 0xe50c,
	fa_house_flag = 0xe50d,
	fa_house_flood_water = 0xe50e,
	fa_house_flood_water_circle_arrow_right = 0xe50f,
	fa_house_laptop = 0xe066,
	fa_laptop_house = 0xe066,
	fa_house_lock = 0xe510,
	fa_house_medical = 0xe3b2,
	fa_house_medical_circle_check = 0xe511,
	fa_house_medical_circle_exclamation = 0xe512,
	fa_house_medical_circle_xmark = 0xe513,
	fa_house_medical_flag = 0xe514,
	fa_house_signal = 0xe012,
	fa_house_tsunami = 0xe515,
	fa_house_user = 0xe1b0,
	fa_home_user = 0xe1b0,
	fa_hryvnia_sign = 0xf6f2,
	fa_hryvnia = 0xf6f2,
	fa_hurricane = 0xf751,
	fa_i = 0x49,
	fa_i_cursor = 0xf246,
	fa_ice_cream = 0xf810,
	fa_icicles = 0xf7ad,
	fa_icons = 0xf86d,
	fa_heart_music_camera_bolt = 0xf86d,
	fa_id_badge = 0xf2c1,
	fa_id_card = 0xf2c2,
	fa_drivers_license = 0xf2c2,
	fa_id_card_clip = 0xf47f,
	fa_id_card_alt = 0xf47f,
	fa_igloo = 0xf7ae,
	fa_image = 0xf03e,
	fa_image_portrait = 0xf3e0,
	fa_portrait = 0xf3e0,
	fa_images = 0xf302,
	fa_inbox = 0xf01c,
	fa_indent = 0xf03c,
	fa_indian_rupee_sign = 0xe1bc,
	fa_indian_rupee = 0xe1bc,
	fa_inr = 0xe1bc,
	fa_industry = 0xf275,
	fa_infinity = 0xf534,
	fa_info = 0xf129,
	fa_italic = 0xf033,
	fa_j = 0x4a,
	fa_jar = 0xe516,
	fa_jar_wheat = 0xe517,
	fa_jedi = 0xf669,
	fa_jet_fighter = 0xf0fb,
	fa_fighter_jet = 0xf0fb,
	fa_jet_fighter_up = 0xe518,
	fa_joint = 0xf595,
	fa_jug_detergent = 0xe519,
	fa_k = 0x4b,
	fa_kaaba = 0xf66b,
	fa_key = 0xf084,
	fa_keyboard = 0xf11c,
	fa_khanda = 0xf66d,
	fa_kip_sign = 0xe1c4,
	fa_kit_medical = 0xf479,
	fa_first_aid = 0xf479,
	fa_kitchen_set = 0xe51a,
	fa_kiwi_bird = 0xf535,
	fa_l = 0x4c,
	fa_land_mine_on = 0xe51b,
	fa_landmark = 0xf66f,
	fa_landmark_dome = 0xf752,
	fa_landmark_alt = 0xf752,
	fa_landmark_flag = 0xe51c,
	fa_language = 0xf1ab,
	fa_laptop = 0xf109,
	fa_laptop_code = 0xf5fc,
	fa_laptop_file = 0xe51d,
	fa_laptop_medical = 0xf812,
	fa_lari_sign = 0xe1c8,
	fa_layer_group = 0xf5fd,
	fa_leaf = 0xf06c,
	fa_left_long = 0xf30a,
	fa_long_arrow_alt_left = 0xf30a,
	fa_left_right = 0xf337,
	fa_arrows_alt_h = 0xf337,
	fa_lemon = 0xf094,
	fa_less_than = 0x3c,
	fa_less_than_equal = 0xf537,
	fa_life_ring = 0xf1cd,
	fa_lightbulb = 0xf0eb,
	fa_lines_leaning = 0xe51e,
	fa_link = 0xf0c1,
	fa_chain = 0xf0c1,
	fa_link_slash = 0xf127,
	fa_chain_broken = 0xf127,
	fa_chain_slash = 0xf127,
	fa_unlink = 0xf127,
	fa_lira_sign = 0xf195,
	fa_list = 0xf03a,
	fa_list_squares = 0xf03a,
	fa_list_check = 0xf0ae,
	fa_tasks = 0xf0ae,
	fa_list_ol = 0xf0cb,
	fa_list_1_2 = 0xf0cb,
	fa_list_numeric = 0xf0cb,
	fa_list_ul = 0xf0ca,
	fa_list_dots = 0xf0ca,
	fa_litecoin_sign = 0xe1d3,
	fa_location_arrow = 0xf124,
	fa_location_crosshairs = 0xf601,
	fa_location = 0xf601,
	fa_location_dot = 0xf3c5,
	fa_map_marker_alt = 0xf3c5,
	fa_location_pin = 0xf041,
	fa_map_marker = 0xf041,
	fa_location_pin_lock = 0xe51f,
	fa_lock = 0xf023,
	fa_lock_open = 0xf3c1,
	fa_locust = 0xe520,
	fa_lungs = 0xf604,
	fa_lungs_virus = 0xe067,
	fa_m = 0x4d,
	fa_magnet = 0xf076,
	fa_magnifying_glass = 0xf002,
	fa_search = 0xf002,
	fa_magnifying_glass_arrow_right = 0xe521,
	fa_magnifying_glass_chart = 0xe522,
	fa_magnifying_glass_dollar = 0xf688,
	fa_search_dollar = 0xf688,
	fa_magnifying_glass_location = 0xf689,
	fa_search_location = 0xf689,
	fa_magnifying_glass_minus = 0xf010,
	fa_search_minus = 0xf010,
	fa_magnifying_glass_plus = 0xf00e,
	fa_search_plus = 0xf00e,
	fa_manat_sign = 0xe1d5,
	fa_map = 0xf279,
	fa_map_location = 0xf59f,
	fa_map_marked = 0xf59f,
	fa_map_location_dot = 0xf5a0,
	fa_map_marked_alt = 0xf5a0,
	fa_map_pin = 0xf276,
	fa_marker = 0xf5a1,
	fa_mars = 0xf222,
	fa_mars_and_venus = 0xf224,
	fa_mars_and_venus_burst = 0xe523,
	fa_mars_double = 0xf227,
	fa_mars_stroke = 0xf229,
	fa_mars_stroke_right = 0xf22b,
	fa_mars_stroke_h = 0xf22b,
	fa_mars_stroke_up = 0xf22a,
	fa_mars_stroke_v = 0xf22a,
	fa_martini_glass = 0xf57b,
	fa_glass_martini_alt = 0xf57b,
	fa_martini_glass_citrus = 0xf561,
	fa_cocktail = 0xf561,
	fa_martini_glass_empty = 0xf000,
	fa_glass_martini = 0xf000,
	fa_mask = 0xf6fa,
	fa_mask_face = 0xe1d7,
	fa_mask_ventilator = 0xe524,
	fa_masks_theater = 0xf630,
	fa_theater_masks = 0xf630,
	fa_mattress_pillow = 0xe525,
	fa_maximize = 0xf31e,
	fa_expand_arrows_alt = 0xf31e,
	fa_medal = 0xf5a2,
	fa_memory = 0xf538,
	fa_menorah = 0xf676,
	fa_mercury = 0xf223,
	fa_message = 0xf27a,
	fa_comment_alt = 0xf27a,
	fa_meteor = 0xf753,
	fa_microchip = 0xf2db,
	fa_microphone = 0xf130,
	fa_microphone_lines = 0xf3c9,
	fa_microphone_alt = 0xf3c9,
	fa_microphone_lines_slash = 0xf539,
	fa_microphone_alt_slash = 0xf539,
	fa_microphone_slash = 0xf131,
	fa_microscope = 0xf610,
	fa_mill_sign = 0xe1ed,
	fa_minimize = 0xf78c,
	fa_compress_arrows_alt = 0xf78c,
	fa_minus = 0xf068,
	fa_subtract = 0xf068,
	fa_mitten = 0xf7b5,
	fa_mobile = 0xf3ce,
	fa_mobile_android = 0xf3ce,
	fa_mobile_phone = 0xf3ce,
	fa_mobile_button = 0xf10b,
	fa_mobile_retro = 0xe527,
	fa_mobile_screen = 0xf3cf,
	fa_mobile_android_alt = 0xf3cf,
	fa_mobile_screen_button = 0xf3cd,
	fa_mobile_alt = 0xf3cd,
	fa_money_bill = 0xf0d6,
	fa_money_bill_1 = 0xf3d1,
	fa_money_bill_alt = 0xf3d1,
	fa_money_bill_1_wave = 0xf53b,
	fa_money_bill_wave_alt = 0xf53b,
	fa_money_bill_transfer = 0xe528,
	fa_money_bill_trend_up = 0xe529,
	fa_money_bill_wave = 0xf53a,
	fa_money_bill_wheat = 0xe52a,
	fa_money_bills = 0xe1f3,
	fa_money_check = 0xf53c,
	fa_money_check_dollar = 0xf53d,
	fa_money_check_alt = 0xf53d,
	fa_monument = 0xf5a6,
	fa_moon = 0xf186,
	fa_mortar_pestle = 0xf5a7,
	fa_mosque = 0xf678,
	fa_mosquito = 0xe52b,
	fa_mosquito_net = 0xe52c,
	fa_motorcycle = 0xf21c,
	fa_mound = 0xe52d,
	fa_mountain = 0xf6fc,
	fa_mountain_city = 0xe52e,
	fa_mountain_sun = 0xe52f,
	fa_mug_hot = 0xf7b6,
	fa_mug_saucer = 0xf0f4,
	fa_coffee = 0xf0f4,
	fa_music = 0xf001,
	fa_n = 0x4e,
	fa_naira_sign = 0xe1f6,
	fa_network_wired = 0xf6ff,
	fa_neuter = 0xf22c,
	fa_newspaper = 0xf1ea,
	fa_not_equal = 0xf53e,
	fa_notdef = 0xe1fe,
	fa_note_sticky = 0xf249,
	fa_sticky_note = 0xf249,
	fa_notes_medical = 0xf481,
	fa_o = 0x4f,
	fa_object_group = 0xf247,
	fa_object_ungroup = 0xf248,
	fa_oil_can = 0xf613,
	fa_oil_well = 0xe532,
	fa_om = 0xf679,
	fa_otter = 0xf700,
	fa_outdent = 0xf03b,
	fa_dedent = 0xf03b,
	fa_p = 0x50,
	fa_pager = 0xf815,
	fa_paint_roller = 0xf5aa,
	fa_paintbrush = 0xf1fc,
	fa_paint_brush = 0xf1fc,
	fa_palette = 0xf53f,
	fa_pallet = 0xf482,
	fa_panorama = 0xe209,
	fa_paper_plane = 0xf1d8,
	fa_paperclip = 0xf0c6,
	fa_parachute_box = 0xf4cd,
	fa_paragraph = 0xf1dd,
	fa_passport = 0xf5ab,
	fa_paste = 0xf0ea,
	fa_file_clipboard = 0xf0ea,
	fa_pause = 0xf04c,
	fa_paw = 0xf1b0,
	fa_peace = 0xf67c,
	fa_pen = 0xf304,
	fa_pen_clip = 0xf305,
	fa_pen_alt = 0xf305,
	fa_pen_fancy = 0xf5ac,
	fa_pen_nib = 0xf5ad,
	fa_pen_ruler = 0xf5ae,
	fa_pencil_ruler = 0xf5ae,
	fa_pen_to_square = 0xf044,
	fa_edit = 0xf044,
	fa_pencil = 0xf303,
	fa_pencil_alt = 0xf303,
	fa_people_arrows = 0xe068,
	fa_people_arrows_left_right = 0xe068,
	fa_people_carry_box = 0xf4ce,
	fa_people_carry = 0xf4ce,
	fa_people_group = 0xe533,
	fa_people_line = 0xe534,
	fa_people_pulling = 0xe535,
	fa_people_robbery = 0xe536,
	fa_people_roof = 0xe537,
	fa_pepper_hot = 0xf816,
	fa_percent = 0x25,
	fa_percentage = 0x25,
	fa_person = 0xf183,
	fa_male = 0xf183,
	fa_person_arrow_down_to_line = 0xe538,
	fa_person_arrow_up_from_line = 0xe539,
	fa_person_biking = 0xf84a,
	fa_biking = 0xf84a,
	fa_person_booth = 0xf756,
	fa_person_breastfeeding = 0xe53a,
	fa_person_burst = 0xe53b,
	fa_person_cane = 0xe53c,
	fa_person_chalkboard = 0xe53d,
	fa_person_circle_check = 0xe53e,
	fa_person_circle_exclamation = 0xe53f,
	fa_person_circle_minus = 0xe540,
	fa_person_circle_plus = 0xe541,
	fa_person_circle_question = 0xe542,
	fa_person_circle_xmark = 0xe543,
	fa_person_digging = 0xf85e,
	fa_digging = 0xf85e,
	fa_person_dots_from_line = 0xf470,
	fa_diagnoses = 0xf470,
	fa_person_dress = 0xf182,
	fa_female = 0xf182,
	fa_person_dress_burst = 0xe544,
	fa_person_drowning = 0xe545,
	fa_person_falling = 0xe546,
	fa_person_falling_burst = 0xe547,
	fa_person_half_dress = 0xe548,
	fa_person_harassing = 0xe549,
	fa_person_hiking = 0xf6ec,
	fa_hiking = 0xf6ec,
	fa_person_military_pointing = 0xe54a,
	fa_person_military_rifle = 0xe54b,
	fa_person_military_to_person = 0xe54c,
	fa_person_praying = 0xf683,
	fa_pray = 0xf683,
	fa_person_pregnant = 0xe31e,
	fa_person_rays = 0xe54d,
	fa_person_rifle = 0xe54e,
	fa_person_running = 0xf70c,
	fa_running = 0xf70c,
	fa_person_shelter = 0xe54f,
	fa_person_skating = 0xf7c5,
	fa_skating = 0xf7c5,
	fa_person_skiing = 0xf7c9,
	fa_skiing = 0xf7c9,
	fa_person_skiing_nordic = 0xf7ca,
	fa_skiing_nordic = 0xf7ca,
	fa_person_snowboarding = 0xf7ce,
	fa_snowboarding = 0xf7ce,
	fa_person_swimming = 0xf5c4,
	fa_swimmer = 0xf5c4,
	fa_person_through_window = 0xe5a9,
	fa_person_walking = 0xf554,
	fa_walking = 0xf554,
	fa_person_walking_arrow_loop_left = 0xe551,
	fa_person_walking_arrow_right = 0xe552,
	fa_person_walking_dashed_line_arrow_right = 0xe553,
	fa_person_walking_luggage = 0xe554,
	fa_person_walking_with_cane = 0xf29d,
	fa_blind = 0xf29d,
	fa_peseta_sign = 0xe221,
	fa_peso_sign = 0xe222,
	fa_phone = 0xf095,
	fa_phone_flip = 0xf879,
	fa_phone_alt = 0xf879,
	fa_phone_slash = 0xf3dd,
	fa_phone_volume = 0xf2a0,
	fa_volume_control_phone = 0xf2a0,
	fa_photo_film = 0xf87c,
	fa_photo_video = 0xf87c,
	fa_piggy_bank = 0xf4d3,
	fa_pills = 0xf484,
	fa_pizza_slice = 0xf818,
	fa_place_of_worship = 0xf67f,
	fa_plane = 0xf072,
	fa_plane_arrival = 0xf5af,
	fa_plane_circle_check = 0xe555,
	fa_plane_circle_exclamation = 0xe556,
	fa_plane_circle_xmark = 0xe557,
	fa_plane_departure = 0xf5b0,
	fa_plane_lock = 0xe558,
	fa_plane_slash = 0xe069,
	fa_plane_up = 0xe22d,
	fa_plant_wilt = 0xe5aa,
	fa_plate_wheat = 0xe55a,
	fa_play = 0xf04b,
	fa_plug = 0xf1e6,
	fa_plug_circle_bolt = 0xe55b,
	fa_plug_circle_check = 0xe55c,
	fa_plug_circle_exclamation = 0xe55d,
	fa_plug_circle_minus = 0xe55e,
	fa_plug_circle_plus = 0xe55f,
	fa_plug_circle_xmark = 0xe560,
	fa_plus = 0x2b,
	fa_add = 0x2b,
	fa_plus_minus = 0xe43c,
	fa_podcast = 0xf2ce,
	fa_poo = 0xf2fe,
	fa_poo_storm = 0xf75a,
	fa_poo_bolt = 0xf75a,
	fa_poop = 0xf619,
	fa_power_off = 0xf011,
	fa_prescription = 0xf5b1,
	fa_prescription_bottle = 0xf485,
	fa_prescription_bottle_medical = 0xf486,
	fa_prescription_bottle_alt = 0xf486,
	fa_print = 0xf02f,
	fa_pump_medical = 0xe06a,
	fa_pump_soap = 0xe06b,
	fa_puzzle_piece = 0xf12e,
	fa_q = 0x51,
	fa_qrcode = 0xf029,
	fa_question = 0x3f,
	fa_quote_left = 0xf10d,
	fa_quote_left_alt = 0xf10d,
	fa_quote_right = 0xf10e,
	fa_quote_right_alt = 0xf10e,
	fa_r = 0x52,
	fa_radiation = 0xf7b9,
	fa_radio = 0xf8d7,
	fa_rainbow = 0xf75b,
	fa_ranking_star = 0xe561,
	fa_receipt = 0xf543,
	fa_record_vinyl = 0xf8d9,
	fa_rectangle_ad = 0xf641,
	fa_ad = 0xf641,
	fa_rectangle_list = 0xf022,
	fa_list_alt = 0xf022,
	fa_rectangle_xmark = 0xf410,
	fa_rectangle_times = 0xf410,
	fa_times_rectangle = 0xf410,
	fa_window_close = 0xf410,
	fa_recycle = 0xf1b8,
	fa_registered = 0xf25d,
	fa_repeat = 0xf363,
	fa_reply = 0xf3e5,
	fa_mail_reply = 0xf3e5,
	fa_reply_all = 0xf122,
	fa_mail_reply_all = 0xf122,
	fa_republican = 0xf75e,
	fa_restroom = 0xf7bd,
	fa_retweet = 0xf079,
	fa_ribbon = 0xf4d6,
	fa_right_from_bracket = 0xf2f5,
	fa_sign_out_alt = 0xf2f5,
	fa_right_left = 0xf362,
	fa_exchange_alt = 0xf362,
	fa_right_long = 0xf30b,
	fa_long_arrow_alt_right = 0xf30b,
	fa_right_to_bracket = 0xf2f6,
	fa_sign_in_alt = 0xf2f6,
	fa_ring = 0xf70b,
	fa_road = 0xf018,
	fa_road_barrier = 0xe562,
	fa_road_bridge = 0xe563,
	fa_road_circle_check = 0xe564,
	fa_road_circle_exclamation = 0xe565,
	fa_road_circle_xmark = 0xe566,
	fa_road_lock = 0xe567,
	fa_road_spikes = 0xe568,
	fa_robot = 0xf544,
	fa_rocket = 0xf135,
	fa_rotate = 0xf2f1,
	fa_sync_alt = 0xf2f1,
	fa_rotate_left = 0xf2ea,
	fa_rotate_back = 0xf2ea,
	fa_rotate_backward = 0xf2ea,
	fa_undo_alt = 0xf2ea,
	fa_rotate_right = 0xf2f9,
	fa_redo_alt = 0xf2f9,
	fa_rotate_forward = 0xf2f9,
	fa_route = 0xf4d7,
	fa_rss = 0xf09e,
	fa_feed = 0xf09e,
	fa_ruble_sign = 0xf158,
	fa_rouble = 0xf158,
	fa_rub = 0xf158,
	fa_ruble = 0xf158,
	fa_rug = 0xe569,
	fa_ruler = 0xf545,
	fa_ruler_combined = 0xf546,
	fa_ruler_horizontal = 0xf547,
	fa_ruler_vertical = 0xf548,
	fa_rupee_sign = 0xf156,
	fa_rupee = 0xf156,
	fa_rupiah_sign = 0xe23d,
	fa_s = 0x53,
	fa_sack_dollar = 0xf81d,
	fa_sack_xmark = 0xe56a,
	fa_sailboat = 0xe445,
	fa_satellite = 0xf7bf,
	fa_satellite_dish = 0xf7c0,
	fa_scale_balanced = 0xf24e,
	fa_balance_scale = 0xf24e,
	fa_scale_unbalanced = 0xf515,
	fa_balance_scale_left = 0xf515,
	fa_scale_unbalanced_flip = 0xf516,
	fa_balance_scale_right = 0xf516,
	fa_school = 0xf549,
	fa_school_circle_check = 0xe56b,
	fa_school_circle_exclamation = 0xe56c,
	fa_school_circle_xmark = 0xe56d,
	fa_school_flag = 0xe56e,
	fa_school_lock = 0xe56f,
	fa_scissors = 0xf0c4,
	fa_cut = 0xf0c4,
	fa_screwdriver = 0xf54a,
	fa_screwdriver_wrench = 0xf7d9,
	fa_tools = 0xf7d9,
	fa_scroll = 0xf70e,
	fa_scroll_torah = 0xf6a0,
	fa_torah = 0xf6a0,
	fa_sd_card = 0xf7c2,
	fa_section = 0xe447,
	fa_seedling = 0xf4d8,
	fa_sprout = 0xf4d8,
	fa_server = 0xf233,
	fa_shapes = 0xf61f,
	fa_triangle_circle_square = 0xf61f,
	fa_share = 0xf064,
	fa_mail_forward = 0xf064,
	fa_share_from_square = 0xf14d,
	fa_share_square = 0xf14d,
	fa_share_nodes = 0xf1e0,
	fa_share_alt = 0xf1e0,
	fa_sheet_plastic = 0xe571,
	fa_shekel_sign = 0xf20b,
	fa_ils = 0xf20b,
	fa_shekel = 0xf20b,
	fa_sheqel = 0xf20b,
	fa_sheqel_sign = 0xf20b,
	fa_shield = 0xf132,
	fa_shield_blank = 0xf132,
	fa_shield_cat = 0xe572,
	fa_shield_dog = 0xe573,
	fa_shield_halved = 0xf3ed,
	fa_shield_alt = 0xf3ed,
	fa_shield_heart = 0xe574,
	fa_shield_virus = 0xe06c,
	fa_ship = 0xf21a,
	fa_shirt = 0xf553,
	fa_t_shirt = 0xf553,
	fa_tshirt = 0xf553,
	fa_shoe_prints = 0xf54b,
	fa_shop = 0xf54f,
	fa_store_alt = 0xf54f,
	fa_shop_lock = 0xe4a5,
	fa_shop_slash = 0xe070,
	fa_store_alt_slash = 0xe070,
	fa_shower = 0xf2cc,
	fa_shrimp = 0xe448,
	fa_shuffle = 0xf074,
	fa_random = 0xf074,
	fa_shuttle_space = 0xf197,
	fa_space_shuttle = 0xf197,
	fa_sign_hanging = 0xf4d9,
	fa_sign = 0xf4d9,
	fa_signal = 0xf012,
	fa_signal_5 = 0xf012,
	fa_signal_perfect = 0xf012,
	fa_signature = 0xf5b7,
	fa_signs_post = 0xf277,
	fa_map_signs = 0xf277,
	fa_sim_card = 0xf7c4,
	fa_sink = 0xe06d,
	fa_sitemap = 0xf0e8,
	fa_skull = 0xf54c,
	fa_skull_crossbones = 0xf714,
	fa_slash = 0xf715,
	fa_sleigh = 0xf7cc,
	fa_sliders = 0xf1de,
	fa_sliders_h = 0xf1de,
	fa_smog = 0xf75f,
	fa_smoking = 0xf48d,
	fa_snowflake = 0xf2dc,
	fa_snowman = 0xf7d0,
	fa_snowplow = 0xf7d2,
	fa_soap = 0xe06e,
	fa_socks = 0xf696,
	fa_solar_panel = 0xf5ba,
	fa_sort = 0xf0dc,
	fa_unsorted = 0xf0dc,
	fa_sort_down = 0xf0dd,
	fa_sort_desc = 0xf0dd,
	fa_sort_up = 0xf0de,
	fa_sort_asc = 0xf0de,
	fa_spa = 0xf5bb,
	fa_spaghetti_monster_flying = 0xf67b,
	fa_pastafarianism = 0xf67b,
	fa_spell_check = 0xf891,
	fa_spider = 0xf717,
	fa_spinner = 0xf110,
	fa_splotch = 0xf5bc,
	fa_spoon = 0xf2e5,
	fa_utensil_spoon = 0xf2e5,
	fa_spray_can = 0xf5bd,
	fa_spray_can_sparkles = 0xf5d0,
	fa_air_freshener = 0xf5d0,
	fa_square = 0xf0c8,
	fa_square_arrow_up_right = 0xf14c,
	fa_external_link_square = 0xf14c,
	fa_square_caret_down = 0xf150,
	fa_caret_square_down = 0xf150,
	fa_square_caret_left = 0xf191,
	fa_caret_square_left = 0xf191,
	fa_square_caret_right = 0xf152,
	fa_caret_square_right = 0xf152,
	fa_square_caret_up = 0xf151,
	fa_caret_square_up = 0xf151,
	fa_square_check = 0xf14a,
	fa_check_square = 0xf14a,
	fa_square_envelope = 0xf199,
	fa_envelope_square = 0xf199,
	fa_square_full = 0xf45c,
	fa_square_h = 0xf0fd,
	fa_h_square = 0xf0fd,
	fa_square_minus = 0xf146,
	fa_minus_square = 0xf146,
	fa_square_nfi = 0xe576,
	fa_square_parking = 0xf540,
	fa_parking = 0xf540,
	fa_square_pen = 0xf14b,
	fa_pen_square = 0xf14b,
	fa_pencil_square = 0xf14b,
	fa_square_person_confined = 0xe577,
	fa_square_phone = 0xf098,
	fa_phone_square = 0xf098,
	fa_square_phone_flip = 0xf87b,
	fa_phone_square_alt = 0xf87b,
	fa_square_plus = 0xf0fe,
	fa_plus_square = 0xf0fe,
	fa_square_poll_horizontal = 0xf682,
	fa_poll_h = 0xf682,
	fa_square_poll_vertical = 0xf681,
	fa_poll = 0xf681,
	fa_square_root_variable = 0xf698,
	fa_square_root_alt = 0xf698,
	fa_square_rss = 0xf143,
	fa_rss_square = 0xf143,
	fa_square_share_nodes = 0xf1e1,
	fa_share_alt_square = 0xf1e1,
	fa_square_up_right = 0xf360,
	fa_external_link_square_alt = 0xf360,
	fa_square_virus = 0xe578,
	fa_square_xmark = 0xf2d3,
	fa_times_square = 0xf2d3,
	fa_xmark_square = 0xf2d3,
	fa_staff_snake = 0xe579,
	fa_rod_asclepius = 0xe579,
	fa_rod_snake = 0xe579,
	fa_staff_aesculapius = 0xe579,
	fa_stairs = 0xe289,
	fa_stamp = 0xf5bf,
	fa_stapler = 0xe5af,
	fa_star = 0xf005,
	fa_star_and_crescent = 0xf699,
	fa_star_half = 0xf089,
	fa_star_half_stroke = 0xf5c0,
	fa_star_half_alt = 0xf5c0,
	fa_star_of_david = 0xf69a,
	fa_star_of_life = 0xf621,
	fa_sterling_sign = 0xf154,
	fa_gbp = 0xf154,
	fa_pound_sign = 0xf154,
	fa_stethoscope = 0xf0f1,
	fa_stop = 0xf04d,
	fa_stopwatch = 0xf2f2,
	fa_stopwatch_20 = 0xe06f,
	fa_store = 0xf54e,
	fa_store_slash = 0xe071,
	fa_street_view = 0xf21d,
	fa_strikethrough = 0xf0cc,
	fa_stroopwafel = 0xf551,
	fa_subscript = 0xf12c,
	fa_suitcase = 0xf0f2,
	fa_suitcase_medical = 0xf0fa,
	fa_medkit = 0xf0fa,
	fa_suitcase_rolling = 0xf5c1,
	fa_sun = 0xf185,
	fa_sun_plant_wilt = 0xe57a,
	fa_superscript = 0xf12b,
	fa_swatchbook = 0xf5c3,
	fa_synagogue = 0xf69b,
	fa_syringe = 0xf48e,
	fa_t = 0x54,
	fa_table = 0xf0ce,
	fa_table_cells = 0xf00a,
	fa_th = 0xf00a,
	fa_table_cells_large = 0xf009,
	fa_th_large = 0xf009,
	fa_table_columns = 0xf0db,
	fa_columns = 0xf0db,
	fa_table_list = 0xf00b,
	fa_th_list = 0xf00b,
	fa_table_tennis_paddle_ball = 0xf45d,
	fa_ping_pong_paddle_ball = 0xf45d,
	fa_table_tennis = 0xf45d,
	fa_tablet = 0xf3fb,
	fa_tablet_android = 0xf3fb,
	fa_tablet_button = 0xf10a,
	fa_tablet_screen_button = 0xf3fa,
	fa_tablet_alt = 0xf3fa,
	fa_tablets = 0xf490,
	fa_tachograph_digital = 0xf566,
	fa_digital_tachograph = 0xf566,
	fa_tag = 0xf02b,
	fa_tags = 0xf02c,
	fa_tape = 0xf4db,
	fa_tarp = 0xe57b,
	fa_tarp_droplet = 0xe57c,
	fa_taxi = 0xf1ba,
	fa_cab = 0xf1ba,
	fa_teeth = 0xf62e,
	fa_teeth_open = 0xf62f,
	fa_temperature_arrow_down = 0xe03f,
	fa_temperature_down = 0xe03f,
	fa_temperature_arrow_up = 0xe040,
	fa_temperature_up = 0xe040,
	fa_temperature_empty = 0xf2cb,
	fa_temperature_0 = 0xf2cb,
	fa_thermometer_0 = 0xf2cb,
	fa_thermometer_empty = 0xf2cb,
	fa_temperature_full = 0xf2c7,
	fa_temperature_4 = 0xf2c7,
	fa_thermometer_4 = 0xf2c7,
	fa_thermometer_full = 0xf2c7,
	fa_temperature_half = 0xf2c9,
	fa_temperature_2 = 0xf2c9,
	fa_thermometer_2 = 0xf2c9,
	fa_thermometer_half = 0xf2c9,
	fa_temperature_high = 0xf769,
	fa_temperature_low = 0xf76b,
	fa_temperature_quarter = 0xf2ca,
	fa_temperature_1 = 0xf2ca,
	fa_thermometer_1 = 0xf2ca,
	fa_thermometer_quarter = 0xf2ca,
	fa_temperature_three_quarters = 0xf2c8,
	fa_temperature_3 = 0xf2c8,
	fa_thermometer_3 = 0xf2c8,
	fa_thermometer_three_quarters = 0xf2c8,
	fa_tenge_sign = 0xf7d7,
	fa_tenge = 0xf7d7,
	fa_tent = 0xe57d,
	fa_tent_arrow_down_to_line = 0xe57e,
	fa_tent_arrow_left_right = 0xe57f,
	fa_tent_arrow_turn_left = 0xe580,
	fa_tent_arrows_down = 0xe581,
	fa_tents = 0xe582,
	fa_terminal = 0xf120,
	fa_text_height = 0xf034,
	fa_text_slash = 0xf87d,
	fa_remove_format = 0xf87d,
	fa_text_width = 0xf035,
	fa_thermometer = 0xf491,
	fa_thumbs_down = 0xf165,
	fa_thumbs_up = 0xf164,
	fa_thumbtack = 0xf08d,
	fa_thumb_tack = 0xf08d,
	fa_ticket = 0xf145,
	fa_ticket_simple = 0xf3ff,
	fa_ticket_alt = 0xf3ff,
	fa_timeline = 0xe29c,
	fa_toggle_off = 0xf204,
	fa_toggle_on = 0xf205,
	fa_toilet = 0xf7d8,
	fa_toilet_paper = 0xf71e,
	fa_toilet_paper_slash = 0xe072,
	fa_toilet_portable = 0xe583,
	fa_toilets_portable = 0xe584,
	fa_toolbox = 0xf552,
	fa_tooth = 0xf5c9,
	fa_torii_gate = 0xf6a1,
	fa_tornado = 0xf76f,
	fa_tower_broadcast = 0xf519,
	fa_broadcast_tower = 0xf519,
	fa_tower_cell = 0xe585,
	fa_tower_observation = 0xe586,
	fa_tractor = 0xf722,
	fa_trademark = 0xf25c,
	fa_traffic_light = 0xf637,
	fa_trailer = 0xe041,
	fa_train = 0xf238,
	fa_train_subway = 0xf239,
	fa_subway = 0xf239,
	fa_train_tram = 0xe5b4,
	fa_transgender = 0xf225,
	fa_transgender_alt = 0xf225,
	fa_trash = 0xf1f8,
	fa_trash_arrow_up = 0xf829,
	fa_trash_restore = 0xf829,
	fa_trash_can = 0xf2ed,
	fa_trash_alt = 0xf2ed,
	fa_trash_can_arrow_up = 0xf82a,
	fa_trash_restore_alt = 0xf82a,
	fa_tree = 0xf1bb,
	fa_tree_city = 0xe587,
	fa_triangle_exclamation = 0xf071,
	fa_exclamation_triangle = 0xf071,
	fa_warning = 0xf071,
	fa_trophy = 0xf091,
	fa_trowel = 0xe589,
	fa_trowel_bricks = 0xe58a,
	fa_truck = 0xf0d1,
	fa_truck_arrow_right = 0xe58b,
	fa_truck_droplet = 0xe58c,
	fa_truck_fast = 0xf48b,
	fa_shipping_fast = 0xf48b,
	fa_truck_field = 0xe58d,
	fa_truck_field_un = 0xe58e,
	fa_truck_front = 0xe2b7,
	fa_truck_medical = 0xf0f9,
	fa_ambulance = 0xf0f9,
	fa_truck_monster = 0xf63b,
	fa_truck_moving = 0xf4df,
	fa_truck_pickup = 0xf63c,
	fa_truck_plane = 0xe58f,
	fa_truck_ramp_box = 0xf4de,
	fa_truck_loading = 0xf4de,
	fa_tty = 0xf1e4,
	fa_teletype = 0xf1e4,
	fa_turkish_lira_sign = 0xe2bb,
	fa_try = 0xe2bb,
	fa_turkish_lira = 0xe2bb,
	fa_turn_down = 0xf3be,
	fa_level_down_alt = 0xf3be,
	fa_turn_up = 0xf3bf,
	fa_level_up_alt = 0xf3bf,
	fa_tv = 0xf26c,
	fa_television = 0xf26c,
	fa_tv_alt = 0xf26c,
	fa_u = 0x55,
	fa_umbrella = 0xf0e9,
	fa_umbrella_beach = 0xf5ca,
	fa_underline = 0xf0cd,
	fa_universal_access = 0xf29a,
	fa_unlock = 0xf09c,
	fa_unlock_keyhole = 0xf13e,
	fa_unlock_alt = 0xf13e,
	fa_up_down = 0xf338,
	fa_arrows_alt_v = 0xf338,
	fa_up_down_left_right = 0xf0b2,
	fa_arrows_alt = 0xf0b2,
	fa_up_long = 0xf30c,
	fa_long_arrow_alt_up = 0xf30c,
	fa_up_right_and_down_left_from_center = 0xf424,
	fa_expand_alt = 0xf424,
	fa_up_right_from_square = 0xf35d,
	fa_external_link_alt = 0xf35d,
	fa_upload = 0xf093,
	fa_user = 0xf007,
	fa_user_astronaut = 0xf4fb,
	fa_user_check = 0xf4fc,
	fa_user_clock = 0xf4fd,
	fa_user_doctor = 0xf0f0,
	fa_user_md = 0xf0f0,
	fa_user_gear = 0xf4fe,
	fa_user_cog = 0xf4fe,
	fa_user_graduate = 0xf501,
	fa_user_group = 0xf500,
	fa_user_friends = 0xf500,
	fa_user_injured = 0xf728,
	fa_user_large = 0xf406,
	fa_user_alt = 0xf406,
	fa_user_large_slash = 0xf4fa,
	fa_user_alt_slash = 0xf4fa,
	fa_user_lock = 0xf502,
	fa_user_minus = 0xf503,
	fa_user_ninja = 0xf504,
	fa_user_nurse = 0xf82f,
	fa_user_pen = 0xf4ff,
	fa_user_edit = 0xf4ff,
	fa_user_plus = 0xf234,
	fa_user_secret = 0xf21b,
	fa_user_shield = 0xf505,
	fa_user_slash = 0xf506,
	fa_user_tag = 0xf507,
	fa_user_tie = 0xf508,
	fa_user_xmark = 0xf235,
	fa_user_times = 0xf235,
	fa_users = 0xf0c0,
	fa_users_between_lines = 0xe591,
	fa_users_gear = 0xf509,
	fa_users_cog = 0xf509,
	fa_users_line = 0xe592,
	fa_users_rays = 0xe593,
	fa_users_rectangle = 0xe594,
	fa_users_slash = 0xe073,
	fa_users_viewfinder = 0xe595,
	fa_utensils = 0xf2e7,
	fa_cutlery = 0xf2e7,
	fa_v = 0x56,
	fa_van_shuttle = 0xf5b6,
	fa_shuttle_van = 0xf5b6,
	fa_vault = 0xe2c5,
	fa_vector_square = 0xf5cb,
	fa_venus = 0xf221,
	fa_venus_double = 0xf226,
	fa_venus_mars = 0xf228,
	fa_vest = 0xe085,
	fa_vest_patches = 0xe086,
	fa_vial = 0xf492,
	fa_vial_circle_check = 0xe596,
	fa_vial_virus = 0xe597,
	fa_vials = 0xf493,
	fa_video = 0xf03d,
	fa_video_camera = 0xf03d,
	fa_video_slash = 0xf4e2,
	fa_vihara = 0xf6a7,
	fa_virus = 0xe074,
	fa_virus_covid = 0xe4a8,
	fa_virus_covid_slash = 0xe4a9,
	fa_virus_slash = 0xe075,
	fa_viruses = 0xe076,
	fa_voicemail = 0xf897,
	fa_volcano = 0xf770,
	fa_volleyball = 0xf45f,
	fa_volleyball_ball = 0xf45f,
	fa_volume_high = 0xf028,
	fa_volume_up = 0xf028,
	fa_volume_low = 0xf027,
	fa_volume_down = 0xf027,
	fa_volume_off = 0xf026,
	fa_volume_xmark = 0xf6a9,
	fa_volume_mute = 0xf6a9,
	fa_volume_times = 0xf6a9,
	fa_vr_cardboard = 0xf729,
	fa_w = 0x57,
	fa_walkie_talkie = 0xf8ef,
	fa_wallet = 0xf555,
	fa_wand_magic = 0xf0d0,
	fa_magic = 0xf0d0,
	fa_wand_magic_sparkles = 0xe2ca,
	fa_magic_wand_sparkles = 0xe2ca,
	fa_wand_sparkles = 0xf72b,
	fa_warehouse = 0xf494,
	fa_water = 0xf773,
	fa_water_ladder = 0xf5c5,
	fa_ladder_water = 0xf5c5,
	fa_swimming_pool = 0xf5c5,
	fa_wave_square = 0xf83e,
	fa_weight_hanging = 0xf5cd,
	fa_weight_scale = 0xf496,
	fa_weight = 0xf496,
	fa_wheat_awn = 0xe2cd,
	fa_wheat_alt = 0xe2cd,
	fa_wheat_awn_circle_exclamation = 0xe598,
	fa_wheelchair = 0xf193,
	fa_wheelchair_move = 0xe2ce,
	fa_wheelchair_alt = 0xe2ce,
	fa_whiskey_glass = 0xf7a0,
	fa_glass_whiskey = 0xf7a0,
	fa_wifi = 0xf1eb,
	fa_wifi_3 = 0xf1eb,
	fa_wifi_strong = 0xf1eb,
	fa_wind = 0xf72e,
	fa_window_maximize = 0xf2d0,
	fa_window_minimize = 0xf2d1,
	fa_window_restore = 0xf2d2,
	fa_wine_bottle = 0xf72f,
	fa_wine_glass = 0xf4e3,
	fa_wine_glass_empty = 0xf5ce,
	fa_wine_glass_alt = 0xf5ce,
	fa_won_sign = 0xf159,
	fa_krw = 0xf159,
	fa_won = 0xf159,
	fa_worm = 0xe599,
	fa_wrench = 0xf0ad,
	fa_x = 0x58,
	fa_x_ray = 0xf497,
	fa_xmark = 0xf00d,
	fa_close = 0xf00d,
	fa_multiply = 0xf00d,
	fa_remove = 0xf00d,
	fa_times = 0xf00d,
	fa_xmarks_lines = 0xe59a,
	fa_y = 0x59,
	fa_yen_sign = 0xf157,
	fa_cny = 0xf157,
	fa_jpy = 0xf157,
	fa_rmb = 0xf157,
	fa_yen = 0xf157,
	fa_yin_yang = 0xf6ad,
	fa_z = 0x5a

};

enum fa_brand_icons : uint16_t {
	fa_42_group = 0xe080,
	fa_innosoft = 0xe080,
	fa_500px = 0xf26e,
	fa_accessible_icon = 0xf368,
	fa_accusoft = 0xf369,
	fa_adn = 0xf170,
	fa_adversal = 0xf36a,
	fa_affiliatetheme = 0xf36b,
	fa_airbnb = 0xf834,
	fa_algolia = 0xf36c,
	fa_alipay = 0xf642,
	fa_amazon = 0xf270,
	fa_amazon_pay = 0xf42c,
	fa_amilia = 0xf36d,
	fa_android = 0xf17b,
	fa_angellist = 0xf209,
	fa_angrycreative = 0xf36e,
	fa_angular = 0xf420,
	fa_app_store = 0xf36f,
	fa_app_store_ios = 0xf370,
	fa_apper = 0xf371,
	fa_apple = 0xf179,
	fa_apple_pay = 0xf415,
	fa_artstation = 0xf77a,
	fa_asymmetrik = 0xf372,
	fa_atlassian = 0xf77b,
	fa_audible = 0xf373,
	fa_autoprefixer = 0xf41c,
	fa_avianex = 0xf374,
	fa_aviato = 0xf421,
	fa_aws = 0xf375,
	fa_bandcamp = 0xf2d5,
	fa_battle_net = 0xf835,
	fa_behance = 0xf1b4,
	fa_bilibili = 0xe3d9,
	fa_bimobject = 0xf378,
	fa_bitbucket = 0xf171,
	fa_bitcoin = 0xf379,
	fa_bity = 0xf37a,
	fa_black_tie = 0xf27e,
	fa_blackberry = 0xf37b,
	fa_blogger = 0xf37c,
	fa_blogger_b = 0xf37d,
	fa_bluetooth = 0xf293,
	fa_bluetooth_b = 0xf294,
	fa_bootstrap = 0xf836,
	fa_bots = 0xe340,
	fa_brave = 0xe63c,
	fa_brave_reverse = 0xe63d,
	fa_btc = 0xf15a,
	fa_buffer = 0xf837,
	fa_buromobelexperte = 0xf37f,
	fa_buy_n_large = 0xf8a6,
	fa_buysellads = 0xf20d,
	fa_canadian_maple_leaf = 0xf785,
	fa_cc_amazon_pay = 0xf42d,
	fa_cc_amex = 0xf1f3,
	fa_cc_apple_pay = 0xf416,
	fa_cc_diners_club = 0xf24c,
	fa_cc_discover = 0xf1f2,
	fa_cc_jcb = 0xf24b,
	fa_cc_mastercard = 0xf1f1,
	fa_cc_paypal = 0xf1f4,
	fa_cc_stripe = 0xf1f5,
	fa_cc_visa = 0xf1f0,
	fa_centercode = 0xf380,
	fa_centos = 0xf789,
	fa_chrome = 0xf268,
	fa_chromecast = 0xf838,
	fa_cloudflare = 0xe07d,
	fa_cloudscale = 0xf383,
	fa_cloudsmith = 0xf384,
	fa_cloudversify = 0xf385,
	fa_cmplid = 0xe360,
	fa_codepen = 0xf1cb,
	fa_codiepie = 0xf284,
	fa_confluence = 0xf78d,
	fa_connectdevelop = 0xf20e,
	fa_contao = 0xf26d,
	fa_cotton_bureau = 0xf89e,
	fa_cpanel = 0xf388,
	fa_creative_commons = 0xf25e,
	fa_creative_commons_by = 0xf4e7,
	fa_creative_commons_nc = 0xf4e8,
	fa_creative_commons_nc_eu = 0xf4e9,
	fa_creative_commons_nc_jp = 0xf4ea,
	fa_creative_commons_nd = 0xf4eb,
	fa_creative_commons_pd = 0xf4ec,
	fa_creative_commons_pd_alt = 0xf4ed,
	fa_creative_commons_remix = 0xf4ee,
	fa_creative_commons_sa = 0xf4ef,
	fa_creative_commons_sampling = 0xf4f0,
	fa_creative_commons_sampling_plus = 0xf4f1,
	fa_creative_commons_share = 0xf4f2,
	fa_creative_commons_zero = 0xf4f3,
	fa_critical_role = 0xf6c9,
	fa_css3 = 0xf13c,
	fa_css3_alt = 0xf38b,
	fa_cuttlefish = 0xf38c,
	fa_d_and_d = 0xf38d,
	fa_d_and_d_beyond = 0xf6ca,
	fa_dailymotion = 0xe052,
	fa_dashcube = 0xf210,
	fa_debian = 0xe60b,
	fa_deezer = 0xe077,
	fa_delicious = 0xf1a5,
	fa_deploydog = 0xf38e,
	fa_deskpro = 0xf38f,
	fa_dev = 0xf6cc,
	fa_deviantart = 0xf1bd,
	fa_dhl = 0xf790,
	fa_diaspora = 0xf791,
	fa_digg = 0xf1a6,
	fa_digital_ocean = 0xf391,
	fa_discord = 0xf392,
	fa_discourse = 0xf393,
	fa_dochub = 0xf394,
	fa_docker = 0xf395,
	fa_draft2digital = 0xf396,
	fa_dribbble = 0xf17d,
	fa_dropbox = 0xf16b,
	fa_drupal = 0xf1a9,
	fa_dyalog = 0xf399,
	fa_earlybirds = 0xf39a,
	fa_ebay = 0xf4f4,
	fa_edge = 0xf282,
	fa_edge_legacy = 0xe078,
	fa_elementor = 0xf430,
	fa_ello = 0xf5f1,
	fa_ember = 0xf423,
	fa_empire = 0xf1d1,
	fa_envira = 0xf299,
	fa_erlang = 0xf39d,
	fa_ethereum = 0xf42e,
	fa_etsy = 0xf2d7,
	fa_evernote = 0xf839,
	fa_expeditedssl = 0xf23e,
	fa_facebook = 0xf09a,
	fa_facebook_f = 0xf39e,
	fa_facebook_messenger = 0xf39f,
	fa_fantasy_flight_games = 0xf6dc,
	fa_fedex = 0xf797,
	fa_fedora = 0xf798,
	fa_figma = 0xf799,
	fa_firefox = 0xf269,
	fa_firefox_browser = 0xe007,
	fa_first_order = 0xf2b0,
	fa_first_order_alt = 0xf50a,
	fa_firstdraft = 0xf3a1,
	fa_flickr = 0xf16e,
	fa_flipboard = 0xf44d,
	fa_fly = 0xf417,
	fa_font_awesome = 0xf2b4,
	fa_font_awesome_flag = 0xf2b4,
	fa_font_awesome_logo_full = 0xf2b4,
	fa_fonticons = 0xf280,
	fa_fonticons_fi = 0xf3a2,
	fa_fort_awesome = 0xf286,
	fa_fort_awesome_alt = 0xf3a3,
	fa_forumbee = 0xf211,
	fa_foursquare = 0xf180,
	fa_free_code_camp = 0xf2c5,
	fa_freebsd = 0xf3a4,
	fa_fulcrum = 0xf50b,
	fa_galactic_republic = 0xf50c,
	fa_galactic_senate = 0xf50d,
	fa_get_pocket = 0xf265,
	fa_gg = 0xf260,
	fa_gg_circle = 0xf261,
	fa_git = 0xf1d3,
	fa_git_alt = 0xf841,
	fa_github = 0xf09b,
	fa_github_alt = 0xf113,
	fa_gitkraken = 0xf3a6,
	fa_gitlab = 0xf296,
	fa_gitter = 0xf426,
	fa_glide = 0xf2a5,
	fa_glide_g = 0xf2a6,
	fa_gofore = 0xf3a7,
	fa_golang = 0xe40f,
	fa_goodreads = 0xf3a8,
	fa_goodreads_g = 0xf3a9,
	fa_google = 0xf1a0,
	fa_google_drive = 0xf3aa,
	fa_google_pay = 0xe079,
	fa_google_play = 0xf3ab,
	fa_google_plus = 0xf2b3,
	fa_google_plus_g = 0xf0d5,
	fa_google_scholar = 0xe63b,
	fa_google_wallet = 0xf1ee,
	fa_gratipay = 0xf184,
	fa_grav = 0xf2d6,
	fa_gripfire = 0xf3ac,
	fa_grunt = 0xf3ad,
	fa_guilded = 0xe07e,
	fa_gulp = 0xf3ae,
	fa_hacker_news = 0xf1d4,
	fa_hackerrank = 0xf5f7,
	fa_hashnode = 0xe499,
	fa_hips = 0xf452,
	fa_hire_a_helper = 0xf3b0,
	fa_hive = 0xe07f,
	fa_hooli = 0xf427,
	fa_hornbill = 0xf592,
	fa_hotjar = 0xf3b1,
	fa_houzz = 0xf27c,
	fa_html5 = 0xf13b,
	fa_hubspot = 0xf3b2,
	fa_ideal = 0xe013,
	fa_imdb = 0xf2d8,
	fa_instagram = 0xf16d,
	fa_instalod = 0xe081,
	fa_intercom = 0xf7af,
	fa_internet_explorer = 0xf26b,
	fa_invision = 0xf7b0,
	fa_ioxhost = 0xf208,
	fa_itch_io = 0xf83a,
	fa_itunes = 0xf3b4,
	fa_itunes_note = 0xf3b5,
	fa_java = 0xf4e4,
	fa_jedi_order = 0xf50e,
	fa_jenkins = 0xf3b6,
	fa_jira = 0xf7b1,
	fa_joget = 0xf3b7,
	fa_joomla = 0xf1aa,
	fa_js = 0xf3b8,
	fa_jsfiddle = 0xf1cc,
	fa_kaggle = 0xf5fa,
	fa_keybase = 0xf4f5,
	fa_keycdn = 0xf3ba,
	fa_kickstarter = 0xf3bb,
	fa_kickstarter_k = 0xf3bc,
	fa_korvue = 0xf42f,
	fa_laravel = 0xf3bd,
	fa_lastfm = 0xf202,
	fa_leanpub = 0xf212,
	fa_less = 0xf41d,
	fa_letterboxd = 0xe62d,
	fa_line = 0xf3c0,
	fa_linkedin = 0xf08c,
	fa_linkedin_in = 0xf0e1,
	fa_linode = 0xf2b8,
	fa_linux = 0xf17c,
	fa_lyft = 0xf3c3,
	fa_magento = 0xf3c4,
	fa_mailchimp = 0xf59e,
	fa_mandalorian = 0xf50f,
	fa_markdown = 0xf60f,
	fa_mastodon = 0xf4f6,
	fa_maxcdn = 0xf136,
	fa_mdb = 0xf8ca,
	fa_medapps = 0xf3c6,
	fa_medium = 0xf23a,
	fa_medium_m = 0xf23a,
	fa_medrt = 0xf3c8,
	fa_meetup = 0xf2e0,
	fa_megaport = 0xf5a3,
	fa_mendeley = 0xf7b3,
	fa_meta = 0xe49b,
	fa_microblog = 0xe01a,
	fa_microsoft = 0xf3ca,
	fa_mintbit = 0xe62f,
	fa_mix = 0xf3cb,
	fa_mixcloud = 0xf289,
	fa_mixer = 0xe056,
	fa_mizuni = 0xf3cc,
	fa_modx = 0xf285,
	fa_monero = 0xf3d0,
	fa_napster = 0xf3d2,
	fa_neos = 0xf612,
	fa_nfc_directional = 0xe530,
	fa_nfc_symbol = 0xe531,
	fa_nimblr = 0xf5a8,
	fa_node = 0xf419,
	fa_node_js = 0xf3d3,
	fa_npm = 0xf3d4,
	fa_ns8 = 0xf3d5,
	fa_nutritionix = 0xf3d6,
	fa_octopus_deploy = 0xe082,
	fa_odnoklassniki = 0xf263,
	fa_odysee = 0xe5c6,
	fa_old_republic = 0xf510,
	fa_opencart = 0xf23d,
	fa_openid = 0xf19b,
	fa_opensuse = 0xe62b,
	fa_opera = 0xf26a,
	fa_optin_monster = 0xf23c,
	fa_orcid = 0xf8d2,
	fa_osi = 0xf41a,
	fa_padlet = 0xe4a0,
	fa_page4 = 0xf3d7,
	fa_pagelines = 0xf18c,
	fa_palfed = 0xf3d8,
	fa_patreon = 0xf3d9,
	fa_paypal = 0xf1ed,
	fa_perbyte = 0xe083,
	fa_periscope = 0xf3da,
	fa_phabricator = 0xf3db,
	fa_phoenix_framework = 0xf3dc,
	fa_phoenix_squadron = 0xf511,
	fa_php = 0xf457,
	fa_pied_piper = 0xf2ae,
	fa_pied_piper_alt = 0xf1a8,
	fa_pied_piper_hat = 0xf4e5,
	fa_pied_piper_pp = 0xf1a7,
	fa_pinterest = 0xf0d2,
	fa_pinterest_p = 0xf231,
	fa_pix = 0xe43a,
	fa_pixiv = 0xe640,
	fa_playstation = 0xf3df,
	fa_product_hunt = 0xf288,
	fa_pushed = 0xf3e1,
	fa_python = 0xf3e2,
	fa_qq = 0xf1d6,
	fa_quinscape = 0xf459,
	fa_quora = 0xf2c4,
	fa_r_project = 0xf4f7,
	fa_raspberry_pi = 0xf7bb,
	fa_ravelry = 0xf2d9,
	fa_react = 0xf41b,
	fa_reacteurope = 0xf75d,
	fa_readme = 0xf4d5,
	fa_rebel = 0xf1d0,
	fa_red_river = 0xf3e3,
	fa_reddit = 0xf1a1,
	fa_reddit_alien = 0xf281,
	fa_redhat = 0xf7bc,
	fa_renren = 0xf18b,
	fa_replyd = 0xf3e6,
	fa_researchgate = 0xf4f8,
	fa_resolving = 0xf3e7,
	fa_rev = 0xf5b2,
	fa_rocketchat = 0xf3e8,
	fa_rockrms = 0xf3e9,
	fa_rust = 0xe07a,
	fa_safari = 0xf267,
	fa_salesforce = 0xf83b,
	fa_sass = 0xf41e,
	fa_schlix = 0xf3ea,
	fa_screenpal = 0xe570,
	fa_scribd = 0xf28a,
	fa_searchengin = 0xf3eb,
	fa_sellcast = 0xf2da,
	fa_sellsy = 0xf213,
	fa_servicestack = 0xf3ec,
	fa_shirtsinbulk = 0xf214,
	fa_shoelace = 0xe60c,
	fa_shopify = 0xe057,
	fa_shopware = 0xf5b5,
	fa_signal_messenger = 0xe663,
	fa_simplybuilt = 0xf215,
	fa_sistrix = 0xf3ee,
	fa_sith = 0xf512,
	fa_sitrox = 0xe44a,
	fa_sketch = 0xf7c6,
	fa_skyatlas = 0xf216,
	fa_skype = 0xf17e,
	fa_slack = 0xf198,
	fa_slack_hash = 0xf198,
	fa_slideshare = 0xf1e7,
	fa_snapchat = 0xf2ab,
	fa_snapchat_ghost = 0xf2ab,
	fa_soundcloud = 0xf1be,
	fa_sourcetree = 0xf7d3,
	fa_space_awesome = 0xe5ac,
	fa_speakap = 0xf3f3,
	fa_speaker_deck = 0xf83c,
	fa_spotify = 0xf1bc,
	fa_square_behance = 0xf1b5,
	fa_behance_square = 0xf1b5,
	fa_square_dribbble = 0xf397,
	fa_dribbble_square = 0xf397,
	fa_square_facebook = 0xf082,
	fa_facebook_square = 0xf082,
	fa_square_font_awesome = 0xe5ad,
	fa_square_font_awesome_stroke = 0xf35c,
	fa_font_awesome_alt = 0xf35c,
	fa_square_git = 0xf1d2,
	fa_git_square = 0xf1d2,
	fa_square_github = 0xf092,
	fa_github_square = 0xf092,
	fa_square_gitlab = 0xe5ae,
	fa_gitlab_square = 0xe5ae,
	fa_square_google_plus = 0xf0d4,
	fa_google_plus_square = 0xf0d4,
	fa_square_hacker_news = 0xf3af,
	fa_hacker_news_square = 0xf3af,
	fa_square_instagram = 0xe055,
	fa_instagram_square = 0xe055,
	fa_square_js = 0xf3b9,
	fa_js_square = 0xf3b9,
	fa_square_lastfm = 0xf203,
	fa_lastfm_square = 0xf203,
	fa_square_letterboxd = 0xe62e,
	fa_square_odnoklassniki = 0xf264,
	fa_odnoklassniki_square = 0xf264,
	fa_square_pied_piper = 0xe01e,
	fa_pied_piper_square = 0xe01e,
	fa_square_pinterest = 0xf0d3,
	fa_pinterest_square = 0xf0d3,
	fa_square_reddit = 0xf1a2,
	fa_reddit_square = 0xf1a2,
	fa_square_snapchat = 0xf2ad,
	fa_snapchat_square = 0xf2ad,
	fa_square_steam = 0xf1b7,
	fa_steam_square = 0xf1b7,
	fa_square_threads = 0xe619,
	fa_square_tumblr = 0xf174,
	fa_tumblr_square = 0xf174,
	fa_square_twitter = 0xf081,
	fa_twitter_square = 0xf081,
	fa_square_viadeo = 0xf2aa,
	fa_viadeo_square = 0xf2aa,
	fa_square_vimeo = 0xf194,
	fa_vimeo_square = 0xf194,
	fa_square_whatsapp = 0xf40c,
	fa_whatsapp_square = 0xf40c,
	fa_square_x_twitter = 0xe61a,
	fa_square_xing = 0xf169,
	fa_xing_square = 0xf169,
	fa_square_youtube = 0xf431,
	fa_youtube_square = 0xf431,
	fa_squarespace = 0xf5be,
	fa_stack_exchange = 0xf18d,
	fa_stack_overflow = 0xf16c,
	fa_stackpath = 0xf842,
	fa_staylinked = 0xf3f5,
	fa_steam = 0xf1b6,
	fa_steam_symbol = 0xf3f6,
	fa_sticker_mule = 0xf3f7,
	fa_strava = 0xf428,
	fa_stripe = 0xf429,
	fa_stripe_s = 0xf42a,
	fa_stubber = 0xe5c7,
	fa_studiovinari = 0xf3f8,
	fa_stumbleupon = 0xf1a4,
	fa_stumbleupon_circle = 0xf1a3,
	fa_superpowers = 0xf2dd,
	fa_supple = 0xf3f9,
	fa_suse = 0xf7d6,
	fa_swift = 0xf8e1,
	fa_symfony = 0xf83d,
	fa_teamspeak = 0xf4f9,
	fa_telegram = 0xf2c6,
	fa_telegram_plane = 0xf2c6,
	fa_tencent_weibo = 0xf1d5,
	fa_the_red_yeti = 0xf69d,
	fa_themeco = 0xf5c6,
	fa_themeisle = 0xf2b2,
	fa_think_peaks = 0xf731,
	fa_threads = 0xe618,
	fa_tiktok = 0xe07b,
	fa_trade_federation = 0xf513,
	fa_trello = 0xf181,
	fa_tumblr = 0xf173,
	fa_twitch = 0xf1e8,
	fa_twitter = 0xf099,
	fa_typo3 = 0xf42b,
	fa_uber = 0xf402,
	fa_ubuntu = 0xf7df,
	fa_uikit = 0xf403,
	fa_umbraco = 0xf8e8,
	fa_uncharted = 0xe084,
	fa_uniregistry = 0xf404,
	fa_unity = 0xe049,
	fa_unsplash = 0xe07c,
	fa_untappd = 0xf405,
	fa_ups = 0xf7e0,
	fa_upwork = 0xe641,
	fa_usb = 0xf287,
	fa_usps = 0xf7e1,
	fa_ussunnah = 0xf407,
	fa_vaadin = 0xf408,
	fa_viacoin = 0xf237,
	fa_viadeo = 0xf2a9,
	fa_viber = 0xf409,
	fa_vimeo = 0xf40a,
	fa_vimeo_v = 0xf27d,
	fa_vine = 0xf1ca,
	fa_vk = 0xf189,
	fa_vnv = 0xf40b,
	fa_vuejs = 0xf41f,
	fa_watchman_monitoring = 0xe087,
	fa_waze = 0xf83f,
	fa_webflow = 0xe65c,
	fa_weebly = 0xf5cc,
	fa_weibo = 0xf18a,
	fa_weixin = 0xf1d7,
	fa_whatsapp = 0xf232,
	fa_whmcs = 0xf40d,
	fa_wikipedia_w = 0xf266,
	fa_windows = 0xf17a,
	fa_wirsindhandwerk = 0xe2d0,
	fa_wsh = 0xe2d0,
	fa_wix = 0xf5cf,
	fa_wizards_of_the_coast = 0xf730,
	fa_wodu = 0xe088,
	fa_wolf_pack_battalion = 0xf514,
	fa_wordpress = 0xf19a,
	fa_wordpress_simple = 0xf411,
	fa_wpbeginner = 0xf297,
	fa_wpexplorer = 0xf2de,
	fa_wpforms = 0xf298,
	fa_wpressr = 0xf3e4,
	fa_rendact = 0xf3e4,
	fa_x_twitter = 0xe61b,
	fa_xbox = 0xf412,
	fa_xing = 0xf168,
	fa_y_combinator = 0xf23b,
	fa_yahoo = 0xf19e,
	fa_yammer = 0xf840,
	fa_yandex = 0xf413,
	fa_yandex_international = 0xf414,
	fa_yarn = 0xf7e3,
	fa_yelp = 0xf1e9,
	fa_yoast = 0xf2b1,
	fa_youtube = 0xf167,
	fa_zhihu = 0xf63f

};
}// namespace fa
