static const fa::QtAwesomeNamedIcon faCommonIconArray[] = {
		{"0", fa::fa_0},
		{"1", fa::fa_1},
		{"2", fa::fa_2},
		{"3", fa::fa_3},
		{"4", fa::fa_4},
		{"5", fa::fa_5},
		{"6", fa::fa_6},
		{"7", fa::fa_7},
		{"8", fa::fa_8},
		{"9", fa::fa_9},
		{"a", fa::fa_a},
		{"address-book", fa::fa_address_book},
		{"contact-book", fa::fa_contact_book},
		{"address-card", fa::fa_address_card},
		{"contact-card", fa::fa_contact_card},
		{"vcard", fa::fa_vcard},
		{"align-center", fa::fa_align_center},
		{"align-justify", fa::fa_align_justify},
		{"align-left", fa::fa_align_left},
		{"align-right", fa::fa_align_right},
		{"anchor", fa::fa_anchor},
		{"anchor-circle-check", fa::fa_anchor_circle_check},
		{"anchor-circle-exclamation", fa::fa_anchor_circle_exclamation},
		{"anchor-circle-xmark", fa::fa_anchor_circle_xmark},
		{"anchor-lock", fa::fa_anchor_lock},
		{"angle-down", fa::fa_angle_down},
		{"angle-left", fa::fa_angle_left},
		{"angle-right", fa::fa_angle_right},
		{"angle-up", fa::fa_angle_up},
		{"angles-down", fa::fa_angles_down},
		{"angle-double-down", fa::fa_angle_double_down},
		{"angles-left", fa::fa_angles_left},
		{"angle-double-left", fa::fa_angle_double_left},
		{"angles-right", fa::fa_angles_right},
		{"angle-double-right", fa::fa_angle_double_right},
		{"angles-up", fa::fa_angles_up},
		{"angle-double-up", fa::fa_angle_double_up},
		{"ankh", fa::fa_ankh},
		{"apple-whole", fa::fa_apple_whole},
		{"apple-alt", fa::fa_apple_alt},
		{"archway", fa::fa_archway},
		{"arrow-down", fa::fa_arrow_down},
		{"arrow-down-1-9", fa::fa_arrow_down_1_9},
		{"sort-numeric-asc", fa::fa_sort_numeric_asc},
		{"sort-numeric-down", fa::fa_sort_numeric_down},
		{"arrow-down-9-1", fa::fa_arrow_down_9_1},
		{"sort-numeric-desc", fa::fa_sort_numeric_desc},
		{"sort-numeric-down-alt", fa::fa_sort_numeric_down_alt},
		{"arrow-down-a-z", fa::fa_arrow_down_a_z},
		{"sort-alpha-asc", fa::fa_sort_alpha_asc},
		{"sort-alpha-down", fa::fa_sort_alpha_down},
		{"arrow-down-long", fa::fa_arrow_down_long},
		{"long-arrow-down", fa::fa_long_arrow_down},
		{"arrow-down-short-wide", fa::fa_arrow_down_short_wide},
		{"sort-amount-desc", fa::fa_sort_amount_desc},
		{"sort-amount-down-alt", fa::fa_sort_amount_down_alt},
		{"arrow-down-up-across-line", fa::fa_arrow_down_up_across_line},
		{"arrow-down-up-lock", fa::fa_arrow_down_up_lock},
		{"arrow-down-wide-short", fa::fa_arrow_down_wide_short},
		{"sort-amount-asc", fa::fa_sort_amount_asc},
		{"sort-amount-down", fa::fa_sort_amount_down},
		{"arrow-down-z-a", fa::fa_arrow_down_z_a},
		{"sort-alpha-desc", fa::fa_sort_alpha_desc},
		{"sort-alpha-down-alt", fa::fa_sort_alpha_down_alt},
		{"arrow-left", fa::fa_arrow_left},
		{"arrow-left-long", fa::fa_arrow_left_long},
		{"long-arrow-left", fa::fa_long_arrow_left},
		{"arrow-pointer", fa::fa_arrow_pointer},
		{"mouse-pointer", fa::fa_mouse_pointer},
		{"arrow-right", fa::fa_arrow_right},
		{"arrow-right-arrow-left", fa::fa_arrow_right_arrow_left},
		{"exchange", fa::fa_exchange},
		{"arrow-right-from-bracket", fa::fa_arrow_right_from_bracket},
		{"sign-out", fa::fa_sign_out},
		{"arrow-right-long", fa::fa_arrow_right_long},
		{"long-arrow-right", fa::fa_long_arrow_right},
		{"arrow-right-to-bracket", fa::fa_arrow_right_to_bracket},
		{"sign-in", fa::fa_sign_in},
		{"arrow-right-to-city", fa::fa_arrow_right_to_city},
		{"arrow-rotate-left", fa::fa_arrow_rotate_left},
		{"arrow-left-rotate", fa::fa_arrow_left_rotate},
		{"arrow-rotate-back", fa::fa_arrow_rotate_back},
		{"arrow-rotate-backward", fa::fa_arrow_rotate_backward},
		{"undo", fa::fa_undo},
		{"arrow-rotate-right", fa::fa_arrow_rotate_right},
		{"arrow-right-rotate", fa::fa_arrow_right_rotate},
		{"arrow-rotate-forward", fa::fa_arrow_rotate_forward},
		{"redo", fa::fa_redo},
		{"arrow-trend-down", fa::fa_arrow_trend_down},
		{"arrow-trend-up", fa::fa_arrow_trend_up},
		{"arrow-turn-down", fa::fa_arrow_turn_down},
		{"level-down", fa::fa_level_down},
		{"arrow-turn-up", fa::fa_arrow_turn_up},
		{"level-up", fa::fa_level_up},
		{"arrow-up", fa::fa_arrow_up},
		{"arrow-up-1-9", fa::fa_arrow_up_1_9},
		{"sort-numeric-up", fa::fa_sort_numeric_up},
		{"arrow-up-9-1", fa::fa_arrow_up_9_1},
		{"sort-numeric-up-alt", fa::fa_sort_numeric_up_alt},
		{"arrow-up-a-z", fa::fa_arrow_up_a_z},
		{"sort-alpha-up", fa::fa_sort_alpha_up},
		{"arrow-up-from-bracket", fa::fa_arrow_up_from_bracket},
		{"arrow-up-from-ground-water", fa::fa_arrow_up_from_ground_water},
		{"arrow-up-from-water-pump", fa::fa_arrow_up_from_water_pump},
		{"arrow-up-long", fa::fa_arrow_up_long},
		{"long-arrow-up", fa::fa_long_arrow_up},
		{"arrow-up-right-dots", fa::fa_arrow_up_right_dots},
		{"arrow-up-right-from-square", fa::fa_arrow_up_right_from_square},
		{"external-link", fa::fa_external_link},
		{"arrow-up-short-wide", fa::fa_arrow_up_short_wide},
		{"sort-amount-up-alt", fa::fa_sort_amount_up_alt},
		{"arrow-up-wide-short", fa::fa_arrow_up_wide_short},
		{"sort-amount-up", fa::fa_sort_amount_up},
		{"arrow-up-z-a", fa::fa_arrow_up_z_a},
		{"sort-alpha-up-alt", fa::fa_sort_alpha_up_alt},
		{"arrows-down-to-line", fa::fa_arrows_down_to_line},
		{"arrows-down-to-people", fa::fa_arrows_down_to_people},
		{"arrows-left-right", fa::fa_arrows_left_right},
		{"arrows-h", fa::fa_arrows_h},
		{"arrows-left-right-to-line", fa::fa_arrows_left_right_to_line},
		{"arrows-rotate", fa::fa_arrows_rotate},
		{"refresh", fa::fa_refresh},
		{"sync", fa::fa_sync},
		{"arrows-spin", fa::fa_arrows_spin},
		{"arrows-split-up-and-left", fa::fa_arrows_split_up_and_left},
		{"arrows-to-circle", fa::fa_arrows_to_circle},
		{"arrows-to-dot", fa::fa_arrows_to_dot},
		{"arrows-to-eye", fa::fa_arrows_to_eye},
		{"arrows-turn-right", fa::fa_arrows_turn_right},
		{"arrows-turn-to-dots", fa::fa_arrows_turn_to_dots},
		{"arrows-up-down", fa::fa_arrows_up_down},
		{"arrows-v", fa::fa_arrows_v},
		{"arrows-up-down-left-right", fa::fa_arrows_up_down_left_right},
		{"arrows", fa::fa_arrows},
		{"arrows-up-to-line", fa::fa_arrows_up_to_line},
		{"asterisk", fa::fa_asterisk},
		{"at", fa::fa_at},
		{"atom", fa::fa_atom},
		{"audio-description", fa::fa_audio_description},
		{"austral-sign", fa::fa_austral_sign},
		{"award", fa::fa_award},
		{"b", fa::fa_b},
		{"baby", fa::fa_baby},
		{"baby-carriage", fa::fa_baby_carriage},
		{"carriage-baby", fa::fa_carriage_baby},
		{"backward", fa::fa_backward},
		{"backward-fast", fa::fa_backward_fast},
		{"fast-backward", fa::fa_fast_backward},
		{"backward-step", fa::fa_backward_step},
		{"step-backward", fa::fa_step_backward},
		{"bacon", fa::fa_bacon},
		{"bacteria", fa::fa_bacteria},
		{"bacterium", fa::fa_bacterium},
		{"bag-shopping", fa::fa_bag_shopping},
		{"shopping-bag", fa::fa_shopping_bag},
		{"bahai", fa::fa_bahai},
		{"haykal", fa::fa_haykal},
		{"baht-sign", fa::fa_baht_sign},
		{"ban", fa::fa_ban},
		{"cancel", fa::fa_cancel},
		{"ban-smoking", fa::fa_ban_smoking},
		{"smoking-ban", fa::fa_smoking_ban},
		{"bandage", fa::fa_bandage},
		{"band-aid", fa::fa_band_aid},
		{"bangladeshi-taka-sign", fa::fa_bangladeshi_taka_sign},
		{"barcode", fa::fa_barcode},
		{"bars", fa::fa_bars},
		{"navicon", fa::fa_navicon},
		{"bars-progress", fa::fa_bars_progress},
		{"tasks-alt", fa::fa_tasks_alt},
		{"bars-staggered", fa::fa_bars_staggered},
		{"reorder", fa::fa_reorder},
		{"stream", fa::fa_stream},
		{"baseball", fa::fa_baseball},
		{"baseball-ball", fa::fa_baseball_ball},
		{"baseball-bat-ball", fa::fa_baseball_bat_ball},
		{"basket-shopping", fa::fa_basket_shopping},
		{"shopping-basket", fa::fa_shopping_basket},
		{"basketball", fa::fa_basketball},
		{"basketball-ball", fa::fa_basketball_ball},
		{"bath", fa::fa_bath},
		{"bathtub", fa::fa_bathtub},
		{"battery-empty", fa::fa_battery_empty},
		{"battery-0", fa::fa_battery_0},
		{"battery-full", fa::fa_battery_full},
		{"battery", fa::fa_battery},
		{"battery-5", fa::fa_battery_5},
		{"battery-half", fa::fa_battery_half},
		{"battery-3", fa::fa_battery_3},
		{"battery-quarter", fa::fa_battery_quarter},
		{"battery-2", fa::fa_battery_2},
		{"battery-three-quarters", fa::fa_battery_three_quarters},
		{"battery-4", fa::fa_battery_4},
		{"bed", fa::fa_bed},
		{"bed-pulse", fa::fa_bed_pulse},
		{"procedures", fa::fa_procedures},
		{"beer-mug-empty", fa::fa_beer_mug_empty},
		{"beer", fa::fa_beer},
		{"bell", fa::fa_bell},
		{"bell-concierge", fa::fa_bell_concierge},
		{"concierge-bell", fa::fa_concierge_bell},
		{"bell-slash", fa::fa_bell_slash},
		{"bezier-curve", fa::fa_bezier_curve},
		{"bicycle", fa::fa_bicycle},
		{"binoculars", fa::fa_binoculars},
		{"biohazard", fa::fa_biohazard},
		{"bitcoin-sign", fa::fa_bitcoin_sign},
		{"blender", fa::fa_blender},
		{"blender-phone", fa::fa_blender_phone},
		{"blog", fa::fa_blog},
		{"bold", fa::fa_bold},
		{"bolt", fa::fa_bolt},
		{"zap", fa::fa_zap},
		{"bolt-lightning", fa::fa_bolt_lightning},
		{"bomb", fa::fa_bomb},
		{"bone", fa::fa_bone},
		{"bong", fa::fa_bong},
		{"book", fa::fa_book},
		{"book-atlas", fa::fa_book_atlas},
		{"atlas", fa::fa_atlas},
		{"book-bible", fa::fa_book_bible},
		{"bible", fa::fa_bible},
		{"book-bookmark", fa::fa_book_bookmark},
		{"book-journal-whills", fa::fa_book_journal_whills},
		{"journal-whills", fa::fa_journal_whills},
		{"book-medical", fa::fa_book_medical},
		{"book-open", fa::fa_book_open},
		{"book-open-reader", fa::fa_book_open_reader},
		{"book-reader", fa::fa_book_reader},
		{"book-quran", fa::fa_book_quran},
		{"quran", fa::fa_quran},
		{"book-skull", fa::fa_book_skull},
		{"book-dead", fa::fa_book_dead},
		{"book-tanakh", fa::fa_book_tanakh},
		{"tanakh", fa::fa_tanakh},
		{"bookmark", fa::fa_bookmark},
		{"border-all", fa::fa_border_all},
		{"border-none", fa::fa_border_none},
		{"border-top-left", fa::fa_border_top_left},
		{"border-style", fa::fa_border_style},
		{"bore-hole", fa::fa_bore_hole},
		{"bottle-droplet", fa::fa_bottle_droplet},
		{"bottle-water", fa::fa_bottle_water},
		{"bowl-food", fa::fa_bowl_food},
		{"bowl-rice", fa::fa_bowl_rice},
		{"bowling-ball", fa::fa_bowling_ball},
		{"box", fa::fa_box},
		{"box-archive", fa::fa_box_archive},
		{"archive", fa::fa_archive},
		{"box-open", fa::fa_box_open},
		{"box-tissue", fa::fa_box_tissue},
		{"boxes-packing", fa::fa_boxes_packing},
		{"boxes-stacked", fa::fa_boxes_stacked},
		{"boxes", fa::fa_boxes},
		{"boxes-alt", fa::fa_boxes_alt},
		{"braille", fa::fa_braille},
		{"brain", fa::fa_brain},
		{"brazilian-real-sign", fa::fa_brazilian_real_sign},
		{"bread-slice", fa::fa_bread_slice},
		{"bridge", fa::fa_bridge},
		{"bridge-circle-check", fa::fa_bridge_circle_check},
		{"bridge-circle-exclamation", fa::fa_bridge_circle_exclamation},
		{"bridge-circle-xmark", fa::fa_bridge_circle_xmark},
		{"bridge-lock", fa::fa_bridge_lock},
		{"bridge-water", fa::fa_bridge_water},
		{"briefcase", fa::fa_briefcase},
		{"briefcase-medical", fa::fa_briefcase_medical},
		{"broom", fa::fa_broom},
		{"broom-ball", fa::fa_broom_ball},
		{"quidditch", fa::fa_quidditch},
		{"quidditch-broom-ball", fa::fa_quidditch_broom_ball},
		{"brush", fa::fa_brush},
		{"bucket", fa::fa_bucket},
		{"bug", fa::fa_bug},
		{"bug-slash", fa::fa_bug_slash},
		{"bugs", fa::fa_bugs},
		{"building", fa::fa_building},
		{"building-circle-arrow-right", fa::fa_building_circle_arrow_right},
		{"building-circle-check", fa::fa_building_circle_check},
		{"building-circle-exclamation", fa::fa_building_circle_exclamation},
		{"building-circle-xmark", fa::fa_building_circle_xmark},
		{"building-columns", fa::fa_building_columns},
		{"bank", fa::fa_bank},
		{"institution", fa::fa_institution},
		{"museum", fa::fa_museum},
		{"university", fa::fa_university},
		{"building-flag", fa::fa_building_flag},
		{"building-lock", fa::fa_building_lock},
		{"building-ngo", fa::fa_building_ngo},
		{"building-shield", fa::fa_building_shield},
		{"building-un", fa::fa_building_un},
		{"building-user", fa::fa_building_user},
		{"building-wheat", fa::fa_building_wheat},
		{"bullhorn", fa::fa_bullhorn},
		{"bullseye", fa::fa_bullseye},
		{"burger", fa::fa_burger},
		{"hamburger", fa::fa_hamburger},
		{"burst", fa::fa_burst},
		{"bus", fa::fa_bus},
		{"bus-simple", fa::fa_bus_simple},
		{"bus-alt", fa::fa_bus_alt},
		{"business-time", fa::fa_business_time},
		{"briefcase-clock", fa::fa_briefcase_clock},
		{"c", fa::fa_c},
		{"cable-car", fa::fa_cable_car},
		{"tram", fa::fa_tram},
		{"cake-candles", fa::fa_cake_candles},
		{"birthday-cake", fa::fa_birthday_cake},
		{"cake", fa::fa_cake},
		{"calculator", fa::fa_calculator},
		{"calendar", fa::fa_calendar},
		{"calendar-check", fa::fa_calendar_check},
		{"calendar-day", fa::fa_calendar_day},
		{"calendar-days", fa::fa_calendar_days},
		{"calendar-alt", fa::fa_calendar_alt},
		{"calendar-minus", fa::fa_calendar_minus},
		{"calendar-plus", fa::fa_calendar_plus},
		{"calendar-week", fa::fa_calendar_week},
		{"calendar-xmark", fa::fa_calendar_xmark},
		{"calendar-times", fa::fa_calendar_times},
		{"camera", fa::fa_camera},
		{"camera-alt", fa::fa_camera_alt},
		{"camera-retro", fa::fa_camera_retro},
		{"camera-rotate", fa::fa_camera_rotate},
		{"campground", fa::fa_campground},
		{"candy-cane", fa::fa_candy_cane},
		{"cannabis", fa::fa_cannabis},
		{"capsules", fa::fa_capsules},
		{"car", fa::fa_car},
		{"automobile", fa::fa_automobile},
		{"car-battery", fa::fa_car_battery},
		{"battery-car", fa::fa_battery_car},
		{"car-burst", fa::fa_car_burst},
		{"car-crash", fa::fa_car_crash},
		{"car-on", fa::fa_car_on},
		{"car-rear", fa::fa_car_rear},
		{"car-alt", fa::fa_car_alt},
		{"car-side", fa::fa_car_side},
		{"car-tunnel", fa::fa_car_tunnel},
		{"caravan", fa::fa_caravan},
		{"caret-down", fa::fa_caret_down},
		{"caret-left", fa::fa_caret_left},
		{"caret-right", fa::fa_caret_right},
		{"caret-up", fa::fa_caret_up},
		{"carrot", fa::fa_carrot},
		{"cart-arrow-down", fa::fa_cart_arrow_down},
		{"cart-flatbed", fa::fa_cart_flatbed},
		{"dolly-flatbed", fa::fa_dolly_flatbed},
		{"cart-flatbed-suitcase", fa::fa_cart_flatbed_suitcase},
		{"luggage-cart", fa::fa_luggage_cart},
		{"cart-plus", fa::fa_cart_plus},
		{"cart-shopping", fa::fa_cart_shopping},
		{"shopping-cart", fa::fa_shopping_cart},
		{"cash-register", fa::fa_cash_register},
		{"cat", fa::fa_cat},
		{"cedi-sign", fa::fa_cedi_sign},
		{"cent-sign", fa::fa_cent_sign},
		{"certificate", fa::fa_certificate},
		{"chair", fa::fa_chair},
		{"chalkboard", fa::fa_chalkboard},
		{"blackboard", fa::fa_blackboard},
		{"chalkboard-user", fa::fa_chalkboard_user},
		{"chalkboard-teacher", fa::fa_chalkboard_teacher},
		{"champagne-glasses", fa::fa_champagne_glasses},
		{"glass-cheers", fa::fa_glass_cheers},
		{"charging-station", fa::fa_charging_station},
		{"chart-area", fa::fa_chart_area},
		{"area-chart", fa::fa_area_chart},
		{"chart-bar", fa::fa_chart_bar},
		{"bar-chart", fa::fa_bar_chart},
		{"chart-column", fa::fa_chart_column},
		{"chart-gantt", fa::fa_chart_gantt},
		{"chart-line", fa::fa_chart_line},
		{"line-chart", fa::fa_line_chart},
		{"chart-pie", fa::fa_chart_pie},
		{"pie-chart", fa::fa_pie_chart},
		{"chart-simple", fa::fa_chart_simple},
		{"check", fa::fa_check},
		{"check-double", fa::fa_check_double},
		{"check-to-slot", fa::fa_check_to_slot},
		{"vote-yea", fa::fa_vote_yea},
		{"cheese", fa::fa_cheese},
		{"chess", fa::fa_chess},
		{"chess-bishop", fa::fa_chess_bishop},
		{"chess-board", fa::fa_chess_board},
		{"chess-king", fa::fa_chess_king},
		{"chess-knight", fa::fa_chess_knight},
		{"chess-pawn", fa::fa_chess_pawn},
		{"chess-queen", fa::fa_chess_queen},
		{"chess-rook", fa::fa_chess_rook},
		{"chevron-down", fa::fa_chevron_down},
		{"chevron-left", fa::fa_chevron_left},
		{"chevron-right", fa::fa_chevron_right},
		{"chevron-up", fa::fa_chevron_up},
		{"child", fa::fa_child},
		{"child-combatant", fa::fa_child_combatant},
		{"child-rifle", fa::fa_child_rifle},
		{"child-dress", fa::fa_child_dress},
		{"child-reaching", fa::fa_child_reaching},
		{"children", fa::fa_children},
		{"church", fa::fa_church},
		{"circle", fa::fa_circle},
		{"circle-arrow-down", fa::fa_circle_arrow_down},
		{"arrow-circle-down", fa::fa_arrow_circle_down},
		{"circle-arrow-left", fa::fa_circle_arrow_left},
		{"arrow-circle-left", fa::fa_arrow_circle_left},
		{"circle-arrow-right", fa::fa_circle_arrow_right},
		{"arrow-circle-right", fa::fa_arrow_circle_right},
		{"circle-arrow-up", fa::fa_circle_arrow_up},
		{"arrow-circle-up", fa::fa_arrow_circle_up},
		{"circle-check", fa::fa_circle_check},
		{"check-circle", fa::fa_check_circle},
		{"circle-chevron-down", fa::fa_circle_chevron_down},
		{"chevron-circle-down", fa::fa_chevron_circle_down},
		{"circle-chevron-left", fa::fa_circle_chevron_left},
		{"chevron-circle-left", fa::fa_chevron_circle_left},
		{"circle-chevron-right", fa::fa_circle_chevron_right},
		{"chevron-circle-right", fa::fa_chevron_circle_right},
		{"circle-chevron-up", fa::fa_circle_chevron_up},
		{"chevron-circle-up", fa::fa_chevron_circle_up},
		{"circle-dollar-to-slot", fa::fa_circle_dollar_to_slot},
		{"donate", fa::fa_donate},
		{"circle-dot", fa::fa_circle_dot},
		{"dot-circle", fa::fa_dot_circle},
		{"circle-down", fa::fa_circle_down},
		{"arrow-alt-circle-down", fa::fa_arrow_alt_circle_down},
		{"circle-exclamation", fa::fa_circle_exclamation},
		{"exclamation-circle", fa::fa_exclamation_circle},
		{"circle-h", fa::fa_circle_h},
		{"hospital-symbol", fa::fa_hospital_symbol},
		{"circle-half-stroke", fa::fa_circle_half_stroke},
		{"adjust", fa::fa_adjust},
		{"circle-info", fa::fa_circle_info},
		{"info-circle", fa::fa_info_circle},
		{"circle-left", fa::fa_circle_left},
		{"arrow-alt-circle-left", fa::fa_arrow_alt_circle_left},
		{"circle-minus", fa::fa_circle_minus},
		{"minus-circle", fa::fa_minus_circle},
		{"circle-nodes", fa::fa_circle_nodes},
		{"circle-notch", fa::fa_circle_notch},
		{"circle-pause", fa::fa_circle_pause},
		{"pause-circle", fa::fa_pause_circle},
		{"circle-play", fa::fa_circle_play},
		{"play-circle", fa::fa_play_circle},
		{"circle-plus", fa::fa_circle_plus},
		{"plus-circle", fa::fa_plus_circle},
		{"circle-question", fa::fa_circle_question},
		{"question-circle", fa::fa_question_circle},
		{"circle-radiation", fa::fa_circle_radiation},
		{"radiation-alt", fa::fa_radiation_alt},
		{"circle-right", fa::fa_circle_right},
		{"arrow-alt-circle-right", fa::fa_arrow_alt_circle_right},
		{"circle-stop", fa::fa_circle_stop},
		{"stop-circle", fa::fa_stop_circle},
		{"circle-up", fa::fa_circle_up},
		{"arrow-alt-circle-up", fa::fa_arrow_alt_circle_up},
		{"circle-user", fa::fa_circle_user},
		{"user-circle", fa::fa_user_circle},
		{"circle-xmark", fa::fa_circle_xmark},
		{"times-circle", fa::fa_times_circle},
		{"xmark-circle", fa::fa_xmark_circle},
		{"city", fa::fa_city},
		{"clapperboard", fa::fa_clapperboard},
		{"clipboard", fa::fa_clipboard},
		{"clipboard-check", fa::fa_clipboard_check},
		{"clipboard-list", fa::fa_clipboard_list},
		{"clipboard-question", fa::fa_clipboard_question},
		{"clipboard-user", fa::fa_clipboard_user},
		{"clock", fa::fa_clock},
		{"clock-four", fa::fa_clock_four},
		{"clock-rotate-left", fa::fa_clock_rotate_left},
		{"history", fa::fa_history},
		{"clone", fa::fa_clone},
		{"closed-captioning", fa::fa_closed_captioning},
		{"cloud", fa::fa_cloud},
		{"cloud-arrow-down", fa::fa_cloud_arrow_down},
		{"cloud-download", fa::fa_cloud_download},
		{"cloud-download-alt", fa::fa_cloud_download_alt},
		{"cloud-arrow-up", fa::fa_cloud_arrow_up},
		{"cloud-upload", fa::fa_cloud_upload},
		{"cloud-upload-alt", fa::fa_cloud_upload_alt},
		{"cloud-bolt", fa::fa_cloud_bolt},
		{"thunderstorm", fa::fa_thunderstorm},
		{"cloud-meatball", fa::fa_cloud_meatball},
		{"cloud-moon", fa::fa_cloud_moon},
		{"cloud-moon-rain", fa::fa_cloud_moon_rain},
		{"cloud-rain", fa::fa_cloud_rain},
		{"cloud-showers-heavy", fa::fa_cloud_showers_heavy},
		{"cloud-showers-water", fa::fa_cloud_showers_water},
		{"cloud-sun", fa::fa_cloud_sun},
		{"cloud-sun-rain", fa::fa_cloud_sun_rain},
		{"clover", fa::fa_clover},
		{"code", fa::fa_code},
		{"code-branch", fa::fa_code_branch},
		{"code-commit", fa::fa_code_commit},
		{"code-compare", fa::fa_code_compare},
		{"code-fork", fa::fa_code_fork},
		{"code-merge", fa::fa_code_merge},
		{"code-pull-request", fa::fa_code_pull_request},
		{"coins", fa::fa_coins},
		{"colon-sign", fa::fa_colon_sign},
		{"comment", fa::fa_comment},
		{"comment-dollar", fa::fa_comment_dollar},
		{"comment-dots", fa::fa_comment_dots},
		{"commenting", fa::fa_commenting},
		{"comment-medical", fa::fa_comment_medical},
		{"comment-slash", fa::fa_comment_slash},
		{"comment-sms", fa::fa_comment_sms},
		{"sms", fa::fa_sms},
		{"comments", fa::fa_comments},
		{"comments-dollar", fa::fa_comments_dollar},
		{"compact-disc", fa::fa_compact_disc},
		{"compass", fa::fa_compass},
		{"compass-drafting", fa::fa_compass_drafting},
		{"drafting-compass", fa::fa_drafting_compass},
		{"compress", fa::fa_compress},
		{"computer", fa::fa_computer},
		{"computer-mouse", fa::fa_computer_mouse},
		{"mouse", fa::fa_mouse},
		{"cookie", fa::fa_cookie},
		{"cookie-bite", fa::fa_cookie_bite},
		{"copy", fa::fa_copy},
		{"copyright", fa::fa_copyright},
		{"couch", fa::fa_couch},
		{"cow", fa::fa_cow},
		{"credit-card", fa::fa_credit_card},
		{"credit-card-alt", fa::fa_credit_card_alt},
		{"crop", fa::fa_crop},
		{"crop-simple", fa::fa_crop_simple},
		{"crop-alt", fa::fa_crop_alt},
		{"cross", fa::fa_cross},
		{"crosshairs", fa::fa_crosshairs},
		{"crow", fa::fa_crow},
		{"crown", fa::fa_crown},
		{"crutch", fa::fa_crutch},
		{"cruzeiro-sign", fa::fa_cruzeiro_sign},
		{"cube", fa::fa_cube},
		{"cubes", fa::fa_cubes},
		{"cubes-stacked", fa::fa_cubes_stacked},
		{"d", fa::fa_d},
		{"database", fa::fa_database},
		{"delete-left", fa::fa_delete_left},
		{"backspace", fa::fa_backspace},
		{"democrat", fa::fa_democrat},
		{"desktop", fa::fa_desktop},
		{"desktop-alt", fa::fa_desktop_alt},
		{"dharmachakra", fa::fa_dharmachakra},
		{"diagram-next", fa::fa_diagram_next},
		{"diagram-predecessor", fa::fa_diagram_predecessor},
		{"diagram-project", fa::fa_diagram_project},
		{"project-diagram", fa::fa_project_diagram},
		{"diagram-successor", fa::fa_diagram_successor},
		{"diamond", fa::fa_diamond},
		{"diamond-turn-right", fa::fa_diamond_turn_right},
		{"directions", fa::fa_directions},
		{"dice", fa::fa_dice},
		{"dice-d20", fa::fa_dice_d20},
		{"dice-d6", fa::fa_dice_d6},
		{"dice-five", fa::fa_dice_five},
		{"dice-four", fa::fa_dice_four},
		{"dice-one", fa::fa_dice_one},
		{"dice-six", fa::fa_dice_six},
		{"dice-three", fa::fa_dice_three},
		{"dice-two", fa::fa_dice_two},
		{"disease", fa::fa_disease},
		{"display", fa::fa_display},
		{"divide", fa::fa_divide},
		{"dna", fa::fa_dna},
		{"dog", fa::fa_dog},
		{"dollar-sign", fa::fa_dollar_sign},
		{"dollar", fa::fa_dollar},
		{"usd", fa::fa_usd},
		{"dolly", fa::fa_dolly},
		{"dolly-box", fa::fa_dolly_box},
		{"dong-sign", fa::fa_dong_sign},
		{"door-closed", fa::fa_door_closed},
		{"door-open", fa::fa_door_open},
		{"dove", fa::fa_dove},
		{"down-left-and-up-right-to-center", fa::fa_down_left_and_up_right_to_center},
		{"compress-alt", fa::fa_compress_alt},
		{"down-long", fa::fa_down_long},
		{"long-arrow-alt-down", fa::fa_long_arrow_alt_down},
		{"download", fa::fa_download},
		{"dragon", fa::fa_dragon},
		{"draw-polygon", fa::fa_draw_polygon},
		{"droplet", fa::fa_droplet},
		{"tint", fa::fa_tint},
		{"droplet-slash", fa::fa_droplet_slash},
		{"tint-slash", fa::fa_tint_slash},
		{"drum", fa::fa_drum},
		{"drum-steelpan", fa::fa_drum_steelpan},
		{"drumstick-bite", fa::fa_drumstick_bite},
		{"dumbbell", fa::fa_dumbbell},
		{"dumpster", fa::fa_dumpster},
		{"dumpster-fire", fa::fa_dumpster_fire},
		{"dungeon", fa::fa_dungeon},
		{"e", fa::fa_e},
		{"ear-deaf", fa::fa_ear_deaf},
		{"deaf", fa::fa_deaf},
		{"deafness", fa::fa_deafness},
		{"hard-of-hearing", fa::fa_hard_of_hearing},
		{"ear-listen", fa::fa_ear_listen},
		{"assistive-listening-systems", fa::fa_assistive_listening_systems},
		{"earth-africa", fa::fa_earth_africa},
		{"globe-africa", fa::fa_globe_africa},
		{"earth-americas", fa::fa_earth_americas},
		{"earth", fa::fa_earth},
		{"earth-america", fa::fa_earth_america},
		{"globe-americas", fa::fa_globe_americas},
		{"earth-asia", fa::fa_earth_asia},
		{"globe-asia", fa::fa_globe_asia},
		{"earth-europe", fa::fa_earth_europe},
		{"globe-europe", fa::fa_globe_europe},
		{"earth-oceania", fa::fa_earth_oceania},
		{"globe-oceania", fa::fa_globe_oceania},
		{"egg", fa::fa_egg},
		{"eject", fa::fa_eject},
		{"elevator", fa::fa_elevator},
		{"ellipsis", fa::fa_ellipsis},
		{"ellipsis-h", fa::fa_ellipsis_h},
		{"ellipsis-vertical", fa::fa_ellipsis_vertical},
		{"ellipsis-v", fa::fa_ellipsis_v},
		{"envelope", fa::fa_envelope},
		{"envelope-circle-check", fa::fa_envelope_circle_check},
		{"envelope-open", fa::fa_envelope_open},
		{"envelope-open-text", fa::fa_envelope_open_text},
		{"envelopes-bulk", fa::fa_envelopes_bulk},
		{"mail-bulk", fa::fa_mail_bulk},
		{"equals", fa::fa_equals},
		{"eraser", fa::fa_eraser},
		{"ethernet", fa::fa_ethernet},
		{"euro-sign", fa::fa_euro_sign},
		{"eur", fa::fa_eur},
		{"euro", fa::fa_euro},
		{"exclamation", fa::fa_exclamation},
		{"expand", fa::fa_expand},
		{"explosion", fa::fa_explosion},
		{"eye", fa::fa_eye},
		{"eye-dropper", fa::fa_eye_dropper},
		{"eye-dropper-empty", fa::fa_eye_dropper_empty},
		{"eyedropper", fa::fa_eyedropper},
		{"eye-low-vision", fa::fa_eye_low_vision},
		{"low-vision", fa::fa_low_vision},
		{"eye-slash", fa::fa_eye_slash},
		{"f", fa::fa_f},
		{"face-angry", fa::fa_face_angry},
		{"angry", fa::fa_angry},
		{"face-dizzy", fa::fa_face_dizzy},
		{"dizzy", fa::fa_dizzy},
		{"face-flushed", fa::fa_face_flushed},
		{"flushed", fa::fa_flushed},
		{"face-frown", fa::fa_face_frown},
		{"frown", fa::fa_frown},
		{"face-frown-open", fa::fa_face_frown_open},
		{"frown-open", fa::fa_frown_open},
		{"face-grimace", fa::fa_face_grimace},
		{"grimace", fa::fa_grimace},
		{"face-grin", fa::fa_face_grin},
		{"grin", fa::fa_grin},
		{"face-grin-beam", fa::fa_face_grin_beam},
		{"grin-beam", fa::fa_grin_beam},
		{"face-grin-beam-sweat", fa::fa_face_grin_beam_sweat},
		{"grin-beam-sweat", fa::fa_grin_beam_sweat},
		{"face-grin-hearts", fa::fa_face_grin_hearts},
		{"grin-hearts", fa::fa_grin_hearts},
		{"face-grin-squint", fa::fa_face_grin_squint},
		{"grin-squint", fa::fa_grin_squint},
		{"face-grin-squint-tears", fa::fa_face_grin_squint_tears},
		{"grin-squint-tears", fa::fa_grin_squint_tears},
		{"face-grin-stars", fa::fa_face_grin_stars},
		{"grin-stars", fa::fa_grin_stars},
		{"face-grin-tears", fa::fa_face_grin_tears},
		{"grin-tears", fa::fa_grin_tears},
		{"face-grin-tongue", fa::fa_face_grin_tongue},
		{"grin-tongue", fa::fa_grin_tongue},
		{"face-grin-tongue-squint", fa::fa_face_grin_tongue_squint},
		{"grin-tongue-squint", fa::fa_grin_tongue_squint},
		{"face-grin-tongue-wink", fa::fa_face_grin_tongue_wink},
		{"grin-tongue-wink", fa::fa_grin_tongue_wink},
		{"face-grin-wide", fa::fa_face_grin_wide},
		{"grin-alt", fa::fa_grin_alt},
		{"face-grin-wink", fa::fa_face_grin_wink},
		{"grin-wink", fa::fa_grin_wink},
		{"face-kiss", fa::fa_face_kiss},
		{"kiss", fa::fa_kiss},
		{"face-kiss-beam", fa::fa_face_kiss_beam},
		{"kiss-beam", fa::fa_kiss_beam},
		{"face-kiss-wink-heart", fa::fa_face_kiss_wink_heart},
		{"kiss-wink-heart", fa::fa_kiss_wink_heart},
		{"face-laugh", fa::fa_face_laugh},
		{"laugh", fa::fa_laugh},
		{"face-laugh-beam", fa::fa_face_laugh_beam},
		{"laugh-beam", fa::fa_laugh_beam},
		{"face-laugh-squint", fa::fa_face_laugh_squint},
		{"laugh-squint", fa::fa_laugh_squint},
		{"face-laugh-wink", fa::fa_face_laugh_wink},
		{"laugh-wink", fa::fa_laugh_wink},
		{"face-meh", fa::fa_face_meh},
		{"meh", fa::fa_meh},
		{"face-meh-blank", fa::fa_face_meh_blank},
		{"meh-blank", fa::fa_meh_blank},
		{"face-rolling-eyes", fa::fa_face_rolling_eyes},
		{"meh-rolling-eyes", fa::fa_meh_rolling_eyes},
		{"face-sad-cry", fa::fa_face_sad_cry},
		{"sad-cry", fa::fa_sad_cry},
		{"face-sad-tear", fa::fa_face_sad_tear},
		{"sad-tear", fa::fa_sad_tear},
		{"face-smile", fa::fa_face_smile},
		{"smile", fa::fa_smile},
		{"face-smile-beam", fa::fa_face_smile_beam},
		{"smile-beam", fa::fa_smile_beam},
		{"face-smile-wink", fa::fa_face_smile_wink},
		{"smile-wink", fa::fa_smile_wink},
		{"face-surprise", fa::fa_face_surprise},
		{"surprise", fa::fa_surprise},
		{"face-tired", fa::fa_face_tired},
		{"tired", fa::fa_tired},
		{"fan", fa::fa_fan},
		{"faucet", fa::fa_faucet},
		{"faucet-drip", fa::fa_faucet_drip},
		{"fax", fa::fa_fax},
		{"feather", fa::fa_feather},
		{"feather-pointed", fa::fa_feather_pointed},
		{"feather-alt", fa::fa_feather_alt},
		{"ferry", fa::fa_ferry},
		{"file", fa::fa_file},
		{"file-arrow-down", fa::fa_file_arrow_down},
		{"file-download", fa::fa_file_download},
		{"file-arrow-up", fa::fa_file_arrow_up},
		{"file-upload", fa::fa_file_upload},
		{"file-audio", fa::fa_file_audio},
		{"file-circle-check", fa::fa_file_circle_check},
		{"file-circle-exclamation", fa::fa_file_circle_exclamation},
		{"file-circle-minus", fa::fa_file_circle_minus},
		{"file-circle-plus", fa::fa_file_circle_plus},
		{"file-circle-question", fa::fa_file_circle_question},
		{"file-circle-xmark", fa::fa_file_circle_xmark},
		{"file-code", fa::fa_file_code},
		{"file-contract", fa::fa_file_contract},
		{"file-csv", fa::fa_file_csv},
		{"file-excel", fa::fa_file_excel},
		{"file-export", fa::fa_file_export},
		{"arrow-right-from-file", fa::fa_arrow_right_from_file},
		{"file-image", fa::fa_file_image},
		{"file-import", fa::fa_file_import},
		{"arrow-right-to-file", fa::fa_arrow_right_to_file},
		{"file-invoice", fa::fa_file_invoice},
		{"file-invoice-dollar", fa::fa_file_invoice_dollar},
		{"file-lines", fa::fa_file_lines},
		{"file-alt", fa::fa_file_alt},
		{"file-text", fa::fa_file_text},
		{"file-medical", fa::fa_file_medical},
		{"file-pdf", fa::fa_file_pdf},
		{"file-pen", fa::fa_file_pen},
		{"file-edit", fa::fa_file_edit},
		{"file-powerpoint", fa::fa_file_powerpoint},
		{"file-prescription", fa::fa_file_prescription},
		{"file-shield", fa::fa_file_shield},
		{"file-signature", fa::fa_file_signature},
		{"file-video", fa::fa_file_video},
		{"file-waveform", fa::fa_file_waveform},
		{"file-medical-alt", fa::fa_file_medical_alt},
		{"file-word", fa::fa_file_word},
		{"file-zipper", fa::fa_file_zipper},
		{"file-archive", fa::fa_file_archive},
		{"fill", fa::fa_fill},
		{"fill-drip", fa::fa_fill_drip},
		{"film", fa::fa_film},
		{"filter", fa::fa_filter},
		{"filter-circle-dollar", fa::fa_filter_circle_dollar},
		{"funnel-dollar", fa::fa_funnel_dollar},
		{"filter-circle-xmark", fa::fa_filter_circle_xmark},
		{"fingerprint", fa::fa_fingerprint},
		{"fire", fa::fa_fire},
		{"fire-burner", fa::fa_fire_burner},
		{"fire-extinguisher", fa::fa_fire_extinguisher},
		{"fire-flame-curved", fa::fa_fire_flame_curved},
		{"fire-alt", fa::fa_fire_alt},
		{"fire-flame-simple", fa::fa_fire_flame_simple},
		{"burn", fa::fa_burn},
		{"fish", fa::fa_fish},
		{"fish-fins", fa::fa_fish_fins},
		{"flag", fa::fa_flag},
		{"flag-checkered", fa::fa_flag_checkered},
		{"flag-usa", fa::fa_flag_usa},
		{"flask", fa::fa_flask},
		{"flask-vial", fa::fa_flask_vial},
		{"floppy-disk", fa::fa_floppy_disk},
		{"save", fa::fa_save},
		{"florin-sign", fa::fa_florin_sign},
		{"folder", fa::fa_folder},
		{"folder-blank", fa::fa_folder_blank},
		{"folder-closed", fa::fa_folder_closed},
		{"folder-minus", fa::fa_folder_minus},
		{"folder-open", fa::fa_folder_open},
		{"folder-plus", fa::fa_folder_plus},
		{"folder-tree", fa::fa_folder_tree},
		{"font", fa::fa_font},
		{"football", fa::fa_football},
		{"football-ball", fa::fa_football_ball},
		{"forward", fa::fa_forward},
		{"forward-fast", fa::fa_forward_fast},
		{"fast-forward", fa::fa_fast_forward},
		{"forward-step", fa::fa_forward_step},
		{"step-forward", fa::fa_step_forward},
		{"franc-sign", fa::fa_franc_sign},
		{"frog", fa::fa_frog},
		{"futbol", fa::fa_futbol},
		{"futbol-ball", fa::fa_futbol_ball},
		{"soccer-ball", fa::fa_soccer_ball},
		{"g", fa::fa_g},
		{"gamepad", fa::fa_gamepad},
		{"gas-pump", fa::fa_gas_pump},
		{"gauge", fa::fa_gauge},
		{"dashboard", fa::fa_dashboard},
		{"gauge-med", fa::fa_gauge_med},
		{"tachometer-alt-average", fa::fa_tachometer_alt_average},
		{"gauge-high", fa::fa_gauge_high},
		{"tachometer-alt", fa::fa_tachometer_alt},
		{"tachometer-alt-fast", fa::fa_tachometer_alt_fast},
		{"gauge-simple", fa::fa_gauge_simple},
		{"gauge-simple-med", fa::fa_gauge_simple_med},
		{"tachometer-average", fa::fa_tachometer_average},
		{"gauge-simple-high", fa::fa_gauge_simple_high},
		{"tachometer", fa::fa_tachometer},
		{"tachometer-fast", fa::fa_tachometer_fast},
		{"gavel", fa::fa_gavel},
		{"legal", fa::fa_legal},
		{"gear", fa::fa_gear},
		{"cog", fa::fa_cog},
		{"gears", fa::fa_gears},
		{"cogs", fa::fa_cogs},
		{"gem", fa::fa_gem},
		{"genderless", fa::fa_genderless},
		{"ghost", fa::fa_ghost},
		{"gift", fa::fa_gift},
		{"gifts", fa::fa_gifts},
		{"glass-water", fa::fa_glass_water},
		{"glass-water-droplet", fa::fa_glass_water_droplet},
		{"glasses", fa::fa_glasses},
		{"globe", fa::fa_globe},
		{"golf-ball-tee", fa::fa_golf_ball_tee},
		{"golf-ball", fa::fa_golf_ball},
		{"gopuram", fa::fa_gopuram},
		{"graduation-cap", fa::fa_graduation_cap},
		{"mortar-board", fa::fa_mortar_board},
		{"greater-than", fa::fa_greater_than},
		{"greater-than-equal", fa::fa_greater_than_equal},
		{"grip", fa::fa_grip},
		{"grip-horizontal", fa::fa_grip_horizontal},
		{"grip-lines", fa::fa_grip_lines},
		{"grip-lines-vertical", fa::fa_grip_lines_vertical},
		{"grip-vertical", fa::fa_grip_vertical},
		{"group-arrows-rotate", fa::fa_group_arrows_rotate},
		{"guarani-sign", fa::fa_guarani_sign},
		{"guitar", fa::fa_guitar},
		{"gun", fa::fa_gun},
		{"h", fa::fa_h},
		{"hammer", fa::fa_hammer},
		{"hamsa", fa::fa_hamsa},
		{"hand", fa::fa_hand},
		{"hand-paper", fa::fa_hand_paper},
		{"hand-back-fist", fa::fa_hand_back_fist},
		{"hand-rock", fa::fa_hand_rock},
		{"hand-dots", fa::fa_hand_dots},
		{"allergies", fa::fa_allergies},
		{"hand-fist", fa::fa_hand_fist},
		{"fist-raised", fa::fa_fist_raised},
		{"hand-holding", fa::fa_hand_holding},
		{"hand-holding-dollar", fa::fa_hand_holding_dollar},
		{"hand-holding-usd", fa::fa_hand_holding_usd},
		{"hand-holding-droplet", fa::fa_hand_holding_droplet},
		{"hand-holding-water", fa::fa_hand_holding_water},
		{"hand-holding-hand", fa::fa_hand_holding_hand},
		{"hand-holding-heart", fa::fa_hand_holding_heart},
		{"hand-holding-medical", fa::fa_hand_holding_medical},
		{"hand-lizard", fa::fa_hand_lizard},
		{"hand-middle-finger", fa::fa_hand_middle_finger},
		{"hand-peace", fa::fa_hand_peace},
		{"hand-point-down", fa::fa_hand_point_down},
		{"hand-point-left", fa::fa_hand_point_left},
		{"hand-point-right", fa::fa_hand_point_right},
		{"hand-point-up", fa::fa_hand_point_up},
		{"hand-pointer", fa::fa_hand_pointer},
		{"hand-scissors", fa::fa_hand_scissors},
		{"hand-sparkles", fa::fa_hand_sparkles},
		{"hand-spock", fa::fa_hand_spock},
		{"handcuffs", fa::fa_handcuffs},
		{"hands", fa::fa_hands},
		{"sign-language", fa::fa_sign_language},
		{"signing", fa::fa_signing},
		{"hands-asl-interpreting", fa::fa_hands_asl_interpreting},
		{"american-sign-language-interpreting", fa::fa_american_sign_language_interpreting},
		{"asl-interpreting", fa::fa_asl_interpreting},
		{"hands-american-sign-language-interpreting", fa::fa_hands_american_sign_language_interpreting},
		{"hands-bound", fa::fa_hands_bound},
		{"hands-bubbles", fa::fa_hands_bubbles},
		{"hands-wash", fa::fa_hands_wash},
		{"hands-clapping", fa::fa_hands_clapping},
		{"hands-holding", fa::fa_hands_holding},
		{"hands-holding-child", fa::fa_hands_holding_child},
		{"hands-holding-circle", fa::fa_hands_holding_circle},
		{"hands-praying", fa::fa_hands_praying},
		{"praying-hands", fa::fa_praying_hands},
		{"handshake", fa::fa_handshake},
		{"handshake-angle", fa::fa_handshake_angle},
		{"hands-helping", fa::fa_hands_helping},
		{"handshake-simple", fa::fa_handshake_simple},
		{"handshake-alt", fa::fa_handshake_alt},
		{"handshake-simple-slash", fa::fa_handshake_simple_slash},
		{"handshake-alt-slash", fa::fa_handshake_alt_slash},
		{"handshake-slash", fa::fa_handshake_slash},
		{"hanukiah", fa::fa_hanukiah},
		{"hard-drive", fa::fa_hard_drive},
		{"hdd", fa::fa_hdd},
		{"hashtag", fa::fa_hashtag},
		{"hat-cowboy", fa::fa_hat_cowboy},
		{"hat-cowboy-side", fa::fa_hat_cowboy_side},
		{"hat-wizard", fa::fa_hat_wizard},
		{"head-side-cough", fa::fa_head_side_cough},
		{"head-side-cough-slash", fa::fa_head_side_cough_slash},
		{"head-side-mask", fa::fa_head_side_mask},
		{"head-side-virus", fa::fa_head_side_virus},
		{"heading", fa::fa_heading},
		{"header", fa::fa_header},
		{"headphones", fa::fa_headphones},
		{"headphones-simple", fa::fa_headphones_simple},
		{"headphones-alt", fa::fa_headphones_alt},
		{"headset", fa::fa_headset},
		{"heart", fa::fa_heart},
		{"heart-circle-bolt", fa::fa_heart_circle_bolt},
		{"heart-circle-check", fa::fa_heart_circle_check},
		{"heart-circle-exclamation", fa::fa_heart_circle_exclamation},
		{"heart-circle-minus", fa::fa_heart_circle_minus},
		{"heart-circle-plus", fa::fa_heart_circle_plus},
		{"heart-circle-xmark", fa::fa_heart_circle_xmark},
		{"heart-crack", fa::fa_heart_crack},
		{"heart-broken", fa::fa_heart_broken},
		{"heart-pulse", fa::fa_heart_pulse},
		{"heartbeat", fa::fa_heartbeat},
		{"helicopter", fa::fa_helicopter},
		{"helicopter-symbol", fa::fa_helicopter_symbol},
		{"helmet-safety", fa::fa_helmet_safety},
		{"hard-hat", fa::fa_hard_hat},
		{"hat-hard", fa::fa_hat_hard},
		{"helmet-un", fa::fa_helmet_un},
		{"highlighter", fa::fa_highlighter},
		{"hill-avalanche", fa::fa_hill_avalanche},
		{"hill-rockslide", fa::fa_hill_rockslide},
		{"hippo", fa::fa_hippo},
		{"hockey-puck", fa::fa_hockey_puck},
		{"holly-berry", fa::fa_holly_berry},
		{"horse", fa::fa_horse},
		{"horse-head", fa::fa_horse_head},
		{"hospital", fa::fa_hospital},
		{"hospital-alt", fa::fa_hospital_alt},
		{"hospital-wide", fa::fa_hospital_wide},
		{"hospital-user", fa::fa_hospital_user},
		{"hot-tub-person", fa::fa_hot_tub_person},
		{"hot-tub", fa::fa_hot_tub},
		{"hotdog", fa::fa_hotdog},
		{"hotel", fa::fa_hotel},
		{"hourglass", fa::fa_hourglass},
		{"hourglass-empty", fa::fa_hourglass_empty},
		{"hourglass-end", fa::fa_hourglass_end},
		{"hourglass-3", fa::fa_hourglass_3},
		{"hourglass-half", fa::fa_hourglass_half},
		{"hourglass-2", fa::fa_hourglass_2},
		{"hourglass-start", fa::fa_hourglass_start},
		{"hourglass-1", fa::fa_hourglass_1},
		{"house", fa::fa_house},
		{"home", fa::fa_home},
		{"home-alt", fa::fa_home_alt},
		{"home-lg-alt", fa::fa_home_lg_alt},
		{"house-chimney", fa::fa_house_chimney},
		{"home-lg", fa::fa_home_lg},
		{"house-chimney-crack", fa::fa_house_chimney_crack},
		{"house-damage", fa::fa_house_damage},
		{"house-chimney-medical", fa::fa_house_chimney_medical},
		{"clinic-medical", fa::fa_clinic_medical},
		{"house-chimney-user", fa::fa_house_chimney_user},
		{"house-chimney-window", fa::fa_house_chimney_window},
		{"house-circle-check", fa::fa_house_circle_check},
		{"house-circle-exclamation", fa::fa_house_circle_exclamation},
		{"house-circle-xmark", fa::fa_house_circle_xmark},
		{"house-crack", fa::fa_house_crack},
		{"house-fire", fa::fa_house_fire},
		{"house-flag", fa::fa_house_flag},
		{"house-flood-water", fa::fa_house_flood_water},
		{"house-flood-water-circle-arrow-right", fa::fa_house_flood_water_circle_arrow_right},
		{"house-laptop", fa::fa_house_laptop},
		{"laptop-house", fa::fa_laptop_house},
		{"house-lock", fa::fa_house_lock},
		{"house-medical", fa::fa_house_medical},
		{"house-medical-circle-check", fa::fa_house_medical_circle_check},
		{"house-medical-circle-exclamation", fa::fa_house_medical_circle_exclamation},
		{"house-medical-circle-xmark", fa::fa_house_medical_circle_xmark},
		{"house-medical-flag", fa::fa_house_medical_flag},
		{"house-signal", fa::fa_house_signal},
		{"house-tsunami", fa::fa_house_tsunami},
		{"house-user", fa::fa_house_user},
		{"home-user", fa::fa_home_user},
		{"hryvnia-sign", fa::fa_hryvnia_sign},
		{"hryvnia", fa::fa_hryvnia},
		{"hurricane", fa::fa_hurricane},
		{"i", fa::fa_i},
		{"i-cursor", fa::fa_i_cursor},
		{"ice-cream", fa::fa_ice_cream},
		{"icicles", fa::fa_icicles},
		{"icons", fa::fa_icons},
		{"heart-music-camera-bolt", fa::fa_heart_music_camera_bolt},
		{"id-badge", fa::fa_id_badge},
		{"id-card", fa::fa_id_card},
		{"drivers-license", fa::fa_drivers_license},
		{"id-card-clip", fa::fa_id_card_clip},
		{"id-card-alt", fa::fa_id_card_alt},
		{"igloo", fa::fa_igloo},
		{"image", fa::fa_image},
		{"image-portrait", fa::fa_image_portrait},
		{"portrait", fa::fa_portrait},
		{"images", fa::fa_images},
		{"inbox", fa::fa_inbox},
		{"indent", fa::fa_indent},
		{"indian-rupee-sign", fa::fa_indian_rupee_sign},
		{"indian-rupee", fa::fa_indian_rupee},
		{"inr", fa::fa_inr},
		{"industry", fa::fa_industry},
		{"infinity", fa::fa_infinity},
		{"info", fa::fa_info},
		{"italic", fa::fa_italic},
		{"j", fa::fa_j},
		{"jar", fa::fa_jar},
		{"jar-wheat", fa::fa_jar_wheat},
		{"jedi", fa::fa_jedi},
		{"jet-fighter", fa::fa_jet_fighter},
		{"fighter-jet", fa::fa_fighter_jet},
		{"jet-fighter-up", fa::fa_jet_fighter_up},
		{"joint", fa::fa_joint},
		{"jug-detergent", fa::fa_jug_detergent},
		{"k", fa::fa_k},
		{"kaaba", fa::fa_kaaba},
		{"key", fa::fa_key},
		{"keyboard", fa::fa_keyboard},
		{"khanda", fa::fa_khanda},
		{"kip-sign", fa::fa_kip_sign},
		{"kit-medical", fa::fa_kit_medical},
		{"first-aid", fa::fa_first_aid},
		{"kitchen-set", fa::fa_kitchen_set},
		{"kiwi-bird", fa::fa_kiwi_bird},
		{"l", fa::fa_l},
		{"land-mine-on", fa::fa_land_mine_on},
		{"landmark", fa::fa_landmark},
		{"landmark-dome", fa::fa_landmark_dome},
		{"landmark-alt", fa::fa_landmark_alt},
		{"landmark-flag", fa::fa_landmark_flag},
		{"language", fa::fa_language},
		{"laptop", fa::fa_laptop},
		{"laptop-code", fa::fa_laptop_code},
		{"laptop-file", fa::fa_laptop_file},
		{"laptop-medical", fa::fa_laptop_medical},
		{"lari-sign", fa::fa_lari_sign},
		{"layer-group", fa::fa_layer_group},
		{"leaf", fa::fa_leaf},
		{"left-long", fa::fa_left_long},
		{"long-arrow-alt-left", fa::fa_long_arrow_alt_left},
		{"left-right", fa::fa_left_right},
		{"arrows-alt-h", fa::fa_arrows_alt_h},
		{"lemon", fa::fa_lemon},
		{"less-than", fa::fa_less_than},
		{"less-than-equal", fa::fa_less_than_equal},
		{"life-ring", fa::fa_life_ring},
		{"lightbulb", fa::fa_lightbulb},
		{"lines-leaning", fa::fa_lines_leaning},
		{"link", fa::fa_link},
		{"chain", fa::fa_chain},
		{"link-slash", fa::fa_link_slash},
		{"chain-broken", fa::fa_chain_broken},
		{"chain-slash", fa::fa_chain_slash},
		{"unlink", fa::fa_unlink},
		{"lira-sign", fa::fa_lira_sign},
		{"list", fa::fa_list},
		{"list-squares", fa::fa_list_squares},
		{"list-check", fa::fa_list_check},
		{"tasks", fa::fa_tasks},
		{"list-ol", fa::fa_list_ol},
		{"list-1-2", fa::fa_list_1_2},
		{"list-numeric", fa::fa_list_numeric},
		{"list-ul", fa::fa_list_ul},
		{"list-dots", fa::fa_list_dots},
		{"litecoin-sign", fa::fa_litecoin_sign},
		{"location-arrow", fa::fa_location_arrow},
		{"location-crosshairs", fa::fa_location_crosshairs},
		{"location", fa::fa_location},
		{"location-dot", fa::fa_location_dot},
		{"map-marker-alt", fa::fa_map_marker_alt},
		{"location-pin", fa::fa_location_pin},
		{"map-marker", fa::fa_map_marker},
		{"location-pin-lock", fa::fa_location_pin_lock},
		{"lock", fa::fa_lock},
		{"lock-open", fa::fa_lock_open},
		{"locust", fa::fa_locust},
		{"lungs", fa::fa_lungs},
		{"lungs-virus", fa::fa_lungs_virus},
		{"m", fa::fa_m},
		{"magnet", fa::fa_magnet},
		{"magnifying-glass", fa::fa_magnifying_glass},
		{"search", fa::fa_search},
		{"magnifying-glass-arrow-right", fa::fa_magnifying_glass_arrow_right},
		{"magnifying-glass-chart", fa::fa_magnifying_glass_chart},
		{"magnifying-glass-dollar", fa::fa_magnifying_glass_dollar},
		{"search-dollar", fa::fa_search_dollar},
		{"magnifying-glass-location", fa::fa_magnifying_glass_location},
		{"search-location", fa::fa_search_location},
		{"magnifying-glass-minus", fa::fa_magnifying_glass_minus},
		{"search-minus", fa::fa_search_minus},
		{"magnifying-glass-plus", fa::fa_magnifying_glass_plus},
		{"search-plus", fa::fa_search_plus},
		{"manat-sign", fa::fa_manat_sign},
		{"map", fa::fa_map},
		{"map-location", fa::fa_map_location},
		{"map-marked", fa::fa_map_marked},
		{"map-location-dot", fa::fa_map_location_dot},
		{"map-marked-alt", fa::fa_map_marked_alt},
		{"map-pin", fa::fa_map_pin},
		{"marker", fa::fa_marker},
		{"mars", fa::fa_mars},
		{"mars-and-venus", fa::fa_mars_and_venus},
		{"mars-and-venus-burst", fa::fa_mars_and_venus_burst},
		{"mars-double", fa::fa_mars_double},
		{"mars-stroke", fa::fa_mars_stroke},
		{"mars-stroke-right", fa::fa_mars_stroke_right},
		{"mars-stroke-h", fa::fa_mars_stroke_h},
		{"mars-stroke-up", fa::fa_mars_stroke_up},
		{"mars-stroke-v", fa::fa_mars_stroke_v},
		{"martini-glass", fa::fa_martini_glass},
		{"glass-martini-alt", fa::fa_glass_martini_alt},
		{"martini-glass-citrus", fa::fa_martini_glass_citrus},
		{"cocktail", fa::fa_cocktail},
		{"martini-glass-empty", fa::fa_martini_glass_empty},
		{"glass-martini", fa::fa_glass_martini},
		{"mask", fa::fa_mask},
		{"mask-face", fa::fa_mask_face},
		{"mask-ventilator", fa::fa_mask_ventilator},
		{"masks-theater", fa::fa_masks_theater},
		{"theater-masks", fa::fa_theater_masks},
		{"mattress-pillow", fa::fa_mattress_pillow},
		{"maximize", fa::fa_maximize},
		{"expand-arrows-alt", fa::fa_expand_arrows_alt},
		{"medal", fa::fa_medal},
		{"memory", fa::fa_memory},
		{"menorah", fa::fa_menorah},
		{"mercury", fa::fa_mercury},
		{"message", fa::fa_message},
		{"comment-alt", fa::fa_comment_alt},
		{"meteor", fa::fa_meteor},
		{"microchip", fa::fa_microchip},
		{"microphone", fa::fa_microphone},
		{"microphone-lines", fa::fa_microphone_lines},
		{"microphone-alt", fa::fa_microphone_alt},
		{"microphone-lines-slash", fa::fa_microphone_lines_slash},
		{"microphone-alt-slash", fa::fa_microphone_alt_slash},
		{"microphone-slash", fa::fa_microphone_slash},
		{"microscope", fa::fa_microscope},
		{"mill-sign", fa::fa_mill_sign},
		{"minimize", fa::fa_minimize},
		{"compress-arrows-alt", fa::fa_compress_arrows_alt},
		{"minus", fa::fa_minus},
		{"subtract", fa::fa_subtract},
		{"mitten", fa::fa_mitten},
		{"mobile", fa::fa_mobile},
		{"mobile-android", fa::fa_mobile_android},
		{"mobile-phone", fa::fa_mobile_phone},
		{"mobile-button", fa::fa_mobile_button},
		{"mobile-retro", fa::fa_mobile_retro},
		{"mobile-screen", fa::fa_mobile_screen},
		{"mobile-android-alt", fa::fa_mobile_android_alt},
		{"mobile-screen-button", fa::fa_mobile_screen_button},
		{"mobile-alt", fa::fa_mobile_alt},
		{"money-bill", fa::fa_money_bill},
		{"money-bill-1", fa::fa_money_bill_1},
		{"money-bill-alt", fa::fa_money_bill_alt},
		{"money-bill-1-wave", fa::fa_money_bill_1_wave},
		{"money-bill-wave-alt", fa::fa_money_bill_wave_alt},
		{"money-bill-transfer", fa::fa_money_bill_transfer},
		{"money-bill-trend-up", fa::fa_money_bill_trend_up},
		{"money-bill-wave", fa::fa_money_bill_wave},
		{"money-bill-wheat", fa::fa_money_bill_wheat},
		{"money-bills", fa::fa_money_bills},
		{"money-check", fa::fa_money_check},
		{"money-check-dollar", fa::fa_money_check_dollar},
		{"money-check-alt", fa::fa_money_check_alt},
		{"monument", fa::fa_monument},
		{"moon", fa::fa_moon},
		{"mortar-pestle", fa::fa_mortar_pestle},
		{"mosque", fa::fa_mosque},
		{"mosquito", fa::fa_mosquito},
		{"mosquito-net", fa::fa_mosquito_net},
		{"motorcycle", fa::fa_motorcycle},
		{"mound", fa::fa_mound},
		{"mountain", fa::fa_mountain},
		{"mountain-city", fa::fa_mountain_city},
		{"mountain-sun", fa::fa_mountain_sun},
		{"mug-hot", fa::fa_mug_hot},
		{"mug-saucer", fa::fa_mug_saucer},
		{"coffee", fa::fa_coffee},
		{"music", fa::fa_music},
		{"n", fa::fa_n},
		{"naira-sign", fa::fa_naira_sign},
		{"network-wired", fa::fa_network_wired},
		{"neuter", fa::fa_neuter},
		{"newspaper", fa::fa_newspaper},
		{"not-equal", fa::fa_not_equal},
		{"notdef", fa::fa_notdef},
		{"note-sticky", fa::fa_note_sticky},
		{"sticky-note", fa::fa_sticky_note},
		{"notes-medical", fa::fa_notes_medical},
		{"o", fa::fa_o},
		{"object-group", fa::fa_object_group},
		{"object-ungroup", fa::fa_object_ungroup},
		{"oil-can", fa::fa_oil_can},
		{"oil-well", fa::fa_oil_well},
		{"om", fa::fa_om},
		{"otter", fa::fa_otter},
		{"outdent", fa::fa_outdent},
		{"dedent", fa::fa_dedent},
		{"p", fa::fa_p},
		{"pager", fa::fa_pager},
		{"paint-roller", fa::fa_paint_roller},
		{"paintbrush", fa::fa_paintbrush},
		{"paint-brush", fa::fa_paint_brush},
		{"palette", fa::fa_palette},
		{"pallet", fa::fa_pallet},
		{"panorama", fa::fa_panorama},
		{"paper-plane", fa::fa_paper_plane},
		{"paperclip", fa::fa_paperclip},
		{"parachute-box", fa::fa_parachute_box},
		{"paragraph", fa::fa_paragraph},
		{"passport", fa::fa_passport},
		{"paste", fa::fa_paste},
		{"file-clipboard", fa::fa_file_clipboard},
		{"pause", fa::fa_pause},
		{"paw", fa::fa_paw},
		{"peace", fa::fa_peace},
		{"pen", fa::fa_pen},
		{"pen-clip", fa::fa_pen_clip},
		{"pen-alt", fa::fa_pen_alt},
		{"pen-fancy", fa::fa_pen_fancy},
		{"pen-nib", fa::fa_pen_nib},
		{"pen-ruler", fa::fa_pen_ruler},
		{"pencil-ruler", fa::fa_pencil_ruler},
		{"pen-to-square", fa::fa_pen_to_square},
		{"edit", fa::fa_edit},
		{"pencil", fa::fa_pencil},
		{"pencil-alt", fa::fa_pencil_alt},
		{"people-arrows", fa::fa_people_arrows},
		{"people-arrows-left-right", fa::fa_people_arrows_left_right},
		{"people-carry-box", fa::fa_people_carry_box},
		{"people-carry", fa::fa_people_carry},
		{"people-group", fa::fa_people_group},
		{"people-line", fa::fa_people_line},
		{"people-pulling", fa::fa_people_pulling},
		{"people-robbery", fa::fa_people_robbery},
		{"people-roof", fa::fa_people_roof},
		{"pepper-hot", fa::fa_pepper_hot},
		{"percent", fa::fa_percent},
		{"percentage", fa::fa_percentage},
		{"person", fa::fa_person},
		{"male", fa::fa_male},
		{"person-arrow-down-to-line", fa::fa_person_arrow_down_to_line},
		{"person-arrow-up-from-line", fa::fa_person_arrow_up_from_line},
		{"person-biking", fa::fa_person_biking},
		{"biking", fa::fa_biking},
		{"person-booth", fa::fa_person_booth},
		{"person-breastfeeding", fa::fa_person_breastfeeding},
		{"person-burst", fa::fa_person_burst},
		{"person-cane", fa::fa_person_cane},
		{"person-chalkboard", fa::fa_person_chalkboard},
		{"person-circle-check", fa::fa_person_circle_check},
		{"person-circle-exclamation", fa::fa_person_circle_exclamation},
		{"person-circle-minus", fa::fa_person_circle_minus},
		{"person-circle-plus", fa::fa_person_circle_plus},
		{"person-circle-question", fa::fa_person_circle_question},
		{"person-circle-xmark", fa::fa_person_circle_xmark},
		{"person-digging", fa::fa_person_digging},
		{"digging", fa::fa_digging},
		{"person-dots-from-line", fa::fa_person_dots_from_line},
		{"diagnoses", fa::fa_diagnoses},
		{"person-dress", fa::fa_person_dress},
		{"female", fa::fa_female},
		{"person-dress-burst", fa::fa_person_dress_burst},
		{"person-drowning", fa::fa_person_drowning},
		{"person-falling", fa::fa_person_falling},
		{"person-falling-burst", fa::fa_person_falling_burst},
		{"person-half-dress", fa::fa_person_half_dress},
		{"person-harassing", fa::fa_person_harassing},
		{"person-hiking", fa::fa_person_hiking},
		{"hiking", fa::fa_hiking},
		{"person-military-pointing", fa::fa_person_military_pointing},
		{"person-military-rifle", fa::fa_person_military_rifle},
		{"person-military-to-person", fa::fa_person_military_to_person},
		{"person-praying", fa::fa_person_praying},
		{"pray", fa::fa_pray},
		{"person-pregnant", fa::fa_person_pregnant},
		{"person-rays", fa::fa_person_rays},
		{"person-rifle", fa::fa_person_rifle},
		{"person-running", fa::fa_person_running},
		{"running", fa::fa_running},
		{"person-shelter", fa::fa_person_shelter},
		{"person-skating", fa::fa_person_skating},
		{"skating", fa::fa_skating},
		{"person-skiing", fa::fa_person_skiing},
		{"skiing", fa::fa_skiing},
		{"person-skiing-nordic", fa::fa_person_skiing_nordic},
		{"skiing-nordic", fa::fa_skiing_nordic},
		{"person-snowboarding", fa::fa_person_snowboarding},
		{"snowboarding", fa::fa_snowboarding},
		{"person-swimming", fa::fa_person_swimming},
		{"swimmer", fa::fa_swimmer},
		{"person-through-window", fa::fa_person_through_window},
		{"person-walking", fa::fa_person_walking},
		{"walking", fa::fa_walking},
		{"person-walking-arrow-loop-left", fa::fa_person_walking_arrow_loop_left},
		{"person-walking-arrow-right", fa::fa_person_walking_arrow_right},
		{"person-walking-dashed-line-arrow-right", fa::fa_person_walking_dashed_line_arrow_right},
		{"person-walking-luggage", fa::fa_person_walking_luggage},
		{"person-walking-with-cane", fa::fa_person_walking_with_cane},
		{"blind", fa::fa_blind},
		{"peseta-sign", fa::fa_peseta_sign},
		{"peso-sign", fa::fa_peso_sign},
		{"phone", fa::fa_phone},
		{"phone-flip", fa::fa_phone_flip},
		{"phone-alt", fa::fa_phone_alt},
		{"phone-slash", fa::fa_phone_slash},
		{"phone-volume", fa::fa_phone_volume},
		{"volume-control-phone", fa::fa_volume_control_phone},
		{"photo-film", fa::fa_photo_film},
		{"photo-video", fa::fa_photo_video},
		{"piggy-bank", fa::fa_piggy_bank},
		{"pills", fa::fa_pills},
		{"pizza-slice", fa::fa_pizza_slice},
		{"place-of-worship", fa::fa_place_of_worship},
		{"plane", fa::fa_plane},
		{"plane-arrival", fa::fa_plane_arrival},
		{"plane-circle-check", fa::fa_plane_circle_check},
		{"plane-circle-exclamation", fa::fa_plane_circle_exclamation},
		{"plane-circle-xmark", fa::fa_plane_circle_xmark},
		{"plane-departure", fa::fa_plane_departure},
		{"plane-lock", fa::fa_plane_lock},
		{"plane-slash", fa::fa_plane_slash},
		{"plane-up", fa::fa_plane_up},
		{"plant-wilt", fa::fa_plant_wilt},
		{"plate-wheat", fa::fa_plate_wheat},
		{"play", fa::fa_play},
		{"plug", fa::fa_plug},
		{"plug-circle-bolt", fa::fa_plug_circle_bolt},
		{"plug-circle-check", fa::fa_plug_circle_check},
		{"plug-circle-exclamation", fa::fa_plug_circle_exclamation},
		{"plug-circle-minus", fa::fa_plug_circle_minus},
		{"plug-circle-plus", fa::fa_plug_circle_plus},
		{"plug-circle-xmark", fa::fa_plug_circle_xmark},
		{"plus", fa::fa_plus},
		{"add", fa::fa_add},
		{"plus-minus", fa::fa_plus_minus},
		{"podcast", fa::fa_podcast},
		{"poo", fa::fa_poo},
		{"poo-storm", fa::fa_poo_storm},
		{"poo-bolt", fa::fa_poo_bolt},
		{"poop", fa::fa_poop},
		{"power-off", fa::fa_power_off},
		{"prescription", fa::fa_prescription},
		{"prescription-bottle", fa::fa_prescription_bottle},
		{"prescription-bottle-medical", fa::fa_prescription_bottle_medical},
		{"prescription-bottle-alt", fa::fa_prescription_bottle_alt},
		{"print", fa::fa_print},
		{"pump-medical", fa::fa_pump_medical},
		{"pump-soap", fa::fa_pump_soap},
		{"puzzle-piece", fa::fa_puzzle_piece},
		{"q", fa::fa_q},
		{"qrcode", fa::fa_qrcode},
		{"question", fa::fa_question},
		{"quote-left", fa::fa_quote_left},
		{"quote-left-alt", fa::fa_quote_left_alt},
		{"quote-right", fa::fa_quote_right},
		{"quote-right-alt", fa::fa_quote_right_alt},
		{"r", fa::fa_r},
		{"radiation", fa::fa_radiation},
		{"radio", fa::fa_radio},
		{"rainbow", fa::fa_rainbow},
		{"ranking-star", fa::fa_ranking_star},
		{"receipt", fa::fa_receipt},
		{"record-vinyl", fa::fa_record_vinyl},
		{"rectangle-ad", fa::fa_rectangle_ad},
		{"ad", fa::fa_ad},
		{"rectangle-list", fa::fa_rectangle_list},
		{"list-alt", fa::fa_list_alt},
		{"rectangle-xmark", fa::fa_rectangle_xmark},
		{"rectangle-times", fa::fa_rectangle_times},
		{"times-rectangle", fa::fa_times_rectangle},
		{"window-close", fa::fa_window_close},
		{"recycle", fa::fa_recycle},
		{"registered", fa::fa_registered},
		{"repeat", fa::fa_repeat},
		{"reply", fa::fa_reply},
		{"mail-reply", fa::fa_mail_reply},
		{"reply-all", fa::fa_reply_all},
		{"mail-reply-all", fa::fa_mail_reply_all},
		{"republican", fa::fa_republican},
		{"restroom", fa::fa_restroom},
		{"retweet", fa::fa_retweet},
		{"ribbon", fa::fa_ribbon},
		{"right-from-bracket", fa::fa_right_from_bracket},
		{"sign-out-alt", fa::fa_sign_out_alt},
		{"right-left", fa::fa_right_left},
		{"exchange-alt", fa::fa_exchange_alt},
		{"right-long", fa::fa_right_long},
		{"long-arrow-alt-right", fa::fa_long_arrow_alt_right},
		{"right-to-bracket", fa::fa_right_to_bracket},
		{"sign-in-alt", fa::fa_sign_in_alt},
		{"ring", fa::fa_ring},
		{"road", fa::fa_road},
		{"road-barrier", fa::fa_road_barrier},
		{"road-bridge", fa::fa_road_bridge},
		{"road-circle-check", fa::fa_road_circle_check},
		{"road-circle-exclamation", fa::fa_road_circle_exclamation},
		{"road-circle-xmark", fa::fa_road_circle_xmark},
		{"road-lock", fa::fa_road_lock},
		{"road-spikes", fa::fa_road_spikes},
		{"robot", fa::fa_robot},
		{"rocket", fa::fa_rocket},
		{"rotate", fa::fa_rotate},
		{"sync-alt", fa::fa_sync_alt},
		{"rotate-left", fa::fa_rotate_left},
		{"rotate-back", fa::fa_rotate_back},
		{"rotate-backward", fa::fa_rotate_backward},
		{"undo-alt", fa::fa_undo_alt},
		{"rotate-right", fa::fa_rotate_right},
		{"redo-alt", fa::fa_redo_alt},
		{"rotate-forward", fa::fa_rotate_forward},
		{"route", fa::fa_route},
		{"rss", fa::fa_rss},
		{"feed", fa::fa_feed},
		{"ruble-sign", fa::fa_ruble_sign},
		{"rouble", fa::fa_rouble},
		{"rub", fa::fa_rub},
		{"ruble", fa::fa_ruble},
		{"rug", fa::fa_rug},
		{"ruler", fa::fa_ruler},
		{"ruler-combined", fa::fa_ruler_combined},
		{"ruler-horizontal", fa::fa_ruler_horizontal},
		{"ruler-vertical", fa::fa_ruler_vertical},
		{"rupee-sign", fa::fa_rupee_sign},
		{"rupee", fa::fa_rupee},
		{"rupiah-sign", fa::fa_rupiah_sign},
		{"s", fa::fa_s},
		{"sack-dollar", fa::fa_sack_dollar},
		{"sack-xmark", fa::fa_sack_xmark},
		{"sailboat", fa::fa_sailboat},
		{"satellite", fa::fa_satellite},
		{"satellite-dish", fa::fa_satellite_dish},
		{"scale-balanced", fa::fa_scale_balanced},
		{"balance-scale", fa::fa_balance_scale},
		{"scale-unbalanced", fa::fa_scale_unbalanced},
		{"balance-scale-left", fa::fa_balance_scale_left},
		{"scale-unbalanced-flip", fa::fa_scale_unbalanced_flip},
		{"balance-scale-right", fa::fa_balance_scale_right},
		{"school", fa::fa_school},
		{"school-circle-check", fa::fa_school_circle_check},
		{"school-circle-exclamation", fa::fa_school_circle_exclamation},
		{"school-circle-xmark", fa::fa_school_circle_xmark},
		{"school-flag", fa::fa_school_flag},
		{"school-lock", fa::fa_school_lock},
		{"scissors", fa::fa_scissors},
		{"cut", fa::fa_cut},
		{"screwdriver", fa::fa_screwdriver},
		{"screwdriver-wrench", fa::fa_screwdriver_wrench},
		{"tools", fa::fa_tools},
		{"scroll", fa::fa_scroll},
		{"scroll-torah", fa::fa_scroll_torah},
		{"torah", fa::fa_torah},
		{"sd-card", fa::fa_sd_card},
		{"section", fa::fa_section},
		{"seedling", fa::fa_seedling},
		{"sprout", fa::fa_sprout},
		{"server", fa::fa_server},
		{"shapes", fa::fa_shapes},
		{"triangle-circle-square", fa::fa_triangle_circle_square},
		{"share", fa::fa_share},
		{"mail-forward", fa::fa_mail_forward},
		{"share-from-square", fa::fa_share_from_square},
		{"share-square", fa::fa_share_square},
		{"share-nodes", fa::fa_share_nodes},
		{"share-alt", fa::fa_share_alt},
		{"sheet-plastic", fa::fa_sheet_plastic},
		{"shekel-sign", fa::fa_shekel_sign},
		{"ils", fa::fa_ils},
		{"shekel", fa::fa_shekel},
		{"sheqel", fa::fa_sheqel},
		{"sheqel-sign", fa::fa_sheqel_sign},
		{"shield", fa::fa_shield},
		{"shield-blank", fa::fa_shield_blank},
		{"shield-cat", fa::fa_shield_cat},
		{"shield-dog", fa::fa_shield_dog},
		{"shield-halved", fa::fa_shield_halved},
		{"shield-alt", fa::fa_shield_alt},
		{"shield-heart", fa::fa_shield_heart},
		{"shield-virus", fa::fa_shield_virus},
		{"ship", fa::fa_ship},
		{"shirt", fa::fa_shirt},
		{"t-shirt", fa::fa_t_shirt},
		{"tshirt", fa::fa_tshirt},
		{"shoe-prints", fa::fa_shoe_prints},
		{"shop", fa::fa_shop},
		{"store-alt", fa::fa_store_alt},
		{"shop-lock", fa::fa_shop_lock},
		{"shop-slash", fa::fa_shop_slash},
		{"store-alt-slash", fa::fa_store_alt_slash},
		{"shower", fa::fa_shower},
		{"shrimp", fa::fa_shrimp},
		{"shuffle", fa::fa_shuffle},
		{"random", fa::fa_random},
		{"shuttle-space", fa::fa_shuttle_space},
		{"space-shuttle", fa::fa_space_shuttle},
		{"sign-hanging", fa::fa_sign_hanging},
		{"sign", fa::fa_sign},
		{"signal", fa::fa_signal},
		{"signal-5", fa::fa_signal_5},
		{"signal-perfect", fa::fa_signal_perfect},
		{"signature", fa::fa_signature},
		{"signs-post", fa::fa_signs_post},
		{"map-signs", fa::fa_map_signs},
		{"sim-card", fa::fa_sim_card},
		{"sink", fa::fa_sink},
		{"sitemap", fa::fa_sitemap},
		{"skull", fa::fa_skull},
		{"skull-crossbones", fa::fa_skull_crossbones},
		{"slash", fa::fa_slash},
		{"sleigh", fa::fa_sleigh},
		{"sliders", fa::fa_sliders},
		{"sliders-h", fa::fa_sliders_h},
		{"smog", fa::fa_smog},
		{"smoking", fa::fa_smoking},
		{"snowflake", fa::fa_snowflake},
		{"snowman", fa::fa_snowman},
		{"snowplow", fa::fa_snowplow},
		{"soap", fa::fa_soap},
		{"socks", fa::fa_socks},
		{"solar-panel", fa::fa_solar_panel},
		{"sort", fa::fa_sort},
		{"unsorted", fa::fa_unsorted},
		{"sort-down", fa::fa_sort_down},
		{"sort-desc", fa::fa_sort_desc},
		{"sort-up", fa::fa_sort_up},
		{"sort-asc", fa::fa_sort_asc},
		{"spa", fa::fa_spa},
		{"spaghetti-monster-flying", fa::fa_spaghetti_monster_flying},
		{"pastafarianism", fa::fa_pastafarianism},
		{"spell-check", fa::fa_spell_check},
		{"spider", fa::fa_spider},
		{"spinner", fa::fa_spinner},
		{"splotch", fa::fa_splotch},
		{"spoon", fa::fa_spoon},
		{"utensil-spoon", fa::fa_utensil_spoon},
		{"spray-can", fa::fa_spray_can},
		{"spray-can-sparkles", fa::fa_spray_can_sparkles},
		{"air-freshener", fa::fa_air_freshener},
		{"square", fa::fa_square},
		{"square-arrow-up-right", fa::fa_square_arrow_up_right},
		{"external-link-square", fa::fa_external_link_square},
		{"square-caret-down", fa::fa_square_caret_down},
		{"caret-square-down", fa::fa_caret_square_down},
		{"square-caret-left", fa::fa_square_caret_left},
		{"caret-square-left", fa::fa_caret_square_left},
		{"square-caret-right", fa::fa_square_caret_right},
		{"caret-square-right", fa::fa_caret_square_right},
		{"square-caret-up", fa::fa_square_caret_up},
		{"caret-square-up", fa::fa_caret_square_up},
		{"square-check", fa::fa_square_check},
		{"check-square", fa::fa_check_square},
		{"square-envelope", fa::fa_square_envelope},
		{"envelope-square", fa::fa_envelope_square},
		{"square-full", fa::fa_square_full},
		{"square-h", fa::fa_square_h},
		{"h-square", fa::fa_h_square},
		{"square-minus", fa::fa_square_minus},
		{"minus-square", fa::fa_minus_square},
		{"square-nfi", fa::fa_square_nfi},
		{"square-parking", fa::fa_square_parking},
		{"parking", fa::fa_parking},
		{"square-pen", fa::fa_square_pen},
		{"pen-square", fa::fa_pen_square},
		{"pencil-square", fa::fa_pencil_square},
		{"square-person-confined", fa::fa_square_person_confined},
		{"square-phone", fa::fa_square_phone},
		{"phone-square", fa::fa_phone_square},
		{"square-phone-flip", fa::fa_square_phone_flip},
		{"phone-square-alt", fa::fa_phone_square_alt},
		{"square-plus", fa::fa_square_plus},
		{"plus-square", fa::fa_plus_square},
		{"square-poll-horizontal", fa::fa_square_poll_horizontal},
		{"poll-h", fa::fa_poll_h},
		{"square-poll-vertical", fa::fa_square_poll_vertical},
		{"poll", fa::fa_poll},
		{"square-root-variable", fa::fa_square_root_variable},
		{"square-root-alt", fa::fa_square_root_alt},
		{"square-rss", fa::fa_square_rss},
		{"rss-square", fa::fa_rss_square},
		{"square-share-nodes", fa::fa_square_share_nodes},
		{"share-alt-square", fa::fa_share_alt_square},
		{"square-up-right", fa::fa_square_up_right},
		{"external-link-square-alt", fa::fa_external_link_square_alt},
		{"square-virus", fa::fa_square_virus},
		{"square-xmark", fa::fa_square_xmark},
		{"times-square", fa::fa_times_square},
		{"xmark-square", fa::fa_xmark_square},
		{"staff-snake", fa::fa_staff_snake},
		{"rod-asclepius", fa::fa_rod_asclepius},
		{"rod-snake", fa::fa_rod_snake},
		{"staff-aesculapius", fa::fa_staff_aesculapius},
		{"stairs", fa::fa_stairs},
		{"stamp", fa::fa_stamp},
		{"stapler", fa::fa_stapler},
		{"star", fa::fa_star},
		{"star-and-crescent", fa::fa_star_and_crescent},
		{"star-half", fa::fa_star_half},
		{"star-half-stroke", fa::fa_star_half_stroke},
		{"star-half-alt", fa::fa_star_half_alt},
		{"star-of-david", fa::fa_star_of_david},
		{"star-of-life", fa::fa_star_of_life},
		{"sterling-sign", fa::fa_sterling_sign},
		{"gbp", fa::fa_gbp},
		{"pound-sign", fa::fa_pound_sign},
		{"stethoscope", fa::fa_stethoscope},
		{"stop", fa::fa_stop},
		{"stopwatch", fa::fa_stopwatch},
		{"stopwatch-20", fa::fa_stopwatch_20},
		{"store", fa::fa_store},
		{"store-slash", fa::fa_store_slash},
		{"street-view", fa::fa_street_view},
		{"strikethrough", fa::fa_strikethrough},
		{"stroopwafel", fa::fa_stroopwafel},
		{"subscript", fa::fa_subscript},
		{"suitcase", fa::fa_suitcase},
		{"suitcase-medical", fa::fa_suitcase_medical},
		{"medkit", fa::fa_medkit},
		{"suitcase-rolling", fa::fa_suitcase_rolling},
		{"sun", fa::fa_sun},
		{"sun-plant-wilt", fa::fa_sun_plant_wilt},
		{"superscript", fa::fa_superscript},
		{"swatchbook", fa::fa_swatchbook},
		{"synagogue", fa::fa_synagogue},
		{"syringe", fa::fa_syringe},
		{"t", fa::fa_t},
		{"table", fa::fa_table},
		{"table-cells", fa::fa_table_cells},
		{"th", fa::fa_th},
		{"table-cells-large", fa::fa_table_cells_large},
		{"th-large", fa::fa_th_large},
		{"table-columns", fa::fa_table_columns},
		{"columns", fa::fa_columns},
		{"table-list", fa::fa_table_list},
		{"th-list", fa::fa_th_list},
		{"table-tennis-paddle-ball", fa::fa_table_tennis_paddle_ball},
		{"ping-pong-paddle-ball", fa::fa_ping_pong_paddle_ball},
		{"table-tennis", fa::fa_table_tennis},
		{"tablet", fa::fa_tablet},
		{"tablet-android", fa::fa_tablet_android},
		{"tablet-button", fa::fa_tablet_button},
		{"tablet-screen-button", fa::fa_tablet_screen_button},
		{"tablet-alt", fa::fa_tablet_alt},
		{"tablets", fa::fa_tablets},
		{"tachograph-digital", fa::fa_tachograph_digital},
		{"digital-tachograph", fa::fa_digital_tachograph},
		{"tag", fa::fa_tag},
		{"tags", fa::fa_tags},
		{"tape", fa::fa_tape},
		{"tarp", fa::fa_tarp},
		{"tarp-droplet", fa::fa_tarp_droplet},
		{"taxi", fa::fa_taxi},
		{"cab", fa::fa_cab},
		{"teeth", fa::fa_teeth},
		{"teeth-open", fa::fa_teeth_open},
		{"temperature-arrow-down", fa::fa_temperature_arrow_down},
		{"temperature-down", fa::fa_temperature_down},
		{"temperature-arrow-up", fa::fa_temperature_arrow_up},
		{"temperature-up", fa::fa_temperature_up},
		{"temperature-empty", fa::fa_temperature_empty},
		{"temperature-0", fa::fa_temperature_0},
		{"thermometer-0", fa::fa_thermometer_0},
		{"thermometer-empty", fa::fa_thermometer_empty},
		{"temperature-full", fa::fa_temperature_full},
		{"temperature-4", fa::fa_temperature_4},
		{"thermometer-4", fa::fa_thermometer_4},
		{"thermometer-full", fa::fa_thermometer_full},
		{"temperature-half", fa::fa_temperature_half},
		{"temperature-2", fa::fa_temperature_2},
		{"thermometer-2", fa::fa_thermometer_2},
		{"thermometer-half", fa::fa_thermometer_half},
		{"temperature-high", fa::fa_temperature_high},
		{"temperature-low", fa::fa_temperature_low},
		{"temperature-quarter", fa::fa_temperature_quarter},
		{"temperature-1", fa::fa_temperature_1},
		{"thermometer-1", fa::fa_thermometer_1},
		{"thermometer-quarter", fa::fa_thermometer_quarter},
		{"temperature-three-quarters", fa::fa_temperature_three_quarters},
		{"temperature-3", fa::fa_temperature_3},
		{"thermometer-3", fa::fa_thermometer_3},
		{"thermometer-three-quarters", fa::fa_thermometer_three_quarters},
		{"tenge-sign", fa::fa_tenge_sign},
		{"tenge", fa::fa_tenge},
		{"tent", fa::fa_tent},
		{"tent-arrow-down-to-line", fa::fa_tent_arrow_down_to_line},
		{"tent-arrow-left-right", fa::fa_tent_arrow_left_right},
		{"tent-arrow-turn-left", fa::fa_tent_arrow_turn_left},
		{"tent-arrows-down", fa::fa_tent_arrows_down},
		{"tents", fa::fa_tents},
		{"terminal", fa::fa_terminal},
		{"text-height", fa::fa_text_height},
		{"text-slash", fa::fa_text_slash},
		{"remove-format", fa::fa_remove_format},
		{"text-width", fa::fa_text_width},
		{"thermometer", fa::fa_thermometer},
		{"thumbs-down", fa::fa_thumbs_down},
		{"thumbs-up", fa::fa_thumbs_up},
		{"thumbtack", fa::fa_thumbtack},
		{"thumb-tack", fa::fa_thumb_tack},
		{"ticket", fa::fa_ticket},
		{"ticket-simple", fa::fa_ticket_simple},
		{"ticket-alt", fa::fa_ticket_alt},
		{"timeline", fa::fa_timeline},
		{"toggle-off", fa::fa_toggle_off},
		{"toggle-on", fa::fa_toggle_on},
		{"toilet", fa::fa_toilet},
		{"toilet-paper", fa::fa_toilet_paper},
		{"toilet-paper-slash", fa::fa_toilet_paper_slash},
		{"toilet-portable", fa::fa_toilet_portable},
		{"toilets-portable", fa::fa_toilets_portable},
		{"toolbox", fa::fa_toolbox},
		{"tooth", fa::fa_tooth},
		{"torii-gate", fa::fa_torii_gate},
		{"tornado", fa::fa_tornado},
		{"tower-broadcast", fa::fa_tower_broadcast},
		{"broadcast-tower", fa::fa_broadcast_tower},
		{"tower-cell", fa::fa_tower_cell},
		{"tower-observation", fa::fa_tower_observation},
		{"tractor", fa::fa_tractor},
		{"trademark", fa::fa_trademark},
		{"traffic-light", fa::fa_traffic_light},
		{"trailer", fa::fa_trailer},
		{"train", fa::fa_train},
		{"train-subway", fa::fa_train_subway},
		{"subway", fa::fa_subway},
		{"train-tram", fa::fa_train_tram},
		{"transgender", fa::fa_transgender},
		{"transgender-alt", fa::fa_transgender_alt},
		{"trash", fa::fa_trash},
		{"trash-arrow-up", fa::fa_trash_arrow_up},
		{"trash-restore", fa::fa_trash_restore},
		{"trash-can", fa::fa_trash_can},
		{"trash-alt", fa::fa_trash_alt},
		{"trash-can-arrow-up", fa::fa_trash_can_arrow_up},
		{"trash-restore-alt", fa::fa_trash_restore_alt},
		{"tree", fa::fa_tree},
		{"tree-city", fa::fa_tree_city},
		{"triangle-exclamation", fa::fa_triangle_exclamation},
		{"exclamation-triangle", fa::fa_exclamation_triangle},
		{"warning", fa::fa_warning},
		{"trophy", fa::fa_trophy},
		{"trowel", fa::fa_trowel},
		{"trowel-bricks", fa::fa_trowel_bricks},
		{"truck", fa::fa_truck},
		{"truck-arrow-right", fa::fa_truck_arrow_right},
		{"truck-droplet", fa::fa_truck_droplet},
		{"truck-fast", fa::fa_truck_fast},
		{"shipping-fast", fa::fa_shipping_fast},
		{"truck-field", fa::fa_truck_field},
		{"truck-field-un", fa::fa_truck_field_un},
		{"truck-front", fa::fa_truck_front},
		{"truck-medical", fa::fa_truck_medical},
		{"ambulance", fa::fa_ambulance},
		{"truck-monster", fa::fa_truck_monster},
		{"truck-moving", fa::fa_truck_moving},
		{"truck-pickup", fa::fa_truck_pickup},
		{"truck-plane", fa::fa_truck_plane},
		{"truck-ramp-box", fa::fa_truck_ramp_box},
		{"truck-loading", fa::fa_truck_loading},
		{"tty", fa::fa_tty},
		{"teletype", fa::fa_teletype},
		{"turkish-lira-sign", fa::fa_turkish_lira_sign},
		{"try", fa::fa_try},
		{"turkish-lira", fa::fa_turkish_lira},
		{"turn-down", fa::fa_turn_down},
		{"level-down-alt", fa::fa_level_down_alt},
		{"turn-up", fa::fa_turn_up},
		{"level-up-alt", fa::fa_level_up_alt},
		{"tv", fa::fa_tv},
		{"television", fa::fa_television},
		{"tv-alt", fa::fa_tv_alt},
		{"u", fa::fa_u},
		{"umbrella", fa::fa_umbrella},
		{"umbrella-beach", fa::fa_umbrella_beach},
		{"underline", fa::fa_underline},
		{"universal-access", fa::fa_universal_access},
		{"unlock", fa::fa_unlock},
		{"unlock-keyhole", fa::fa_unlock_keyhole},
		{"unlock-alt", fa::fa_unlock_alt},
		{"up-down", fa::fa_up_down},
		{"arrows-alt-v", fa::fa_arrows_alt_v},
		{"up-down-left-right", fa::fa_up_down_left_right},
		{"arrows-alt", fa::fa_arrows_alt},
		{"up-long", fa::fa_up_long},
		{"long-arrow-alt-up", fa::fa_long_arrow_alt_up},
		{"up-right-and-down-left-from-center", fa::fa_up_right_and_down_left_from_center},
		{"expand-alt", fa::fa_expand_alt},
		{"up-right-from-square", fa::fa_up_right_from_square},
		{"external-link-alt", fa::fa_external_link_alt},
		{"upload", fa::fa_upload},
		{"user", fa::fa_user},
		{"user-astronaut", fa::fa_user_astronaut},
		{"user-check", fa::fa_user_check},
		{"user-clock", fa::fa_user_clock},
		{"user-doctor", fa::fa_user_doctor},
		{"user-md", fa::fa_user_md},
		{"user-gear", fa::fa_user_gear},
		{"user-cog", fa::fa_user_cog},
		{"user-graduate", fa::fa_user_graduate},
		{"user-group", fa::fa_user_group},
		{"user-friends", fa::fa_user_friends},
		{"user-injured", fa::fa_user_injured},
		{"user-large", fa::fa_user_large},
		{"user-alt", fa::fa_user_alt},
		{"user-large-slash", fa::fa_user_large_slash},
		{"user-alt-slash", fa::fa_user_alt_slash},
		{"user-lock", fa::fa_user_lock},
		{"user-minus", fa::fa_user_minus},
		{"user-ninja", fa::fa_user_ninja},
		{"user-nurse", fa::fa_user_nurse},
		{"user-pen", fa::fa_user_pen},
		{"user-edit", fa::fa_user_edit},
		{"user-plus", fa::fa_user_plus},
		{"user-secret", fa::fa_user_secret},
		{"user-shield", fa::fa_user_shield},
		{"user-slash", fa::fa_user_slash},
		{"user-tag", fa::fa_user_tag},
		{"user-tie", fa::fa_user_tie},
		{"user-xmark", fa::fa_user_xmark},
		{"user-times", fa::fa_user_times},
		{"users", fa::fa_users},
		{"users-between-lines", fa::fa_users_between_lines},
		{"users-gear", fa::fa_users_gear},
		{"users-cog", fa::fa_users_cog},
		{"users-line", fa::fa_users_line},
		{"users-rays", fa::fa_users_rays},
		{"users-rectangle", fa::fa_users_rectangle},
		{"users-slash", fa::fa_users_slash},
		{"users-viewfinder", fa::fa_users_viewfinder},
		{"utensils", fa::fa_utensils},
		{"cutlery", fa::fa_cutlery},
		{"v", fa::fa_v},
		{"van-shuttle", fa::fa_van_shuttle},
		{"shuttle-van", fa::fa_shuttle_van},
		{"vault", fa::fa_vault},
		{"vector-square", fa::fa_vector_square},
		{"venus", fa::fa_venus},
		{"venus-double", fa::fa_venus_double},
		{"venus-mars", fa::fa_venus_mars},
		{"vest", fa::fa_vest},
		{"vest-patches", fa::fa_vest_patches},
		{"vial", fa::fa_vial},
		{"vial-circle-check", fa::fa_vial_circle_check},
		{"vial-virus", fa::fa_vial_virus},
		{"vials", fa::fa_vials},
		{"video", fa::fa_video},
		{"video-camera", fa::fa_video_camera},
		{"video-slash", fa::fa_video_slash},
		{"vihara", fa::fa_vihara},
		{"virus", fa::fa_virus},
		{"virus-covid", fa::fa_virus_covid},
		{"virus-covid-slash", fa::fa_virus_covid_slash},
		{"virus-slash", fa::fa_virus_slash},
		{"viruses", fa::fa_viruses},
		{"voicemail", fa::fa_voicemail},
		{"volcano", fa::fa_volcano},
		{"volleyball", fa::fa_volleyball},
		{"volleyball-ball", fa::fa_volleyball_ball},
		{"volume-high", fa::fa_volume_high},
		{"volume-up", fa::fa_volume_up},
		{"volume-low", fa::fa_volume_low},
		{"volume-down", fa::fa_volume_down},
		{"volume-off", fa::fa_volume_off},
		{"volume-xmark", fa::fa_volume_xmark},
		{"volume-mute", fa::fa_volume_mute},
		{"volume-times", fa::fa_volume_times},
		{"vr-cardboard", fa::fa_vr_cardboard},
		{"w", fa::fa_w},
		{"walkie-talkie", fa::fa_walkie_talkie},
		{"wallet", fa::fa_wallet},
		{"wand-magic", fa::fa_wand_magic},
		{"magic", fa::fa_magic},
		{"wand-magic-sparkles", fa::fa_wand_magic_sparkles},
		{"magic-wand-sparkles", fa::fa_magic_wand_sparkles},
		{"wand-sparkles", fa::fa_wand_sparkles},
		{"warehouse", fa::fa_warehouse},
		{"water", fa::fa_water},
		{"water-ladder", fa::fa_water_ladder},
		{"ladder-water", fa::fa_ladder_water},
		{"swimming-pool", fa::fa_swimming_pool},
		{"wave-square", fa::fa_wave_square},
		{"weight-hanging", fa::fa_weight_hanging},
		{"weight-scale", fa::fa_weight_scale},
		{"weight", fa::fa_weight},
		{"wheat-awn", fa::fa_wheat_awn},
		{"wheat-alt", fa::fa_wheat_alt},
		{"wheat-awn-circle-exclamation", fa::fa_wheat_awn_circle_exclamation},
		{"wheelchair", fa::fa_wheelchair},
		{"wheelchair-move", fa::fa_wheelchair_move},
		{"wheelchair-alt", fa::fa_wheelchair_alt},
		{"whiskey-glass", fa::fa_whiskey_glass},
		{"glass-whiskey", fa::fa_glass_whiskey},
		{"wifi", fa::fa_wifi},
		{"wifi-3", fa::fa_wifi_3},
		{"wifi-strong", fa::fa_wifi_strong},
		{"wind", fa::fa_wind},
		{"window-maximize", fa::fa_window_maximize},
		{"window-minimize", fa::fa_window_minimize},
		{"window-restore", fa::fa_window_restore},
		{"wine-bottle", fa::fa_wine_bottle},
		{"wine-glass", fa::fa_wine_glass},
		{"wine-glass-empty", fa::fa_wine_glass_empty},
		{"wine-glass-alt", fa::fa_wine_glass_alt},
		{"won-sign", fa::fa_won_sign},
		{"krw", fa::fa_krw},
		{"won", fa::fa_won},
		{"worm", fa::fa_worm},
		{"wrench", fa::fa_wrench},
		{"x", fa::fa_x},
		{"x-ray", fa::fa_x_ray},
		{"xmark", fa::fa_xmark},
		{"close", fa::fa_close},
		{"multiply", fa::fa_multiply},
		{"remove", fa::fa_remove},
		{"times", fa::fa_times},
		{"xmarks-lines", fa::fa_xmarks_lines},
		{"y", fa::fa_y},
		{"yen-sign", fa::fa_yen_sign},
		{"cny", fa::fa_cny},
		{"jpy", fa::fa_jpy},
		{"rmb", fa::fa_rmb},
		{"yen", fa::fa_yen},
		{"yin-yang", fa::fa_yin_yang},
		{"z", fa::fa_z}

};

static const fa::QtAwesomeNamedIcon faBrandsIconArray[] = {
		{"42-group", fa::fa_42_group},
		{"innosoft", fa::fa_innosoft},
		{"500px", fa::fa_500px},
		{"accessible-icon", fa::fa_accessible_icon},
		{"accusoft", fa::fa_accusoft},
		{"adn", fa::fa_adn},
		{"adversal", fa::fa_adversal},
		{"affiliatetheme", fa::fa_affiliatetheme},
		{"airbnb", fa::fa_airbnb},
		{"algolia", fa::fa_algolia},
		{"alipay", fa::fa_alipay},
		{"amazon", fa::fa_amazon},
		{"amazon-pay", fa::fa_amazon_pay},
		{"amilia", fa::fa_amilia},
		{"android", fa::fa_android},
		{"angellist", fa::fa_angellist},
		{"angrycreative", fa::fa_angrycreative},
		{"angular", fa::fa_angular},
		{"app-store", fa::fa_app_store},
		{"app-store-ios", fa::fa_app_store_ios},
		{"apper", fa::fa_apper},
		{"apple", fa::fa_apple},
		{"apple-pay", fa::fa_apple_pay},
		{"artstation", fa::fa_artstation},
		{"asymmetrik", fa::fa_asymmetrik},
		{"atlassian", fa::fa_atlassian},
		{"audible", fa::fa_audible},
		{"autoprefixer", fa::fa_autoprefixer},
		{"avianex", fa::fa_avianex},
		{"aviato", fa::fa_aviato},
		{"aws", fa::fa_aws},
		{"bandcamp", fa::fa_bandcamp},
		{"battle-net", fa::fa_battle_net},
		{"behance", fa::fa_behance},
		{"bilibili", fa::fa_bilibili},
		{"bimobject", fa::fa_bimobject},
		{"bitbucket", fa::fa_bitbucket},
		{"bitcoin", fa::fa_bitcoin},
		{"bity", fa::fa_bity},
		{"black-tie", fa::fa_black_tie},
		{"blackberry", fa::fa_blackberry},
		{"blogger", fa::fa_blogger},
		{"blogger-b", fa::fa_blogger_b},
		{"bluetooth", fa::fa_bluetooth},
		{"bluetooth-b", fa::fa_bluetooth_b},
		{"bootstrap", fa::fa_bootstrap},
		{"bots", fa::fa_bots},
		{"brave", fa::fa_brave},
		{"brave-reverse", fa::fa_brave_reverse},
		{"btc", fa::fa_btc},
		{"buffer", fa::fa_buffer},
		{"buromobelexperte", fa::fa_buromobelexperte},
		{"buy-n-large", fa::fa_buy_n_large},
		{"buysellads", fa::fa_buysellads},
		{"canadian-maple-leaf", fa::fa_canadian_maple_leaf},
		{"cc-amazon-pay", fa::fa_cc_amazon_pay},
		{"cc-amex", fa::fa_cc_amex},
		{"cc-apple-pay", fa::fa_cc_apple_pay},
		{"cc-diners-club", fa::fa_cc_diners_club},
		{"cc-discover", fa::fa_cc_discover},
		{"cc-jcb", fa::fa_cc_jcb},
		{"cc-mastercard", fa::fa_cc_mastercard},
		{"cc-paypal", fa::fa_cc_paypal},
		{"cc-stripe", fa::fa_cc_stripe},
		{"cc-visa", fa::fa_cc_visa},
		{"centercode", fa::fa_centercode},
		{"centos", fa::fa_centos},
		{"chrome", fa::fa_chrome},
		{"chromecast", fa::fa_chromecast},
		{"cloudflare", fa::fa_cloudflare},
		{"cloudscale", fa::fa_cloudscale},
		{"cloudsmith", fa::fa_cloudsmith},
		{"cloudversify", fa::fa_cloudversify},
		{"cmplid", fa::fa_cmplid},
		{"codepen", fa::fa_codepen},
		{"codiepie", fa::fa_codiepie},
		{"confluence", fa::fa_confluence},
		{"connectdevelop", fa::fa_connectdevelop},
		{"contao", fa::fa_contao},
		{"cotton-bureau", fa::fa_cotton_bureau},
		{"cpanel", fa::fa_cpanel},
		{"creative-commons", fa::fa_creative_commons},
		{"creative-commons-by", fa::fa_creative_commons_by},
		{"creative-commons-nc", fa::fa_creative_commons_nc},
		{"creative-commons-nc-eu", fa::fa_creative_commons_nc_eu},
		{"creative-commons-nc-jp", fa::fa_creative_commons_nc_jp},
		{"creative-commons-nd", fa::fa_creative_commons_nd},
		{"creative-commons-pd", fa::fa_creative_commons_pd},
		{"creative-commons-pd-alt", fa::fa_creative_commons_pd_alt},
		{"creative-commons-remix", fa::fa_creative_commons_remix},
		{"creative-commons-sa", fa::fa_creative_commons_sa},
		{"creative-commons-sampling", fa::fa_creative_commons_sampling},
		{"creative-commons-sampling-plus", fa::fa_creative_commons_sampling_plus},
		{"creative-commons-share", fa::fa_creative_commons_share},
		{"creative-commons-zero", fa::fa_creative_commons_zero},
		{"critical-role", fa::fa_critical_role},
		{"css3", fa::fa_css3},
		{"css3-alt", fa::fa_css3_alt},
		{"cuttlefish", fa::fa_cuttlefish},
		{"d-and-d", fa::fa_d_and_d},
		{"d-and-d-beyond", fa::fa_d_and_d_beyond},
		{"dailymotion", fa::fa_dailymotion},
		{"dashcube", fa::fa_dashcube},
		{"debian", fa::fa_debian},
		{"deezer", fa::fa_deezer},
		{"delicious", fa::fa_delicious},
		{"deploydog", fa::fa_deploydog},
		{"deskpro", fa::fa_deskpro},
		{"dev", fa::fa_dev},
		{"deviantart", fa::fa_deviantart},
		{"dhl", fa::fa_dhl},
		{"diaspora", fa::fa_diaspora},
		{"digg", fa::fa_digg},
		{"digital-ocean", fa::fa_digital_ocean},
		{"discord", fa::fa_discord},
		{"discourse", fa::fa_discourse},
		{"dochub", fa::fa_dochub},
		{"docker", fa::fa_docker},
		{"draft2digital", fa::fa_draft2digital},
		{"dribbble", fa::fa_dribbble},
		{"dropbox", fa::fa_dropbox},
		{"drupal", fa::fa_drupal},
		{"dyalog", fa::fa_dyalog},
		{"earlybirds", fa::fa_earlybirds},
		{"ebay", fa::fa_ebay},
		{"edge", fa::fa_edge},
		{"edge-legacy", fa::fa_edge_legacy},
		{"elementor", fa::fa_elementor},
		{"ello", fa::fa_ello},
		{"ember", fa::fa_ember},
		{"empire", fa::fa_empire},
		{"envira", fa::fa_envira},
		{"erlang", fa::fa_erlang},
		{"ethereum", fa::fa_ethereum},
		{"etsy", fa::fa_etsy},
		{"evernote", fa::fa_evernote},
		{"expeditedssl", fa::fa_expeditedssl},
		{"facebook", fa::fa_facebook},
		{"facebook-f", fa::fa_facebook_f},
		{"facebook-messenger", fa::fa_facebook_messenger},
		{"fantasy-flight-games", fa::fa_fantasy_flight_games},
		{"fedex", fa::fa_fedex},
		{"fedora", fa::fa_fedora},
		{"figma", fa::fa_figma},
		{"firefox", fa::fa_firefox},
		{"firefox-browser", fa::fa_firefox_browser},
		{"first-order", fa::fa_first_order},
		{"first-order-alt", fa::fa_first_order_alt},
		{"firstdraft", fa::fa_firstdraft},
		{"flickr", fa::fa_flickr},
		{"flipboard", fa::fa_flipboard},
		{"fly", fa::fa_fly},
		{"font-awesome", fa::fa_font_awesome},
		{"font-awesome-flag", fa::fa_font_awesome_flag},
		{"font-awesome-logo-full", fa::fa_font_awesome_logo_full},
		{"fonticons", fa::fa_fonticons},
		{"fonticons-fi", fa::fa_fonticons_fi},
		{"fort-awesome", fa::fa_fort_awesome},
		{"fort-awesome-alt", fa::fa_fort_awesome_alt},
		{"forumbee", fa::fa_forumbee},
		{"foursquare", fa::fa_foursquare},
		{"free-code-camp", fa::fa_free_code_camp},
		{"freebsd", fa::fa_freebsd},
		{"fulcrum", fa::fa_fulcrum},
		{"galactic-republic", fa::fa_galactic_republic},
		{"galactic-senate", fa::fa_galactic_senate},
		{"get-pocket", fa::fa_get_pocket},
		{"gg", fa::fa_gg},
		{"gg-circle", fa::fa_gg_circle},
		{"git", fa::fa_git},
		{"git-alt", fa::fa_git_alt},
		{"github", fa::fa_github},
		{"github-alt", fa::fa_github_alt},
		{"gitkraken", fa::fa_gitkraken},
		{"gitlab", fa::fa_gitlab},
		{"gitter", fa::fa_gitter},
		{"glide", fa::fa_glide},
		{"glide-g", fa::fa_glide_g},
		{"gofore", fa::fa_gofore},
		{"golang", fa::fa_golang},
		{"goodreads", fa::fa_goodreads},
		{"goodreads-g", fa::fa_goodreads_g},
		{"google", fa::fa_google},
		{"google-drive", fa::fa_google_drive},
		{"google-pay", fa::fa_google_pay},
		{"google-play", fa::fa_google_play},
		{"google-plus", fa::fa_google_plus},
		{"google-plus-g", fa::fa_google_plus_g},
		{"google-scholar", fa::fa_google_scholar},
		{"google-wallet", fa::fa_google_wallet},
		{"gratipay", fa::fa_gratipay},
		{"grav", fa::fa_grav},
		{"gripfire", fa::fa_gripfire},
		{"grunt", fa::fa_grunt},
		{"guilded", fa::fa_guilded},
		{"gulp", fa::fa_gulp},
		{"hacker-news", fa::fa_hacker_news},
		{"hackerrank", fa::fa_hackerrank},
		{"hashnode", fa::fa_hashnode},
		{"hips", fa::fa_hips},
		{"hire-a-helper", fa::fa_hire_a_helper},
		{"hive", fa::fa_hive},
		{"hooli", fa::fa_hooli},
		{"hornbill", fa::fa_hornbill},
		{"hotjar", fa::fa_hotjar},
		{"houzz", fa::fa_houzz},
		{"html5", fa::fa_html5},
		{"hubspot", fa::fa_hubspot},
		{"ideal", fa::fa_ideal},
		{"imdb", fa::fa_imdb},
		{"instagram", fa::fa_instagram},
		{"instalod", fa::fa_instalod},
		{"intercom", fa::fa_intercom},
		{"internet-explorer", fa::fa_internet_explorer},
		{"invision", fa::fa_invision},
		{"ioxhost", fa::fa_ioxhost},
		{"itch-io", fa::fa_itch_io},
		{"itunes", fa::fa_itunes},
		{"itunes-note", fa::fa_itunes_note},
		{"java", fa::fa_java},
		{"jedi-order", fa::fa_jedi_order},
		{"jenkins", fa::fa_jenkins},
		{"jira", fa::fa_jira},
		{"joget", fa::fa_joget},
		{"joomla", fa::fa_joomla},
		{"js", fa::fa_js},
		{"jsfiddle", fa::fa_jsfiddle},
		{"kaggle", fa::fa_kaggle},
		{"keybase", fa::fa_keybase},
		{"keycdn", fa::fa_keycdn},
		{"kickstarter", fa::fa_kickstarter},
		{"kickstarter-k", fa::fa_kickstarter_k},
		{"korvue", fa::fa_korvue},
		{"laravel", fa::fa_laravel},
		{"lastfm", fa::fa_lastfm},
		{"leanpub", fa::fa_leanpub},
		{"less", fa::fa_less},
		{"letterboxd", fa::fa_letterboxd},
		{"line", fa::fa_line},
		{"linkedin", fa::fa_linkedin},
		{"linkedin-in", fa::fa_linkedin_in},
		{"linode", fa::fa_linode},
		{"linux", fa::fa_linux},
		{"lyft", fa::fa_lyft},
		{"magento", fa::fa_magento},
		{"mailchimp", fa::fa_mailchimp},
		{"mandalorian", fa::fa_mandalorian},
		{"markdown", fa::fa_markdown},
		{"mastodon", fa::fa_mastodon},
		{"maxcdn", fa::fa_maxcdn},
		{"mdb", fa::fa_mdb},
		{"medapps", fa::fa_medapps},
		{"medium", fa::fa_medium},
		{"medium-m", fa::fa_medium_m},
		{"medrt", fa::fa_medrt},
		{"meetup", fa::fa_meetup},
		{"megaport", fa::fa_megaport},
		{"mendeley", fa::fa_mendeley},
		{"meta", fa::fa_meta},
		{"microblog", fa::fa_microblog},
		{"microsoft", fa::fa_microsoft},
		{"mintbit", fa::fa_mintbit},
		{"mix", fa::fa_mix},
		{"mixcloud", fa::fa_mixcloud},
		{"mixer", fa::fa_mixer},
		{"mizuni", fa::fa_mizuni},
		{"modx", fa::fa_modx},
		{"monero", fa::fa_monero},
		{"napster", fa::fa_napster},
		{"neos", fa::fa_neos},
		{"nfc-directional", fa::fa_nfc_directional},
		{"nfc-symbol", fa::fa_nfc_symbol},
		{"nimblr", fa::fa_nimblr},
		{"node", fa::fa_node},
		{"node-js", fa::fa_node_js},
		{"npm", fa::fa_npm},
		{"ns8", fa::fa_ns8},
		{"nutritionix", fa::fa_nutritionix},
		{"octopus-deploy", fa::fa_octopus_deploy},
		{"odnoklassniki", fa::fa_odnoklassniki},
		{"odysee", fa::fa_odysee},
		{"old-republic", fa::fa_old_republic},
		{"opencart", fa::fa_opencart},
		{"openid", fa::fa_openid},
		{"opensuse", fa::fa_opensuse},
		{"opera", fa::fa_opera},
		{"optin-monster", fa::fa_optin_monster},
		{"orcid", fa::fa_orcid},
		{"osi", fa::fa_osi},
		{"padlet", fa::fa_padlet},
		{"page4", fa::fa_page4},
		{"pagelines", fa::fa_pagelines},
		{"palfed", fa::fa_palfed},
		{"patreon", fa::fa_patreon},
		{"paypal", fa::fa_paypal},
		{"perbyte", fa::fa_perbyte},
		{"periscope", fa::fa_periscope},
		{"phabricator", fa::fa_phabricator},
		{"phoenix-framework", fa::fa_phoenix_framework},
		{"phoenix-squadron", fa::fa_phoenix_squadron},
		{"php", fa::fa_php},
		{"pied-piper", fa::fa_pied_piper},
		{"pied-piper-alt", fa::fa_pied_piper_alt},
		{"pied-piper-hat", fa::fa_pied_piper_hat},
		{"pied-piper-pp", fa::fa_pied_piper_pp},
		{"pinterest", fa::fa_pinterest},
		{"pinterest-p", fa::fa_pinterest_p},
		{"pix", fa::fa_pix},
		{"pixiv", fa::fa_pixiv},
		{"playstation", fa::fa_playstation},
		{"product-hunt", fa::fa_product_hunt},
		{"pushed", fa::fa_pushed},
		{"python", fa::fa_python},
		{"qq", fa::fa_qq},
		{"quinscape", fa::fa_quinscape},
		{"quora", fa::fa_quora},
		{"r-project", fa::fa_r_project},
		{"raspberry-pi", fa::fa_raspberry_pi},
		{"ravelry", fa::fa_ravelry},
		{"react", fa::fa_react},
		{"reacteurope", fa::fa_reacteurope},
		{"readme", fa::fa_readme},
		{"rebel", fa::fa_rebel},
		{"red-river", fa::fa_red_river},
		{"reddit", fa::fa_reddit},
		{"reddit-alien", fa::fa_reddit_alien},
		{"redhat", fa::fa_redhat},
		{"renren", fa::fa_renren},
		{"replyd", fa::fa_replyd},
		{"researchgate", fa::fa_researchgate},
		{"resolving", fa::fa_resolving},
		{"rev", fa::fa_rev},
		{"rocketchat", fa::fa_rocketchat},
		{"rockrms", fa::fa_rockrms},
		{"rust", fa::fa_rust},
		{"safari", fa::fa_safari},
		{"salesforce", fa::fa_salesforce},
		{"sass", fa::fa_sass},
		{"schlix", fa::fa_schlix},
		{"screenpal", fa::fa_screenpal},
		{"scribd", fa::fa_scribd},
		{"searchengin", fa::fa_searchengin},
		{"sellcast", fa::fa_sellcast},
		{"sellsy", fa::fa_sellsy},
		{"servicestack", fa::fa_servicestack},
		{"shirtsinbulk", fa::fa_shirtsinbulk},
		{"shoelace", fa::fa_shoelace},
		{"shopify", fa::fa_shopify},
		{"shopware", fa::fa_shopware},
		{"signal-messenger", fa::fa_signal_messenger},
		{"simplybuilt", fa::fa_simplybuilt},
		{"sistrix", fa::fa_sistrix},
		{"sith", fa::fa_sith},
		{"sitrox", fa::fa_sitrox},
		{"sketch", fa::fa_sketch},
		{"skyatlas", fa::fa_skyatlas},
		{"skype", fa::fa_skype},
		{"slack", fa::fa_slack},
		{"slack-hash", fa::fa_slack_hash},
		{"slideshare", fa::fa_slideshare},
		{"snapchat", fa::fa_snapchat},
		{"snapchat-ghost", fa::fa_snapchat_ghost},
		{"soundcloud", fa::fa_soundcloud},
		{"sourcetree", fa::fa_sourcetree},
		{"space-awesome", fa::fa_space_awesome},
		{"speakap", fa::fa_speakap},
		{"speaker-deck", fa::fa_speaker_deck},
		{"spotify", fa::fa_spotify},
		{"square-behance", fa::fa_square_behance},
		{"behance-square", fa::fa_behance_square},
		{"square-dribbble", fa::fa_square_dribbble},
		{"dribbble-square", fa::fa_dribbble_square},
		{"square-facebook", fa::fa_square_facebook},
		{"facebook-square", fa::fa_facebook_square},
		{"square-font-awesome", fa::fa_square_font_awesome},
		{"square-font-awesome-stroke", fa::fa_square_font_awesome_stroke},
		{"font-awesome-alt", fa::fa_font_awesome_alt},
		{"square-git", fa::fa_square_git},
		{"git-square", fa::fa_git_square},
		{"square-github", fa::fa_square_github},
		{"github-square", fa::fa_github_square},
		{"square-gitlab", fa::fa_square_gitlab},
		{"gitlab-square", fa::fa_gitlab_square},
		{"square-google-plus", fa::fa_square_google_plus},
		{"google-plus-square", fa::fa_google_plus_square},
		{"square-hacker-news", fa::fa_square_hacker_news},
		{"hacker-news-square", fa::fa_hacker_news_square},
		{"square-instagram", fa::fa_square_instagram},
		{"instagram-square", fa::fa_instagram_square},
		{"square-js", fa::fa_square_js},
		{"js-square", fa::fa_js_square},
		{"square-lastfm", fa::fa_square_lastfm},
		{"lastfm-square", fa::fa_lastfm_square},
		{"square-letterboxd", fa::fa_square_letterboxd},
		{"square-odnoklassniki", fa::fa_square_odnoklassniki},
		{"odnoklassniki-square", fa::fa_odnoklassniki_square},
		{"square-pied-piper", fa::fa_square_pied_piper},
		{"pied-piper-square", fa::fa_pied_piper_square},
		{"square-pinterest", fa::fa_square_pinterest},
		{"pinterest-square", fa::fa_pinterest_square},
		{"square-reddit", fa::fa_square_reddit},
		{"reddit-square", fa::fa_reddit_square},
		{"square-snapchat", fa::fa_square_snapchat},
		{"snapchat-square", fa::fa_snapchat_square},
		{"square-steam", fa::fa_square_steam},
		{"steam-square", fa::fa_steam_square},
		{"square-threads", fa::fa_square_threads},
		{"square-tumblr", fa::fa_square_tumblr},
		{"tumblr-square", fa::fa_tumblr_square},
		{"square-twitter", fa::fa_square_twitter},
		{"twitter-square", fa::fa_twitter_square},
		{"square-viadeo", fa::fa_square_viadeo},
		{"viadeo-square", fa::fa_viadeo_square},
		{"square-vimeo", fa::fa_square_vimeo},
		{"vimeo-square", fa::fa_vimeo_square},
		{"square-whatsapp", fa::fa_square_whatsapp},
		{"whatsapp-square", fa::fa_whatsapp_square},
		{"square-x-twitter", fa::fa_square_x_twitter},
		{"square-xing", fa::fa_square_xing},
		{"xing-square", fa::fa_xing_square},
		{"square-youtube", fa::fa_square_youtube},
		{"youtube-square", fa::fa_youtube_square},
		{"squarespace", fa::fa_squarespace},
		{"stack-exchange", fa::fa_stack_exchange},
		{"stack-overflow", fa::fa_stack_overflow},
		{"stackpath", fa::fa_stackpath},
		{"staylinked", fa::fa_staylinked},
		{"steam", fa::fa_steam},
		{"steam-symbol", fa::fa_steam_symbol},
		{"sticker-mule", fa::fa_sticker_mule},
		{"strava", fa::fa_strava},
		{"stripe", fa::fa_stripe},
		{"stripe-s", fa::fa_stripe_s},
		{"stubber", fa::fa_stubber},
		{"studiovinari", fa::fa_studiovinari},
		{"stumbleupon", fa::fa_stumbleupon},
		{"stumbleupon-circle", fa::fa_stumbleupon_circle},
		{"superpowers", fa::fa_superpowers},
		{"supple", fa::fa_supple},
		{"suse", fa::fa_suse},
		{"swift", fa::fa_swift},
		{"symfony", fa::fa_symfony},
		{"teamspeak", fa::fa_teamspeak},
		{"telegram", fa::fa_telegram},
		{"telegram-plane", fa::fa_telegram_plane},
		{"tencent-weibo", fa::fa_tencent_weibo},
		{"the-red-yeti", fa::fa_the_red_yeti},
		{"themeco", fa::fa_themeco},
		{"themeisle", fa::fa_themeisle},
		{"think-peaks", fa::fa_think_peaks},
		{"threads", fa::fa_threads},
		{"tiktok", fa::fa_tiktok},
		{"trade-federation", fa::fa_trade_federation},
		{"trello", fa::fa_trello},
		{"tumblr", fa::fa_tumblr},
		{"twitch", fa::fa_twitch},
		{"twitter", fa::fa_twitter},
		{"typo3", fa::fa_typo3},
		{"uber", fa::fa_uber},
		{"ubuntu", fa::fa_ubuntu},
		{"uikit", fa::fa_uikit},
		{"umbraco", fa::fa_umbraco},
		{"uncharted", fa::fa_uncharted},
		{"uniregistry", fa::fa_uniregistry},
		{"unity", fa::fa_unity},
		{"unsplash", fa::fa_unsplash},
		{"untappd", fa::fa_untappd},
		{"ups", fa::fa_ups},
		{"upwork", fa::fa_upwork},
		{"usb", fa::fa_usb},
		{"usps", fa::fa_usps},
		{"ussunnah", fa::fa_ussunnah},
		{"vaadin", fa::fa_vaadin},
		{"viacoin", fa::fa_viacoin},
		{"viadeo", fa::fa_viadeo},
		{"viber", fa::fa_viber},
		{"vimeo", fa::fa_vimeo},
		{"vimeo-v", fa::fa_vimeo_v},
		{"vine", fa::fa_vine},
		{"vk", fa::fa_vk},
		{"vnv", fa::fa_vnv},
		{"vuejs", fa::fa_vuejs},
		{"watchman-monitoring", fa::fa_watchman_monitoring},
		{"waze", fa::fa_waze},
		{"webflow", fa::fa_webflow},
		{"weebly", fa::fa_weebly},
		{"weibo", fa::fa_weibo},
		{"weixin", fa::fa_weixin},
		{"whatsapp", fa::fa_whatsapp},
		{"whmcs", fa::fa_whmcs},
		{"wikipedia-w", fa::fa_wikipedia_w},
		{"windows", fa::fa_windows},
		{"wirsindhandwerk", fa::fa_wirsindhandwerk},
		{"wsh", fa::fa_wsh},
		{"wix", fa::fa_wix},
		{"wizards-of-the-coast", fa::fa_wizards_of_the_coast},
		{"wodu", fa::fa_wodu},
		{"wolf-pack-battalion", fa::fa_wolf_pack_battalion},
		{"wordpress", fa::fa_wordpress},
		{"wordpress-simple", fa::fa_wordpress_simple},
		{"wpbeginner", fa::fa_wpbeginner},
		{"wpexplorer", fa::fa_wpexplorer},
		{"wpforms", fa::fa_wpforms},
		{"wpressr", fa::fa_wpressr},
		{"rendact", fa::fa_rendact},
		{"x-twitter", fa::fa_x_twitter},
		{"xbox", fa::fa_xbox},
		{"xing", fa::fa_xing},
		{"y-combinator", fa::fa_y_combinator},
		{"yahoo", fa::fa_yahoo},
		{"yammer", fa::fa_yammer},
		{"yandex", fa::fa_yandex},
		{"yandex-international", fa::fa_yandex_international},
		{"yarn", fa::fa_yarn},
		{"yelp", fa::fa_yelp},
		{"yoast", fa::fa_yoast},
		{"youtube", fa::fa_youtube},
		{"zhihu", fa::fa_zhihu}

};

static const fa::QtAwesomeNamedIcon faRegularFreeIconArray[] = {
		{"address-book", fa::fa_address_book},
		{"contact-book", fa::fa_contact_book},
		{"address-card", fa::fa_address_card},
		{"contact-card", fa::fa_contact_card},
		{"vcard", fa::fa_vcard},
		{"bell", fa::fa_bell},
		{"bell-slash", fa::fa_bell_slash},
		{"bookmark", fa::fa_bookmark},
		{"building", fa::fa_building},
		{"calendar", fa::fa_calendar},
		{"calendar-check", fa::fa_calendar_check},
		{"calendar-days", fa::fa_calendar_days},
		{"calendar-alt", fa::fa_calendar_alt},
		{"calendar-minus", fa::fa_calendar_minus},
		{"calendar-plus", fa::fa_calendar_plus},
		{"calendar-xmark", fa::fa_calendar_xmark},
		{"calendar-times", fa::fa_calendar_times},
		{"chart-bar", fa::fa_chart_bar},
		{"bar-chart", fa::fa_bar_chart},
		{"chess-bishop", fa::fa_chess_bishop},
		{"chess-king", fa::fa_chess_king},
		{"chess-knight", fa::fa_chess_knight},
		{"chess-pawn", fa::fa_chess_pawn},
		{"chess-queen", fa::fa_chess_queen},
		{"chess-rook", fa::fa_chess_rook},
		{"circle", fa::fa_circle},
		{"circle-check", fa::fa_circle_check},
		{"check-circle", fa::fa_check_circle},
		{"circle-dot", fa::fa_circle_dot},
		{"dot-circle", fa::fa_dot_circle},
		{"circle-down", fa::fa_circle_down},
		{"arrow-alt-circle-down", fa::fa_arrow_alt_circle_down},
		{"circle-left", fa::fa_circle_left},
		{"arrow-alt-circle-left", fa::fa_arrow_alt_circle_left},
		{"circle-pause", fa::fa_circle_pause},
		{"pause-circle", fa::fa_pause_circle},
		{"circle-play", fa::fa_circle_play},
		{"play-circle", fa::fa_play_circle},
		{"circle-question", fa::fa_circle_question},
		{"question-circle", fa::fa_question_circle},
		{"circle-right", fa::fa_circle_right},
		{"arrow-alt-circle-right", fa::fa_arrow_alt_circle_right},
		{"circle-stop", fa::fa_circle_stop},
		{"stop-circle", fa::fa_stop_circle},
		{"circle-up", fa::fa_circle_up},
		{"arrow-alt-circle-up", fa::fa_arrow_alt_circle_up},
		{"circle-user", fa::fa_circle_user},
		{"user-circle", fa::fa_user_circle},
		{"circle-xmark", fa::fa_circle_xmark},
		{"times-circle", fa::fa_times_circle},
		{"xmark-circle", fa::fa_xmark_circle},
		{"clipboard", fa::fa_clipboard},
		{"clock", fa::fa_clock},
		{"clock-four", fa::fa_clock_four},
		{"clone", fa::fa_clone},
		{"closed-captioning", fa::fa_closed_captioning},
		{"comment", fa::fa_comment},
		{"comment-dots", fa::fa_comment_dots},
		{"commenting", fa::fa_commenting},
		{"comments", fa::fa_comments},
		{"compass", fa::fa_compass},
		{"copy", fa::fa_copy},
		{"copyright", fa::fa_copyright},
		{"credit-card", fa::fa_credit_card},
		{"credit-card-alt", fa::fa_credit_card_alt},
		{"envelope", fa::fa_envelope},
		{"envelope-open", fa::fa_envelope_open},
		{"eye", fa::fa_eye},
		{"eye-slash", fa::fa_eye_slash},
		{"face-angry", fa::fa_face_angry},
		{"angry", fa::fa_angry},
		{"face-dizzy", fa::fa_face_dizzy},
		{"dizzy", fa::fa_dizzy},
		{"face-flushed", fa::fa_face_flushed},
		{"flushed", fa::fa_flushed},
		{"face-frown", fa::fa_face_frown},
		{"frown", fa::fa_frown},
		{"face-frown-open", fa::fa_face_frown_open},
		{"frown-open", fa::fa_frown_open},
		{"face-grimace", fa::fa_face_grimace},
		{"grimace", fa::fa_grimace},
		{"face-grin", fa::fa_face_grin},
		{"grin", fa::fa_grin},
		{"face-grin-beam", fa::fa_face_grin_beam},
		{"grin-beam", fa::fa_grin_beam},
		{"face-grin-beam-sweat", fa::fa_face_grin_beam_sweat},
		{"grin-beam-sweat", fa::fa_grin_beam_sweat},
		{"face-grin-hearts", fa::fa_face_grin_hearts},
		{"grin-hearts", fa::fa_grin_hearts},
		{"face-grin-squint", fa::fa_face_grin_squint},
		{"grin-squint", fa::fa_grin_squint},
		{"face-grin-squint-tears", fa::fa_face_grin_squint_tears},
		{"grin-squint-tears", fa::fa_grin_squint_tears},
		{"face-grin-stars", fa::fa_face_grin_stars},
		{"grin-stars", fa::fa_grin_stars},
		{"face-grin-tears", fa::fa_face_grin_tears},
		{"grin-tears", fa::fa_grin_tears},
		{"face-grin-tongue", fa::fa_face_grin_tongue},
		{"grin-tongue", fa::fa_grin_tongue},
		{"face-grin-tongue-squint", fa::fa_face_grin_tongue_squint},
		{"grin-tongue-squint", fa::fa_grin_tongue_squint},
		{"face-grin-tongue-wink", fa::fa_face_grin_tongue_wink},
		{"grin-tongue-wink", fa::fa_grin_tongue_wink},
		{"face-grin-wide", fa::fa_face_grin_wide},
		{"grin-alt", fa::fa_grin_alt},
		{"face-grin-wink", fa::fa_face_grin_wink},
		{"grin-wink", fa::fa_grin_wink},
		{"face-kiss", fa::fa_face_kiss},
		{"kiss", fa::fa_kiss},
		{"face-kiss-beam", fa::fa_face_kiss_beam},
		{"kiss-beam", fa::fa_kiss_beam},
		{"face-kiss-wink-heart", fa::fa_face_kiss_wink_heart},
		{"kiss-wink-heart", fa::fa_kiss_wink_heart},
		{"face-laugh", fa::fa_face_laugh},
		{"laugh", fa::fa_laugh},
		{"face-laugh-beam", fa::fa_face_laugh_beam},
		{"laugh-beam", fa::fa_laugh_beam},
		{"face-laugh-squint", fa::fa_face_laugh_squint},
		{"laugh-squint", fa::fa_laugh_squint},
		{"face-laugh-wink", fa::fa_face_laugh_wink},
		{"laugh-wink", fa::fa_laugh_wink},
		{"face-meh", fa::fa_face_meh},
		{"meh", fa::fa_meh},
		{"face-meh-blank", fa::fa_face_meh_blank},
		{"meh-blank", fa::fa_meh_blank},
		{"face-rolling-eyes", fa::fa_face_rolling_eyes},
		{"meh-rolling-eyes", fa::fa_meh_rolling_eyes},
		{"face-sad-cry", fa::fa_face_sad_cry},
		{"sad-cry", fa::fa_sad_cry},
		{"face-sad-tear", fa::fa_face_sad_tear},
		{"sad-tear", fa::fa_sad_tear},
		{"face-smile", fa::fa_face_smile},
		{"smile", fa::fa_smile},
		{"face-smile-beam", fa::fa_face_smile_beam},
		{"smile-beam", fa::fa_smile_beam},
		{"face-smile-wink", fa::fa_face_smile_wink},
		{"smile-wink", fa::fa_smile_wink},
		{"face-surprise", fa::fa_face_surprise},
		{"surprise", fa::fa_surprise},
		{"face-tired", fa::fa_face_tired},
		{"tired", fa::fa_tired},
		{"file", fa::fa_file},
		{"file-audio", fa::fa_file_audio},
		{"file-code", fa::fa_file_code},
		{"file-excel", fa::fa_file_excel},
		{"file-image", fa::fa_file_image},
		{"file-lines", fa::fa_file_lines},
		{"file-alt", fa::fa_file_alt},
		{"file-text", fa::fa_file_text},
		{"file-pdf", fa::fa_file_pdf},
		{"file-powerpoint", fa::fa_file_powerpoint},
		{"file-video", fa::fa_file_video},
		{"file-word", fa::fa_file_word},
		{"file-zipper", fa::fa_file_zipper},
		{"file-archive", fa::fa_file_archive},
		{"flag", fa::fa_flag},
		{"floppy-disk", fa::fa_floppy_disk},
		{"save", fa::fa_save},
		{"folder", fa::fa_folder},
		{"folder-blank", fa::fa_folder_blank},
		{"folder-closed", fa::fa_folder_closed},
		{"folder-open", fa::fa_folder_open},
		{"futbol", fa::fa_futbol},
		{"futbol-ball", fa::fa_futbol_ball},
		{"soccer-ball", fa::fa_soccer_ball},
		{"gem", fa::fa_gem},
		{"hand", fa::fa_hand},
		{"hand-paper", fa::fa_hand_paper},
		{"hand-back-fist", fa::fa_hand_back_fist},
		{"hand-rock", fa::fa_hand_rock},
		{"hand-lizard", fa::fa_hand_lizard},
		{"hand-peace", fa::fa_hand_peace},
		{"hand-point-down", fa::fa_hand_point_down},
		{"hand-point-left", fa::fa_hand_point_left},
		{"hand-point-right", fa::fa_hand_point_right},
		{"hand-point-up", fa::fa_hand_point_up},
		{"hand-pointer", fa::fa_hand_pointer},
		{"hand-scissors", fa::fa_hand_scissors},
		{"hand-spock", fa::fa_hand_spock},
		{"handshake", fa::fa_handshake},
		{"hard-drive", fa::fa_hard_drive},
		{"hdd", fa::fa_hdd},
		{"heart", fa::fa_heart},
		{"hospital", fa::fa_hospital},
		{"hospital-alt", fa::fa_hospital_alt},
		{"hospital-wide", fa::fa_hospital_wide},
		{"hourglass", fa::fa_hourglass},
		{"hourglass-empty", fa::fa_hourglass_empty},
		{"hourglass-half", fa::fa_hourglass_half},
		{"hourglass-2", fa::fa_hourglass_2},
		{"id-badge", fa::fa_id_badge},
		{"id-card", fa::fa_id_card},
		{"drivers-license", fa::fa_drivers_license},
		{"image", fa::fa_image},
		{"images", fa::fa_images},
		{"keyboard", fa::fa_keyboard},
		{"lemon", fa::fa_lemon},
		{"life-ring", fa::fa_life_ring},
		{"lightbulb", fa::fa_lightbulb},
		{"map", fa::fa_map},
		{"message", fa::fa_message},
		{"comment-alt", fa::fa_comment_alt},
		{"money-bill-1", fa::fa_money_bill_1},
		{"money-bill-alt", fa::fa_money_bill_alt},
		{"moon", fa::fa_moon},
		{"newspaper", fa::fa_newspaper},
		{"note-sticky", fa::fa_note_sticky},
		{"sticky-note", fa::fa_sticky_note},
		{"object-group", fa::fa_object_group},
		{"object-ungroup", fa::fa_object_ungroup},
		{"paper-plane", fa::fa_paper_plane},
		{"paste", fa::fa_paste},
		{"file-clipboard", fa::fa_file_clipboard},
		{"pen-to-square", fa::fa_pen_to_square},
		{"edit", fa::fa_edit},
		{"rectangle-list", fa::fa_rectangle_list},
		{"list-alt", fa::fa_list_alt},
		{"rectangle-xmark", fa::fa_rectangle_xmark},
		{"rectangle-times", fa::fa_rectangle_times},
		{"times-rectangle", fa::fa_times_rectangle},
		{"window-close", fa::fa_window_close},
		{"registered", fa::fa_registered},
		{"share-from-square", fa::fa_share_from_square},
		{"share-square", fa::fa_share_square},
		{"snowflake", fa::fa_snowflake},
		{"square", fa::fa_square},
		{"square-caret-down", fa::fa_square_caret_down},
		{"caret-square-down", fa::fa_caret_square_down},
		{"square-caret-left", fa::fa_square_caret_left},
		{"caret-square-left", fa::fa_caret_square_left},
		{"square-caret-right", fa::fa_square_caret_right},
		{"caret-square-right", fa::fa_caret_square_right},
		{"square-caret-up", fa::fa_square_caret_up},
		{"caret-square-up", fa::fa_caret_square_up},
		{"square-check", fa::fa_square_check},
		{"check-square", fa::fa_check_square},
		{"square-full", fa::fa_square_full},
		{"square-minus", fa::fa_square_minus},
		{"minus-square", fa::fa_minus_square},
		{"square-plus", fa::fa_square_plus},
		{"plus-square", fa::fa_plus_square},
		{"star", fa::fa_star},
		{"star-half", fa::fa_star_half},
		{"star-half-stroke", fa::fa_star_half_stroke},
		{"star-half-alt", fa::fa_star_half_alt},
		{"sun", fa::fa_sun},
		{"thumbs-down", fa::fa_thumbs_down},
		{"thumbs-up", fa::fa_thumbs_up},
		{"trash-can", fa::fa_trash_can},
		{"trash-alt", fa::fa_trash_alt},
		{"user", fa::fa_user},
		{"window-maximize", fa::fa_window_maximize},
		{"window-minimize", fa::fa_window_minimize},
		{"window-restore", fa::fa_window_restore},

};
